C     PROGRAM RCA
C 
C 
C*******************************************************************************
C 
C 
C                  RCA  -  ROW/COLUMN VERSION OF AESOP 
C
C                                                      (RELEASE 2.0)
C 
C                                     DEVELOPED BY: HYDROQUAL INC.
C                                                   MAHWAH, N.J.
C                                     PROGRAMMER: JAMES FITZPATRICK
C 
C*******************************************************************************
C 
C 
C        THE FOLLOWING VARIABLES MUST BE DEFINED FOR EACH SITE SPECIFIC
C        WATER QUALITY MODEL VIA THE INCLUDE BLOCK -RCACM-
C
C        NROW     -  NUMBER OF ROWS IN THE MODEL DOMAIN
C        NCOL     -  NUMBER OF COLUMNS IN THE MODEL DOMAIN
C        NLAYR    -  NUMBER OF DEPTH LAYERS IN THE MODEL DOMAIN
C        NOSYS    -  NUMBER OF SYSTEMS OR WATER QUALITY VARIABLES
C                    IN THE MODEL
C        NPARM2D  -  MAXIMUM NUMBER OF DEPTH-INDEPENDENT PARAMETERS
C        NPARM3D  -  MAXIMUM NUMBER OF DEPTH-DEPENDENT PARAMETERS
C        NBC      -  MAXIMUM NUMBER OF BOUNDARY CONDITIONS PER SYSTEM
C        NWK      -  MAXIMUM NUMBER OF FORCING FUNCTIONS OR LOADS
C                    PER SYSTEM
C
C
C        DESCRIPTION OF COMMON VARIABLES
C 
C        IN       -  INPUT DEVICE NUMBER 
C        OUT      -  OUTPUT DEVICE NUMBER
C        MXACTS   -  NUMBER OF THE MAXIMUM (OR LAST) ACTIVE SYSTEM
C        ISYS     -  SYSTEM CURRENTLY HAVING ITS DERIVATIVES CALCULATED
C        IR,IC,IL -  MODEL SEGMENT CURRENTLY BEING ANALYZED
C        LIST     -  OPTIONS TO LIST COMPONENTS OF MODEL INPUT DECK 
C                     = 0  -  PROVIDE USER WITH LIST OF INPUT
C                     = 1  -  DO NOT LIST INPUT
C 
C 
C        IDISK    -  FLAG WHICH SIGNALS -TUNER- TO INITIATE DISK STORAGE 
C                    OF VARIABLES THAT USER WISHES TO DUMP AT END OF RUN 
C        PRNTG    -  PRINT INTERVAL REQUESTED FOR GLOBAL CONCENTRATIONS
C        IDUMP    -  ARRAY CONTAINING THE SEGMENTS NUMBERS TO BE PRINTED AS
C                    PART OF THE INTERMEDIATE DUMPS, I.E. THE SIX SEGMENTS
C                    WHICH ARE DUMPED TO STANDOUT OUT DURING THE COURSE OF
C                    THE MODEL SIMULATION (PRINTED AT INTERVAL 'PRNTG')
C        PRNTD    -  PRINT INTERVAL USED TO GENERATE THE 'DETAILED' DUMPS
C                    FOR SELECTED SEGMENTS (VIA RCAWRIT)
C        NDMPS    -  NUMBER OF SEGMENTS SPECIFIED FOR DETAILED OUTPUT
C                    DUMPS AT PRINT INTERVAL 'PRNTD'
C        IFDMPS   -  ARRAY CONTAINING LIST OF SEGMENT NUMBERS FOR WHICH
C                    'DETAILED' DUMPS, I.E. CONCENTRATIONS AND OTHER SECONDARY 
C                    VARIABLES OF INTEREST ARE TO BE SAVED VIA RCAWRIT
C        IREC     -  COUNTER USED BY RCA FOR GLOBAL DUMP DISK FILE STORAGE 
C
C
C        SYNAME   -  ALPHA-NUMERIC NAMES FOR EACH SYSTEM 
C        SYSBY    -  VECTOR CONTAINING BYPASS OPTION FOR EACH SYSTEM 
C        RBY      -  VECTOR CONTAINING R BYPASS INDICATOR FOR EACH SYSTEM
C        QBY      -  VECTOR CONTAINING Q BYPASS INDICATOR FOR EACH SYSTEM
C        CYCLE    -  FLAG TO DETERMINE WHETHER INITIAL CONDITIONS ARE TO BE
C                    READ FROM A USER PROVIDED INPUT FILE OR FROM A PREVIOUS
C                    RCA RUN (I.E. THE EQUIVALENT OF A HOT-START); MAY ALSO BE
C                    USED TO LINK A SERIES OF MULTI-YEAR SIMULATIONS TOGETHER
C                    = 0  -  GET INITIAL CONDITIONS FROM USER SUPPLIED INPUT
C                            FILE (EQUIVALENT TO A COLD-START RUN)
C                    = 1  -  GET INITIAL CONDITIONS FROM A PREVIOUS RCA RUN -
C                            THE RCAFIC FILE (EQUIVALENT TO A HOT-START RUN)
C        NEGSLN   -  VARIABLE INDICATING WHETHER OR NOT NEGATIVE SOLUTIONS 
C                    WILL BE PERMITTED 
C                    = 0  -  NEGATIVE SOLUTIONS NOT PERMITTED  (FLOOR = 0.0) 
C                    = 1  -  NEGATIVE SOLUTIONS PERMITTED  (FLOOR = -1.E 35) 
C        ISIM     -  SIMULATION TYPE --- TIME VARIABLE OR STEADY STATE 
C        INTGRTYP -  INTEGRATION METHOD
C                     = 0  -  IMPLICIT - UPWIND
C                     = 1  -  EXPLICIT - UPWIND
C                     = 2  -  LEAPFROG - CENTRAL
C                     = 3  -  SPLIT TIME STEP - UPWIND
C                     = 4  -  EXPLICIT - UPWIND/SMOLARKIEWICZ
C                     = 5  -  LEAPFROG - UPWIND/SMOLARKIEWICZ
C                     = 6  -  SPLIT TIME STEP - UPWIND/SMOLARKIEWICZ
C        ISMOL    -  NUMBER OF ITERATIONS TO BE USED IF SMOLARKIEWICZ CORRECTOR
C                    IS BEING USED AS PART OF THE INTEGRATION PROCEDURE
C        IDIAGDT  -  FLAG USED TO SELECT WHETHER A FULL SIMULATION IS TO BE
C                    PERFORMED OR WHETHER A DIAGNOSTIC ANALYSIS OF INTEGRATION
C                    STEPSIZES IS TO BE PERFROMED
C
C
C        INITB    -  FLAG WHICH PERMITS THE USER TO PROVIDE INITIALIZATION 
C                    OF VARIABLES IN KINETIC SUBROUTINE   - TUNER-
C        TZERO    -  TIME USER WISHES TO START SIMULATION AT, OR TIME FOR
C                    WHICH THE USER WANTS A STEADY STATE SOLUTION
C        DT       -  INTEGRATION INTERVAL
C        TIME     -  INTERNAL CLOCK USED TO KEEP TRACK OF THE SIMULATION TIME 
C        DTWQ     -  INTEGRATION STEPSIZE TO BE USED FOR EVALUATING THE
C                    KINETIC PORTION OF THE MASS BALANCE EQUATIONS
C        TIMEWQ   -  INTERNAL CLOCK USED TO KEEP TRACK OF KINETIC PORTION OF
C                    THE INTEGRATION PROCEDURE
C        SCALT    -  TIME WARP FACTOR ... TIME CONVERSION FACTOR 
C        TEND     -  ENDING TIME OF SIMULATION (OR TIME TO CHANGE 
C                    TO NEXT INTEGRATION INTERVAL) 
C        OMEGA    -  RELAXATION FACTOR USED FOR GAUSS-SEIDEL ITERATION 
C                    IF PERFORMING A STEADY STATE SIMULATION 
C        ITCHCK   -  NUMBER OF GAUSS-SEIDEL ITERATIONS TO BE PERFORMED 
C                    BEFORE CHECKING FOR CONVERGENCE 
C        MXITER   -  MAXIMUM NUMBER OF ITERATIONS PERMITTED
C 
C 
C        CARAY    -  CONCENTRATIONS FOR APPROPRIATE SYSTEM AND SEGMENT 
C        CDARAY   -  CONCENTRATION DERIVATIVES
C        CMAX     -  MAXIMUM ALLOWABLE CONCENTRATION FOR EACH SYSTEM 
C                    IF ANY OF THESE VALUES ARE EXCEEDED IN THEIR RESPECT  -  
C                    IVE SYSTEMS THE PROGRAM WILL ABORT
C        CMIN     -  MINIMUM CONCENTRATIONS, FOR EACH SYSTEM, BELOW WHICH
C                    THE ACCURACY CRITERIA IS IGNORED
C        SCALQ    -  SCALE FACTOR FOR ADVECTIVE FLOW FIELD
C        SCALRX   -  SCALE FACTOR FOR HORIZONTAL EXCHANGE FIELD
C        SCALRY   -  SCALE FACTOR FOR LATERAL EXCHANGE FIELD
C        SCALRZ   -  SCALE FACTOR FOR VERTICAL EXCHANGE FIELD
C
C
C   /INTHIST/  VARIABLES ASSOCIATED WITH INTEGRATION HISTORY TIMESTEPS
C        NSTEP    -  NUMBER OF INTEGRATION STEPSIZES/TIMES TO BE USED
C                    FOR DESCRIBING THE INTEGRATION HISTORY
C        STEP     -  VECTOR CONTAINING THE INTEGRATION STEPSIZES (DT)
C        TBRK     -  VECTOR CONTAINING THE CORRESPONDING TIMES FOR THE VECTOR
C                    -STEP-    STEP(I) WILL BE USED UNTIL TIME TBRK(I), THEN
C                    SWITCH TO STEP(I+1) UNTIL TBRK(I+1), ETC.
C
C
C        PARAM2D  -  2-D ARRAY CONTAINING PARAMETERS FOR EACH LAYER
C        PARAM3D  -  3-D ARRAY CONTAINING PARAMETERS FOR EACH SEGMENT
C        CONST    -  A VECTOR OF CONSTANTS COMMON ACROSS ALL SEGMENTS
C 
C 
C
C 
C        BVOL     -  VECTOR CONTAINING VOLUMES (MCM)
C        VDER     -  VOLUME DERIVATIVES (MCM/DAY)
C        RX,RY,RZ -  ARRAYS CONTAINING THE EXCHANGES (MCM/DAY)
C        QX,QY,QZ -  ARRAYS CONTAINING THE ADVECTIVE FLOWS (MCM/DAY)
C        BBC      -  VECTOR CONTAINING BOUNDARY CONDITIONS (MG/L)
C        SBC      -  VECTOR CONTAINING 'SLOPE' OF BOUNDARY CONDITIONS
C                    IF PIECEWISE LINEAR OPTION SELECTED
C        BPS      -  VECTOR CONTAINING POINT SOURCE LOADS (GMS)
C        BNPS     -  VECTOR CONTAINING NONPOINT SOURCE LOADS (GMS)
C        BFL      -  VECTOR CONTAINING FALL-LINE LOADS (GMS)
C        BATM     -  VECTOR CONTAINING ATMOSPHERIC LOADS (GMS)
C        BFUNC    -  VECTOR CONTAINING KINETIC TIME FUNCTIONS 
C 
C        SCALBC   -  SCALE FACTORS FOR BOUNDARY CONDITIONS
C        SCALPS   -  SCALE FACTORS FOR POINT SOURCE LOADS
C        SCALNPS  -  SCALE FACTORS FOR NONPOINT SOURCE LOADS
C        SCALFL   -  SCALE FACTORS FOR FALL-LINE LOADS
C        SCALATM  -  SCALE FACTORS FOR ATMOSPHERIC LOADS
C 
C        IBCOPT   -  BC OPTION
C                      1=SIGMA LEVEL - CONSTANT
C                      2=SIGMA LEVEL - TIME-VARYING
C                      3=STANDARD LEVEL - CONSTANT
C                      4=STANDARD LEVEL - TIME-VARYING
C        IBCPLOPT -  PIECEWISE LINEAR BOUNDARY CONCENTRATION OPTION
C                      0=STEP FUNCTION APPROXIMATION FOR BCS
C                      1=PIECEWISE LINEAR APPROXIMATION FOR BCS
C        NOBC     -  NUMBER OF B.C.'S FOR EACH SYSTEM
C        STNDLEVL -  STANDARD LEVEL DEPTHS (IF BCS TO BE READ USING
C                    STANDARD LEVEL OPTION)
C        NOBCSL   -  NUMBER OF STANDARD LEVELS FOR EACH BC
C        IPSOPT   -  PS LOADING OPTION (1==>CONSTANT;2==>TIME-VARYING)
C        NOPS     -  NUMBER OF PS LOADS FOR EACH SYSTEM
C        INPSOPT  -  NPS OPTION (1==>CONSTANT;2==>TIME-VARYING)
C        NONPS    -  NUMBER OF NPS LOADS FOR EACH SYSTEM
C        IFLOPT   -  FALL-LINE OPTION (1==>CONSTANT;2==>TIME-VARYING)
C        NOBC     -  NUMBER OF FL LOADS FOR EACH SYSTEM
C        IATMOPT  -  ATMOSPHERIC OPTION (1==>CONSTANT;2==>TIME-VARYING)
C        NOATM    -  NUMBER OF LOADS FOR EACH SYSTEM
C        NOPAM    -  NUMBER OF PARAMETERS FOR EACH SEGMENT 
C        NCONS    -  NUMBER OF CONSTANTS 
C        NFUNC    -  NUMBER OF STRAIGHT LINE FUNCTIONS 
C 
C        ITIMX    -  COUNTER USED TO PICK PROPER 'B' VALUES FOR....
C        ITIMHYD  -  FOR HYDRODYNAMIC FIELDS
C        ITIMF    -  FOR MISC TIME FUNCTIONS 
C 
      INCLUDE  'RCACM'
C 
C               RCA FILE STRUCTURE
C 
C    FILE        
C     NO.  TYPE    NAME            USAGE 
C    ----  ----   ------      -------------------------
C
C      5   ASCII              STANDARD INPUT
C      6   ASCII              STANDARD OUTPUT
C
C
C     10   BINRY  RCAF10      PRINT TIME HISTORY FOR 'GLOBAL' DUMPS
C                             (+ NROW, NCOL, NLAYR, NOSYS AS REC 1)
C                             (+ SYSTEM NAMES AS REC 2)
C                             (+ SYSBY AS REC 3)
C                             (+ FSM LAND MASK AS REC 4)
C     11   BINRY  RCAF11      CONCENTRATIONS FOR EACH MODEL SEGMENT
C     12   BINRY  RCAF12      PRINT TIME HISTORY FOR MORE DETAILED
C                             SEGMENT DUMPS
C                             (+ NOSEG DUMPED AS REC 1)
C                             (+ EQUIV TABLE OF SEGS SAVED AS REC 2)
C     13   BINRY  RCAF13      MORE DETAILED DUMPS FOR SELECTED SEGMENTS
C     14   BINRY  RCAF14      SEDIMENT LAYER DUMP FILES (IF USER MODEL
C                             INCLUDES A SEDIMENT COMPONENT)
C     15   BINRY  RCAFIC      WATER COLUMN ICs FOR MULTI  -  YEAR RUNS
C     16   BINRY  RCAFICSED   INITIAL CONDITIONS FOR MULTI  - YEAR RUNS
C
C   17-19    *       *        CAN BE USED FOR SPECIFIC APPLICATIONS AS
C                             OUTPUT FILES AS NEEDED BY DEFINING IN THE
C                             USER  - SPECIFIED KINETIC SUBROUTINE -TUNER-
C
C
C     20   BINRY  gcm_geom    GEOMETRY AS SUPPLIED FROM HYDRODYNAMIC MODEL
C     20   BINRY  wet_grid    (IROW,ICOL) LOCATIONS OF THE WATER SEGMENTS
C                             IN THE HYDRODYNAMIC MODEL
C     20   BINRY  HYDFILNA    TRANSPORT TERMS AS SUPPLIED FROM HYDRODYNAMIC
C                             MODEL (THIS IS THE ECOM-3D gcm_trans FILE)
C     21   ASCII  BCFILNA     USER SELECTED FILE FOR BOUNDARY CONDITIONS
C     22   ASCII  PSFILNA     USER SELECTED FILE FOR POINT SOURCE LOADS
C     23   ASCII  NPSFILNA    USER SELECTED FILE FOR NONPOINT SOURCE LOADS
C     24   ASCII  FLFILNA     USER SELECTED FILE FOR FALL-LINE LOADS
C     25   ASCII  ATMFILNA    USER SELECTED FILE FOR ATMOSPHERIC LOADS
C     26   ASCII  PCFILNA     USER SELECTED FILE FOR PARAMETERS,
C                             CONSTANTS AND TIME FUNCTIONS
C     27   ASCII  ICFILNA     USER SELECTED FILE FOR INITIAL CONDITIONS
C
C   28-29    *       *        CAN BE USED FOR SPECIFIC APPLICATIONS AS
C                             INPUT FILES AS NEEDED BY DEFINING IN THE
C                             USER  - SPECIFIED KINETIC SUBROUTINE -TUNER-
C
C
C     34   BINRY  scratch     TIME VARIABLE KINETIC FUNCTIONS 

C     41   BINRY  scratch     VARIABLE NAMES FOR SYSTEM DUMPS 
C 
C      *  AS DEFINED BY THE USER
C 

      OPEN (UNIT=10,FILE='RCAF10',FORM='UNFORMATTED')
      OPEN (UNIT=11,FILE='RCAF11',FORM='UNFORMATTED')
      OPEN (UNIT=15,FILE='RCAFIC',FORM='UNFORMATTED')

C 
C        CALLING SEQUENCE FOR RCA SUBROUTINES 
C 
C        CALL INPUT ROUTINES

      CALL RCA01
      CALL RCA02
      CALL RCA03
      CALL RCA04
      CALL RCA05
      CALL RCA06
      CALL RCA07

C        CALL SUBROUTINES TO SOLVE STATE EQUATIONS
      IF(ISIM.EQ.0)   THEN
 
C           TIME-VARIABLE
            
C   IMPLICIT - UPWIND
        IF(INTGRTYP.EQ.0)  CALL RCAIMP
C   EXPLICIT - UPWIND
        IF(INTGRTYP.EQ.1)  CALL RCAEXP
C   LEAPFROG - CENTRAL
        IF(INTGRTYP.EQ.2)  CALL RCALF
C   SPLIT TIME STEP _ UPWIND
        IF(INTGRTYP.EQ.3)  CALL RCASPLT
C   EXPLICIT - UPWIND/SMOLARKIEWICZ
        IF(INTGRTYP.EQ.4)  CALL RCAEXPS
C   LEAPFROG - UPWIND/SMOLARKIEWICZ
        IF(INTGRTYP.EQ.5)  CALL RCALFS
C   EXPLICIT - UPWIND/SMOLARKIEWICZ
        IF(INTGRTYP.EQ.6)  CALL RCASPLTS

c     ELSE

C           STEADY STATE
c        CALL RCASS
      ENDIF

C        CALL OUTPUT ROUTINES
      CALL RCA12

      CLOSE (10,STATUS='KEEP')
      CLOSE (11,STATUS='KEEP')
      CLOSE (13,STATUS='KEEP')
      DO 100 ICLOSE=31,41
  100 CLOSE (ICLOSE,STATUS='DELETE')

      CALL EXIT 
      END 
