      SUBROUTINE RCAEXP
C 
C        RCAEXP IS THE MASTER ROUTINE USED TO INTEGRATE THE DIFFERENTIAL
C               EQUATIONS OF THE WATER QUALITY MODEL.
C               TIME DERIVATIVE: EXPLICIT - 1ST ORDER EULER
C               SPACE DERIVATIVE: UPWIND
C
C************************************************************************
C 
      INCLUDE 'RCACM' 
      INTEGER  NEGSEG(999,3)
      REAL   CONCS(NROW,NCOL,0:NLAYR)
     .      ,DM(NROW,NCOL,NLAYR),BVOLN(NROW,NCOL,NLAYR),TOTMAS
     .      ,AL(NROW,NCOL,NLAYR),AD(NROW,NCOL,NLAYR),AU(NROW,NCOL,NLAYR)
     .      ,BETA(NROW,NCOL,NLAYR),GAMMA(NROW,NCOL,NLAYR)
     .      ,MF(NROW,NCOL,NLAYR)

      EQUIVALENCE
     .       (MF(1,1,1),SCRATCH_TRAN(1,1,1,2))
     .    ,  (AL(1,1,1),SCRATCH_TRAN(1,1,1,3))
     .    ,  (AD(1,1,1),SCRATCH_TRAN(1,1,1,4))
     .    ,  (AU(1,1,1),SCRATCH_TRAN(1,1,1,5))
     .    ,  (BETA(1,1,1),SCRATCH_TRAN(1,1,1,6))
     .    ,  (GAMMA(1,1,1),SCRATCH_TRAN(1,1,1,7))
 

C        INITIALIZATION

      ITRAK=1 
      TIME=0.0 
      TIMEWQ=0.0 
      INITB=0
      DTSTEP = DT
      DT = DT*SCALT
      DTWQ=DTWQ*SCALT
      NNG=1
      NND=1
      R=1.E-10
      IF(TZERO.NE.0.)   THEN
         CALL RCA08
         TIME = TZERO
         NNG=TIME/PRNTG+0.5
         IF (PRNTD.NE.0.0) NND=TIME/PRNTD+0.5
      ENDIF

      DO 30 IL=1,NLAYR
        DO 30 IC=1,NCOL
          DO 30 IR=1,NROW
            CONCS(IR,IC,IL) = 0.0
 30   CONTINUE

C        SET UP -AVECT- ARRAYS FOR TIME = 0.0 (TZERO)
      CALL RCAEXP1

C        PRINT INITIAL CONDITIONS
      CALL RCA09
      IDISK = 3 
 
   40 CONTINUE

C        IF PIECEWISE LINEAR BOUNDARY CONDITIONS THEN
C           GET CONCENTRATIONS AT CURRENT TIME LEVEL

      IF(IBCPLOPT.EQ.1)   THEN
c$doacross local(isys,deltbc,il,ic,ir,i)
        DO 50 ISYS=1,NOSYS
          IF(NOBC(ISYS).EQ.0)  GO TO 50
          DELTBC = TIME - NXBCT
          DO 45 I=1,NOBC(ISYS)
            IR=IBC(1,I,ISYS)
            IC=IBC(2,I,ISYS)
            IL=IBC(3,I,ISYS)
            CARAY(IR,IC,IL,ISYS) = DELTBC*SBC(I,ISYS) + BBC(I,ISYS)
   45     CONTINUE
   50  CONTINUE
      ENDIF

C        EVALUATE KINETIC DERIVATIVES
      IF (DTWQ.EQ.0.0.OR.TIME.GE.TIMEWQ.OR.IDISK.EQ.3) THEN
        CALL TUNER
        IDISK = 0
        TIMEWQ=TIMEWQ+DTWQ
      ENDIF
      INITB = 1


C
C             EULER INTEGRATION STEP
C
C        COMPUTE VOLUMES AT TIME LEVEL 'N+1'

c$doacross local(il,ic,ir,is,ie)
 100  DO 110 IL=1,NLAYR
        DO 110 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 110 IR=IS,IE
            BVOLN(IR,IC,IL) = BVOL(IR,IC,IL) + DT*VDER(IR,IC,IL)
 110  CONTINUE

      DO 400 ISYS=1,NOSYS 
C        CHECK FOR SYSTEM BYPASS
      IF(SYSBY(ISYS).EQ.1)   GO TO 400

      INEGS=0
      TOTMAS=0.0
 
C  CALCULATE CHANGE IN MASS DUE TO ADVECTIVE TRANSPORT
C     AND HORIZONTAL MIXING
 
      IF (NLAYR.GT.1) THEN

c$doacross local(ic,ir,is,ie)
C      SURFACE LAYER
        DO 120 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 120 IR=IS,IE
            IF(FSM(IR,IC).LE.0.)  GO TO 120
            DM(IR,IC,1) = - DIAG(IR,IC,1)*CARAY(IR,IC,1,ISYS)
     .        + AVECT(IR,IC,1,1)*CARAY(IR-1,IC,1,ISYS)
     .         + AVECT(IR,IC,1,2)*CARAY(IR+1,IC,1,ISYS)
     .          + AVECT(IR,IC,1,3)*CARAY(IR,IC-1,1,ISYS)
     .           + AVECT(IR,IC,1,4)*CARAY(IR,IC+1,1,ISYS)
     .            + AVECT(IR,IC,1,6)*CARAY(IR,IC,2,ISYS)
  120   CONTINUE

C      LAYERS 2 TO NLAYR-1
c$doacross local(ic,ir,il,is,ie)
        DO 125 IL=2,NLAYR-1
          DO 125 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 125 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 125
              DM(IR,IC,IL) = - DIAG(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .         + AVECT(IR,IC,IL,1)*CARAY(IR-1,IC,IL,ISYS)
     .          + AVECT(IR,IC,IL,2)*CARAY(IR+1,IC,IL,ISYS)
     .           + AVECT(IR,IC,IL,3)*CARAY(IR,IC-1,IL,ISYS)
     .            + AVECT(IR,IC,IL,4)*CARAY(IR,IC+1,IL,ISYS)
     .             + AVECT(IR,IC,IL,5)*CARAY(IR,IC,IL-1,ISYS)
     .              + AVECT(IR,IC,IL,6)*CARAY(IR,IC,IL+1,ISYS)
  125   CONTINUE

C      BOTTOM LAYER
c$doacross local(ic,ir,is,ie)
        DO 130 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 130 IR=IS,IE
            IF(FSM(IR,IC).LE.0.)  GO TO 130
            DM(IR,IC,NLAYR) =
     .        - DIAG(IR,IC,NLAYR)*CARAY(IR,IC,NLAYR,ISYS)
     .         + AVECT(IR,IC,NLAYR,1)*CARAY(IR-1,IC,NLAYR,ISYS)
     .          + AVECT(IR,IC,NLAYR,2)*CARAY(IR+1,IC,NLAYR,ISYS)
     .           + AVECT(IR,IC,NLAYR,3)*CARAY(IR,IC-1,NLAYR,ISYS)
     .            + AVECT(IR,IC,NLAYR,4)*CARAY(IR,IC+1,NLAYR,ISYS)
     .             + AVECT(IR,IC,NLAYR,5)*CARAY(IR,IC,NLAYR-1,ISYS)
  130   CONTINUE

      ELSE

        IL=1
c$doacross local(ic,ir,is,ie)
        DO 131 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 131 IR=IS,IE
            IF(FSM(IR,IC).LE.0.)  GO TO 131
            DM(IR,IC,IL) = - DIAG(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .       + AVECT(IR,IC,IL,1)*CARAY(IR-1,IC,IL,ISYS)
     .        + AVECT(IR,IC,IL,2)*CARAY(IR+1,IC,IL,ISYS)
     .         + AVECT(IR,IC,IL,3)*CARAY(IR,IC-1,IL,ISYS)
     .          + AVECT(IR,IC,IL,4)*CARAY(IR,IC+1,IL,ISYS)
  131   CONTINUE
      ENDIF

C  ADD SOURCES/SINKS DUE TO KINETICS

c$doacross local(ic,ir,il,is,ie)
      DO 240 IL=1,NLAYR
        DO 240 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 240 IR=IS,IE
            DM(IR,IC,IL) = DM(IR,IC,IL) + CDARAY(IR,IC,IL,ISYS)
 240  CONTINUE

C  ADD LOADS
 
      IF(NOPS(ISYS).GT.0)  THEN
        DO 250 I=1,NOPS(ISYS) 
          IR = IPS(1,I,ISYS)
          IC = IPS(2,I,ISYS)
          IL = IPS(3,I,ISYS)
          DM(IR,IC,IL) = DM(IR,IC,IL) + BPS(I,ISYS)
  250   CONTINUE
      ENDIF

      IF(NONPS(ISYS).GT.0)   THEN
        DO 260 I=1,NONPS(ISYS)
          IR = INPS(1,I,ISYS)
          IC = INPS(2,I,ISYS)
          IL = INPS(3,I,ISYS)
          DM(IR,IC,IL) = DM(IR,IC,IL) + BNPS(I,ISYS)
  260   CONTINUE
      ENDIF

      IF(NOFL(ISYS).GT.0)   THEN
        DO 270 I=1,NOFL(ISYS)
          IR = IFL(1,I,ISYS)
          IC = IFL(2,I,ISYS)
          IL = IFL(3,I,ISYS)
          DM(IR,IC,IL) = DM(IR,IC,IL) + BFL(I,ISYS)
  270   CONTINUE
      ENDIF

      IF(NOATM(ISYS).GT.0)   THEN
        DO 280 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 280 IR=IS,IE
            DM(IR,IC,1) = DM(IR,IC,1) + BATM(ISYS)*XAZ(IR,IC)
  280   CONTINUE
      ENDIF

C
C  CALCULATE CONCENTRATIONS: KINETICS, LOADS 
C

c$doacross local(ic,ir,il,is,ie)
      DO 300 IL=1,NLAYR
        DO 300 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 300 IR=IS,IE
            IF(FSM(IR,IC).LE.0.)  GO TO 300
              CONCS(IR,IC,IL) = (BVOL(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .                           + DT*DM(IR,IC,IL))/BVOLN(IR,IC,IL)
 300  CONTINUE

C  BOUNDARY ELEMENTS ARE UNCHANGED

      DO 305 I=1,NOBCALL
        IR=IBCALL(1,I)
        IC=IBCALL(2,I)
        IL=IBCALL(3,I)
        CONCS(IR,IC,IL)=CARAY(IR,IC,IL,ISYS)
 305  CONTINUE
 

C  IMPLICIT VERTICAL DIFFUSION STEP  (IF NLAYR > 1)

      IF (NLAYR.EQ.1)  GO TO 380
 
C  COMPLETE REST OF INTEGRATION STEP
C  I.E., THE IMPLICIT VERTICAL DIFFUSION STEP
 
c$doacross local(ic,ir,il,is,ie)
      DO 340 IL=1,NLAYR
       DO 340 IC=2,NCOL-1
        IS = IRS(IC)
        IE = IRE(IC)
        DO 340 IR=IS,IE
          MF(IR,IC,IL) = CONCS(IR,IC,IL)*BVOLN(IR,IC,IL)
 340  CONTINUE

C        SET UP DIAGONAL AND OFF-DIAGONAL ELEMENTS

c$doacross local(ic,ir,il,is,ie)
      DO 350 IC=2,NCOL-1
       IS = IRS(IC)
       IE = IRE(IC)
       DO 350 IR=IS,IE
        IF(FSM(IR,IC).LE.0)  GO TO 350
        AL(IR,IC,1) = 0.
        AD(IR,IC,1) = BVOLN(IR,IC,1) + DT*RZ(IR,IC,2)
        AU(IR,IC,1) = -DT*RZ(IR,IC,2)
        DO 345 IL=2,NLAYR-1
          AL(IR,IC,IL) = -DT*RZ(IR,IC,IL)
          AD(IR,IC,IL) = BVOLN(IR,IC,IL) +
     .                        DT*(RZ(IR,IC,IL) + RZ(IR,IC,IL+1))
          AU(IR,IC,IL) = -DT*RZ(IR,IC,IL+1)
  345   CONTINUE
        AL(IR,IC,NLAYR) = -DT*RZ(IR,IC,NLAYR)
        AD(IR,IC,NLAYR) = BVOLN(IR,IC,NLAYR) + DT*RZ(IR,IC,NLAYR)
        AU(IR,IC,NLAYR) = 0.0
  350 CONTINUE

C        FORWARD SWEEP OF TRIDIAGONAL SCHEME

c$doacross local(ic,ir,il,is,ie)
      DO 360 IC=2,NCOL-1
       IS = IRS(IC)
       IE = IRE(IC)
       DO 360 IR=IS,IE
        IF(FSM(IR,IC).LE.0)  GO TO 360
        BETA(IR,IC,1) = AD(IR,IC,1)
        GAMMA(IR,IC,1) = MF(IR,IC,1)/BETA(IR,IC,1)
        DO 355 IL=2,NLAYR
         BETA(IR,IC,IL) = AD(IR,IC,IL)
     .                 - AL(IR,IC,IL)*AU(IR,IC,IL-1)/BETA(IR,IC,IL-1)
         GAMMA(IR,IC,IL) = (MF(IR,IC,IL)
     .                 - AL(IR,IC,IL)*GAMMA(IR,IC,IL-1))/BETA(IR,IC,IL)
  355   CONTINUE
  360 CONTINUE

C        BACKWARD SWEEP FOR SOLUTION

c$doacross local(ic,ir,il,is,ie,ilyr)
      DO 370 IC=2,NCOL-1
       IS = IRS(IC)
       IE = IRE(IC)
       DO 370 IR=IS,IE
        IF(FSM(IR,IC).LE.0)  GO TO 370
        CONCS(IR,IC,NLAYR) = GAMMA(IR,IC,NLAYR)
        DO 365 ILYR=2,NLAYR
         IL = NLAYR+1-ILYR
         CONCS(IR,IC,IL) = GAMMA(IR,IC,IL)
     .              - AU(IR,IC,IL)*CONCS(IR,IC,IL+1)/BETA(IR,IC,IL)
  365   CONTINUE
  370 CONTINUE

C  BOUNDARY ELEMENTS ARE UNCHANGED

      DO 375 I=1,NOBCALL
        IR=IBCALL(1,I)
        IC=IBCALL(2,I)
        IL=IBCALL(3,I)
        CONCS(IR,IC,IL)=CARAY(IR,IC,IL,ISYS)
  375 CONTINUE
 

  380 CONTINUE

c$doacross local(ic,ir,il,is,ie,inegs,totmas) ,share(negseg)
      DO 385 IL=1,NLAYR
        DO 385 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 385 IR=IS,IE
            IF(FSM(IR,IC).LE.0.)GO TO 385
C
C        CHECK TO SEE IF NEGATIVE SOLUTIONS PERMITTED 
C
            IF (NEGSLN.EQ.1)   THEN
              CARAY(IR,IC,IL,ISYS) = CONCS(IR,IC,IL)
            ELSE
C
C        IF NOT ... CHECK TO MAKE SURE SOLUTION IS NOT FORCED THRU ZERO
C
              IF(CONCS(IR,IC,IL).GE.0.0)   THEN
                CARAY(IR,IC,IL,ISYS) = CONCS(IR,IC,IL)
              ELSE
C
C          PREVENT NEG SOLN BY QUARTERING PRESENT CONC AND PRINT WARNING
C
                CARAY(IR,IC,IL,ISYS) = CARAY(IR,IC,IL,ISYS)/4.
c               call m_lock
                INEGS = INEGS+1
                IF(INEGS.LE.999)  THEN
                  NEGSEG(INEGS,1) = IR
                  NEGSEG(INEGS,2) = IC
                  NEGSEG(INEGS,3) = IL
                ENDIF
                TOTMAS = TOTMAS+(CARAY(IR,IC,IL,ISYS)-CONCS(IR,IC,IL))
     .                 *BVOLN(IR,IC,IL)
c               call m_unlock
              ENDIF
            ENDIF
 385  CONTINUE

      IF(INEGS.GT.0)  THEN
c               call m_lock
        CALL RCAMESS(2,0.001*TOTMAS)
        WRITE(OUT,1000)  ((NEGSEG(I,J),J=1,3),I=1,INEGS)
c               call m_unlock
 1000   FORMAT(30X,12I5)
      ENDIF

  400 CONTINUE
C
c$doacross local(il,ic,ir,is,ie)
      DO 415 IL=1,NLAYR
        DO 415 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 415 IR=IS,IE
            BVOL(IR,IC,IL) = BVOLN(IR,IC,IL)
  415 CONTINUE


C  PREPARE FOR NEXT TIME STEP
 
      TIME=TIME + DTSTEP

C        CHECK TO SEE IF IT IS TIME TO PRINT RESULTS

C  FIRST DETAILED DUMPS

      IF(PRNTD.EQ.0.0)  GO TO 435
      EPSLNP = DTSTEP/(PRNTD*2.)
      M=TIME/PRNTD+EPSLNP
      IF(M.LT.NND)  GO TO 435

C  DETAILED PRINT TIME
C  SET IDISK FOR DETAILED DUMPS

      IDISK=2
      NND=M+1
C
C  SECOND GLOBAL DUMPS
C
 435  EPSLNP=DTSTEP/(PRNTG*2.)
      M=TIME/PRNTG+EPSLNP
      IF (M.LT.NNG) GOTO 450
C
C  GLOBAL PRINT TIME
C  CALL PRINT ROUTINE
C
      CALL RCA09
      IDISK=IDISK+1
      NNG=M+1 

C      
C  NOW PERFORM STABILITY CHECK
C 
C 
      DO 445 ISYS=1,NOSYS 
        IF(SYSBY(ISYS).EQ.1)   GO TO 445
        CKMAX = CMAX(ISYS)

        DO 440 IL=1,NLAYR
          DO 440 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 440 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)GO TO 440
C           DO 440 IR=1,NROW
              IF(CARAY(IR,IC,IL,ISYS).GE.CKMAX)   GO TO 500
  440   CONTINUE
  445 CONTINUE
C
C        CHECK TO SEE IF NECESSARY TO UPDATE TIME-VARIABLE FUNCTIONS
C        FIRST POINT SOURCE LOADS 
C
  450 IF(IPSOPT.GT.1.AND.TIME.GE.NXPST)   
     .   CALL RCA10(BPS,NWK,NOPS,NXPST,22,SCALPS)

C
C        NONPOINT SOURCE LOADS 
C
      IF(INPSOPT.GT.1.AND.TIME.GE.NXNPST)   
     .   CALL RCA10(BNPS,NWK,NONPS,NXNPST,23,SCALNPS)

C
C        FALL-LINE LOADS 
C
      IF(IFLOPT.GT.1.AND.TIME.GE.NXFLT)   
     .   CALL RCA10(BFL,NWK,NOFL,NXFLT,24,SCALFL)

C
C        ATMOSPHERIC LOADS 
C
      IF(IATMOPT.GT.1.AND.TIME.GE.NXATMT)   
     .   CALL RCA10(BATM,1,NOATM,NXATMT,25,SCALATM)

C
C        NEXT BOUNDARY CONDITIONS 
C
      IF(IBCOPT.GT.1.AND.TIME.GT.NXBCT) CALL RCA11 

C
C        NEXT TRANSPORT FIELDS
C
      IF(TIME.GE.HYDBRK(ITIMHYD))  THEN
         CALL RCA03A
         CALL RCAEXP1
         ITIMHYD = ITIMHYD+1
      ENDIF


C        CHECK FOR END OF SIMULATION...
C           IF NOT GO BACK AND TAKE NEXT INTEGRATION STEP

      IF(TIME.LE.TEND)   GO TO 40 

      ITRAK = ITRAK+1 
      IF(ITRAK.LE.NSTEP)  THEN
         DT=STEP(ITRAK)
         TEND = TBRK(ITRAK) 
         IF(DT.EQ.0.0)   THEN
            CALL RCAMESS(6,DT)
            CALL EXIT
         ENDIF
      ELSE
         TEND = 0.0
      ENDIF 

      DTSTEP = DT 
      DT = SCALT*DT 
      IF(TEND.GT.0.)   GO TO 40 

C        FINISHED...CALL FOR FINAL DUMPS

      IDISK = 1 
      CALL TUNER
      RETURN


C         STABILITY CRITERIA VIOLATED...ABEND 

  500 CALL RCAMESS(1,CKMAX) 
      WRITE(OUT,9000)  BVOLN(IR,IC,IL),DIAG(IR,IC,IL)
 9000 FORMAT(10X,'VOLUME =',E13.4,'  ,DIAGONAL =',E13.4)
      IREC = IREC+1 
      IDISK = 1 
      CALL TUNER
      RETURN
      END 

