      SUBROUTINE RCAEXPS
C
C        RCAEXPS IS THE MASTER ROUTINE USED TO INTEGRATE THE DIFFERENTIAL
C                EQUATIONS OF THE WATER QUALITY MODEL.
C                TIME DERIVATIVE: EXPLICIT - 1ST ORDER EULER
C                SPACE DERIVATIVE: UPWIND WITH SMOLARKIEWICZ CORRECTOR
C
C************************************************************************
C
      INCLUDE 'RCACM' 
      INTEGER  NEGSEG(99,3)
      REAL   CONCS(NROW,NCOL,NLAYR),TOTMAS
     .      ,DM(NROW,NCOL,NLAYR),BVOLN(NROW,NCOL,NLAYR)
     .      ,DXBAR1(NROW,NCOL),DYBAR1(NROW,NCOL)
     .      ,DXBAR2(NROW,NCOL),DYBAR2(NROW,NCOL)
     .      ,D(NROW,NCOL),U(NROW,NCOL,NLAYR),V(NROW,NCOL,NLAYR)
     .      ,W(NROW,NCOL,NLAYR)
     .      ,ANTIQX(NROW,NCOL,NLAYR),QXFLX(NROW,NCOL,NLAYR)
     .      ,ANTIQY(NROW,NCOL,NLAYR),QYFLX(NROW,NCOL,NLAYR)
     .      ,ANTIQZ(NROW,NCOL,NLAYR),QZFLX(NROW,NCOL,NLAYR)
     .      ,RXFLX(NROW,NCOL,NLAYR),RYFLX(NROW,NCOL,NLAYR)
     .      ,AL(NROW,NCOL,NLAYR),AD(NROW,NCOL,NLAYR),AU(NROW,NCOL,NLAYR)
     .      ,BETA(NROW,NCOL,NLAYR),GAMMA(NROW,NCOL,NLAYR)
     .      ,MF(NROW,NCOL,NLAYR)

      EQUIVALENCE
     .        (MF(1,1,1),SCRATCH_TRAN(1,1,1,2))
     .      , (AL(1,1,1),SCRATCH_TRAN(1,1,1,3))
     .      , (AD(1,1,1),SCRATCH_TRAN(1,1,1,4))
     .      , (AU(1,1,1),SCRATCH_TRAN(1,1,1,5))
     .      , (BETA(1,1,1),SCRATCH_TRAN(1,1,1,6))
     .      , (GAMMA(1,1,1),SCRATCH_TRAN(1,1,1,7))

      EQUIVALENCE
     .        (DM(1,1,1),SCRATCH_TRAN(1,1,1,8))
     .      , (BVOLN(1,1,1),SCRATCH_TRAN(1,1,1,9))
     .      , (ANTIQX(1,1,1),SCRATCH_TRAN(1,1,1,10))
     .      , (ANTIQY(1,1,1),SCRATCH_TRAN(1,1,1,11))
     .      , (ANTIQZ(1,1,1),SCRATCH_TRAN(1,1,1,12))
     .      , (QXFLX(1,1,1),SCRATCH_TRAN(1,1,1,13))
     .      , (QYFLX(1,1,1),SCRATCH_TRAN(1,1,1,14))
     .      , (QZFLX(1,1,1),SCRATCH_TRAN(1,1,1,15))
 
C        INITIALIZATION

      ITRAK=1 
      TIME=0.0 
      TIMEWQ=0.0 
      INITB=0
      DTSTEP = DT
      DT = DT*SCALT
      DTWQ=DTWQ*SCALT
      NNG=1
      NND=1
      R=1.E-15
      IF(TZERO.NE.0.)   THEN
         CALL RCA08
         TIME = TZERO
         NNG=TIME/PRNTG+0.5
         IF (PRNTD.NE.0.0) NND=TIME/PRNTD+0.5
      ENDIF

c$doacross local(il,ic,ir) , share(rxflx,ryflx)
      DO 20 IL=1,NLAYR
        DO 20 IC=1,NCOL
          DO 20 IR=1,NROW
            CONCS(IR,IC,IL) = 0.0
            ANTIQX(IR,IC,IL) = 0.0
            ANTIQY(IR,IC,IL) = 0.0
            ANTIQZ(IR,IC,IL) = 0.0
            QXFLX(IR,IC,IL) = 0.0
            QYFLX(IR,IC,IL) = 0.0
            QZFLX(IR,IC,IL) = 0.0
            RXFLX(IR,IC,IL) = 0.0
            RYFLX(IR,IC,IL) = 0.0
 20   CONTINUE

C        SET UP -AVECT- ARRAYS FOR TIME = 0.0 (TZERO)
      CALL RCAEXP1

C        PRINT INITIAL CONDITIONS
      CALL RCA09
      IDISK = 3 
 
   40 CONTINUE

C        IF PIECEWISE LINEAR BOUNDARY CONDITIONS THEN
C           GET CONCENTRATIONS AT CURRENT TIME LEVEL

      IF(IBCPLOPT.EQ.1)   THEN
c$doacross local(isys,deltbc,il,ic,ir,i)
        DO 50 ISYS=1,NOSYS
          IF(NOBC(ISYS).EQ.0)  GO TO 50
          DELTBC = TIME - NXBCT
          DO 45 I=1,NOBC(ISYS)
            IR=IBC(1,I,ISYS)
            IC=IBC(2,I,ISYS)
            IL=IBC(3,I,ISYS)
            CARAY(IR,IC,IL,ISYS) = DELTBC*SBC(I,ISYS) + BBC(I,ISYS)
   45    CONTINUE
   50  CONTINUE
      ENDIF
 
C        EVALUATE KINETIC DERIVATIVES 

      IF (DTWQ.EQ.0.0.OR.TIME.GE.TIMEWQ.OR.IDISK.EQ.3) THEN
        CALL TUNER
        IDISK = 0
        TIMEWQ=TIMEWQ+DTWQ
      ENDIF
      INITB = 1

 
C        COMPUTE DEPTHS AT TIME LEVEL N

c$doacross local(ic,ir) , share(d)
      DO 60 IC=1,NCOL
       DO 60 IR=1,NROW
          D(IR,IC) = BVOL(IR,IC,1)/(XAZ(IR,IC)+1.0E-20)/DZ(1)
  60  CONTINUE


C        CALCULATE CONSTANTS

c$doacross local(ic,ir) , share(dxbar1,dxbar2,dybar1,dybar2)
      DO 65 IC=2,NCOL
       DO 65 IR=2,NROW
          DXBAR1(IR,IC) = 0.5*(DX(IR,IC)+DX(IR-1,IC))
          DXBAR2(IR,IC) = 0.5*(DX(IR,IC)+DX(IR,IC-1))
          DYBAR1(IR,IC) = 0.5*(DY(IR,IC)+DY(IR-1,IC))
          DYBAR2(IR,IC) = 0.5*(DY(IR,IC)+DY(IR,IC-1))
   65 CONTINUE

c$doacross local(il,ic,ir) , share(u,v,w)
      DO 70 IL=1,NLAYR
        DO 70 IC=2,NCOL
         DO 70 IR=2,NROW
            U(IR,IC,IL)=QX(IR,IC,IL)/
     .         (DZ(IL)*0.5*(D(IR,IC)+D(IR-1,IC))*DYBAR1(IR,IC))
            V(IR,IC,IL)=QY(IR,IC,IL)/
     .         (DZ(IL)*0.5*(D(IR,IC)+D(IR,IC-1))*DXBAR2(IR,IC))
            W(IR,IC,IL)=QZ(IR,IC,IL)/XAZ(IR,IC)
 70   CONTINUE


C
C             EULER INTEGRATION STEP
C

C        COMPUTE VOLUMES AT TIME LEVEL N+1
 
c$doacross local(il,ic,ir,is,ie)
      DO 80 IL=1,NLAYR
c       DO 80 IC=2,NCOL-1
c         IS = IRS(IC)
c         IE = IRE(IC)
c         DO 80 IR=IS,IE
        DO 80 IC=1,NCOL
          DO 80 IR=1,NROW
            BVOLN(IR,IC,IL) = BVOL(IR,IC,IL) + DT*VDER(IR,IC,IL)
  80  CONTINUE


C        MAIN SYSTEM LOOP

      DO 400 ISYS=1,NOSYS 
 
C        CHECK FOR SYSTEM BYPASS
 
        IF(SYSBY(ISYS).EQ.1)   GO TO 400

        INEGS=0
        TOTMAS=0.0
 
C        THE 120 THROUGH 131 DO LOOPS TAKE AN EULER INTEGRATION STEP
C        STEP FOR HORIZONTAL AND VERTICAL ADVECTIVE TRANSPORT
C           (-DIAG- AND -AVECT- CONTAIN JUST  Q  TERMS)
 
       IF (NLAYR.GT.1) THEN


C      SURFACE LAYER
c$doacross local(ic,ir,is,ie)
         DO 120 IC=2,NCOL-1
           IS = IRS(IC)
           IE = IRE(IC)
           DO 120 IR=IS,IE
             IF(FSM(IR,IC).LE.0.)  GO TO 120
             DM(IR,IC,1) = - DIAG(IR,IC,1)*CARAY(IR,IC,1,ISYS)
     .        + AVECT(IR,IC,1,1)*CARAY(IR-1,IC,1,ISYS)
     .         + AVECT(IR,IC,1,2)*CARAY(IR+1,IC,1,ISYS)
     .          + AVECT(IR,IC,1,3)*CARAY(IR,IC-1,1,ISYS)
     .           + AVECT(IR,IC,1,4)*CARAY(IR,IC+1,1,ISYS)
     .            + AVECT(IR,IC,1,6)*CARAY(IR,IC,2,ISYS)
  120    CONTINUE


C      LAYERS 2 TO NLAYR-1
c$doacross local(il,ic,ir,is,ie)
        DO 125 IL=2,NLAYR-1
          DO 125 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 125 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 125
              DM(IR,IC,IL) = - DIAG(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .         + AVECT(IR,IC,IL,1)*CARAY(IR-1,IC,IL,ISYS)
     .          + AVECT(IR,IC,IL,2)*CARAY(IR+1,IC,IL,ISYS)
     .           + AVECT(IR,IC,IL,3)*CARAY(IR,IC-1,IL,ISYS)
     .            + AVECT(IR,IC,IL,4)*CARAY(IR,IC+1,IL,ISYS)
     .             + AVECT(IR,IC,IL,5)*CARAY(IR,IC,IL-1,ISYS)
     .              + AVECT(IR,IC,IL,6)*CARAY(IR,IC,IL+1,ISYS)

  125   CONTINUE

C      BOTTOM LAYER
c$doacross local(ic,ir,is,ie)
         DO 130 IC=2,NCOL-1
           IS = IRS(IC)
           IE = IRE(IC)
           DO 130 IR=IS,IE
             IF(FSM(IR,IC).LE.0.)  GO TO 130
             DM(IR,IC,NLAYR) =
     .        - DIAG(IR,IC,NLAYR)*CARAY(IR,IC,NLAYR,ISYS)
     .         + AVECT(IR,IC,NLAYR,1)*CARAY(IR-1,IC,NLAYR,ISYS)
     .          + AVECT(IR,IC,NLAYR,2)*CARAY(IR+1,IC,NLAYR,ISYS)
     .           + AVECT(IR,IC,NLAYR,3)*CARAY(IR,IC-1,NLAYR,ISYS)
     .            + AVECT(IR,IC,NLAYR,4)*CARAY(IR,IC+1,NLAYR,ISYS)
     .             + AVECT(IR,IC,NLAYR,5)*CARAY(IR,IC,NLAYR-1,ISYS)
  130   CONTINUE


       ELSE

        IL=1
c$doacross local(ic,is,ie,ir)
        DO 131 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 131 IR=IS,IE
             IF(FSM(IR,IC).LE.0.)  GO TO 131
            DM(IR,IC,IL) = - DIAG(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .       + AVECT(IR,IC,IL,1)*CARAY(IR-1,IC,IL,ISYS)
     .        + AVECT(IR,IC,IL,2)*CARAY(IR+1,IC,IL,ISYS)
     .         + AVECT(IR,IC,IL,3)*CARAY(IR,IC-1,IL,ISYS)
     .          + AVECT(IR,IC,IL,4)*CARAY(IR,IC+1,IL,ISYS)
  131   CONTINUE

       ENDIF

C        ADD SOURCES/SINKS DUE TO KINETICS

c$doacross local(il,ic,ir,is,ie)
        DO 135 IL=1,NLAYR
          DO 135 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 135 IR=IS,IE
              DM(IR,IC,IL) = DM(IR,IC,IL) + CDARAY(IR,IC,IL,ISYS)
 135    CONTINUE

C        ADD LOADS
 
        IF(NOPS(ISYS).GT.0)  THEN
          DO 140 I=1,NOPS(ISYS) 
            IR = IPS(1,I,ISYS)
            IC = IPS(2,I,ISYS)
            IL = IPS(3,I,ISYS)
            DM(IR,IC,IL) = DM(IR,IC,IL) + BPS(I,ISYS)
  140     CONTINUE
        ENDIF


        IF(NONPS(ISYS).GT.0)   THEN
          DO 145 I=1,NONPS(ISYS)
            IR = INPS(1,I,ISYS)
            IC = INPS(2,I,ISYS)
            IL = INPS(3,I,ISYS)
            DM(IR,IC,IL) = DM(IR,IC,IL) + BNPS(I,ISYS)
  145     CONTINUE
        ENDIF

        IF(NOFL(ISYS).GT.0)   THEN
          DO 150 I=1,NOFL(ISYS)
            IR = IFL(1,I,ISYS)
            IC = IFL(2,I,ISYS)
            IL = IFL(3,I,ISYS)
            DM(IR,IC,IL) = DM(IR,IC,IL) + BFL(I,ISYS)
  150     CONTINUE
        ENDIF

        IF(NOATM(ISYS).GT.0)   THEN
          DO 155 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 155 IR=IS,IE
              DM(IR,IC,1) = DM(IR,IC,1) + BATM(ISYS)*XAZ(IR,IC)
  155     CONTINUE
        ENDIF


C
C  CALCULATE C AT TIME LEVEL N+1
C    PREDICTOR STEP (UPWIND ADVECTION, KINETICS, LOADS)
C

c$doacross local(il,ic,ir,is,ie)
        DO 170 IL=1,NLAYR
          DO 170 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 170 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 170
              CONCS(IR,IC,IL) = (BVOL(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .                           + DT*DM(IR,IC,IL))/BVOLN(IR,IC,IL)
  170   CONTINUE


C  BOUNDARY ELEMENTS ARE UNCHANGED
 
      DO 175 I=1,NOBCALL
        IR=IBCALL(1,I)
        IC=IBCALL(2,I)
        IL=IBCALL(3,I)
        CONCS(IR,IC,IL)=CARAY(IR,IC,IL,ISYS)
 175  CONTINUE

C
C******************************************************************
C

C           SMOLARKIEWICZ METHOD FOR SECOND-ORDER ACCURACY
 
 
C        CALCULATE ANTIDIFFUSIION FLOWS ( = VELOCITIES * AREAS)
 

c$doacross local(il,ic,ir,is,ie)
      DO 180 IL=1,NLAYR
        DO 180 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 180 IR=IS,IE

          IF(FSM(IR,IC).LE.0.) GO TO 180

            IF(CONCS(IR,IC,IL).LE.1.0E-09 .OR.
     .            CONCS(IR-1,IC,IL).LE.1.0E-09)  THEN
               ANTIQX(IR,IC,IL) = 0.0
            ELSE IF(ABS(U(IR,IC,IL)).LT.(U(IR,IC,IL)*U(IR,IC,IL)*
     .                 DT/DXBAR1(IR,IC)))   THEN
               ANTIQX(IR,IC,IL) = 0.0
            ELSE
               ANTIQX(IR,IC,IL) = ABS(U(IR,IC,IL))*
     .          (1.-ABS(U(IR,IC,IL))*DT/DXBAR1(IR,IC))*
     .               (CONCS(IR,IC,IL)-CONCS(IR-1,IC,IL))/
     .                    (CONCS(IR,IC,IL)+CONCS(IR-1,IC,IL)+R)
            ENDIF

            IF(CONCS(IR,IC,IL).LE.1.0E-09 .OR.
     .            CONCS(IR,IC-1,IL).LE.1.0E-09)  THEN
               ANTIQY(IR,IC,IL) = 0.0
            ELSE IF(ABS(V(IR,IC,IL)).LT.(V(IR,IC,IL)*V(IR,IC,IL)*
     .                 DT/DYBAR2(IR,IC)))   THEN
               ANTIQY(IR,IC,IL) = 0.0
            ELSE
               ANTIQY(IR,IC,IL) = ABS(V(IR,IC,IL))*
     .          (1.-ABS(V(IR,IC,IL))*DT/DYBAR2(IR,IC))*
     .               (CONCS(IR,IC,IL)-CONCS(IR,IC-1,IL))/
     .                    (CONCS(IR,IC,IL)+CONCS(IR,IC-1,IL)+R)
            ENDIF


            IF (IL.GT.1) THEN
               IF(CONCS(IR,IC,IL).LE.1.0E-09 .OR.
     .               CONCS(IR,IC,IL-1).LE.1.0E-09)  THEN
                  ANTIQZ(IR,IC,IL) = 0.0
               ELSE IF(ABS(W(IR,IC,IL)).LT.(W(IR,IC,IL)*W(IR,IC,IL)*
     .                    DT/(D(IR,IC)*DZZ(IL-1))))   THEN
                  ANTIQZ(IR,IC,IL) = 0.0
               ELSE
                 ANTIQZ(IR,IC,IL) = ABS(W(IR,IC,IL))*
     .                (1.-ABS(W(IR,IC,IL))*DT/(D(IR,IC)*DZZ(IL-1)))*
     .                  (CONCS(IR,IC,IL-1)-CONCS(IR,IC,IL))/
     .                       (CONCS(IR,IC,IL-1)+CONCS(IR,IC,IL)+R)
               ENDIF
            ENDIF

 180  CONTINUE


c$doacross local(il,ic,ir,is,ie)
      DO 210 IL=1,NLAYR
        DO 210 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 210 IR=IS,IE

            IF(FSM(IR,IC).LE.0.) GO TO 210

            IF(FSM(IR,IC).EQ.1 .AND. FSM(IR-1,IC).EQ.1.) 
     .        QXFLX(IR,IC,IL) = 0.5*(ANTIQX(IR,IC,IL)*(CONCS(IR,IC,IL)+
     .           CONCS(IR-1,IC,IL))-ABS(ANTIQX(IR,IC,IL))*
     .           (CONCS(IR,IC,IL)-CONCS(IR-1,IC,IL)))
     .           *DYBAR1(IR,IC)*0.5*(D(IR,IC)+D(IR-1,IC))*DZ(IL)

            IF(FSM(IR,IC).EQ.1 .AND. FSM(IR,IC-1).EQ.1.) 
     .        QYFLX(IR,IC,IL) = 0.5*(ANTIQY(IR,IC,IL)*(CONCS(IR,IC,IL)+
     .          CONCS(IR,IC-1,IL))-ABS(ANTIQY(IR,IC,IL))*
     .          (CONCS(IR,IC,IL)-CONCS(IR,IC-1,IL)))      
     .           *DXBAR2(IR,IC)*0.5*(D(IR,IC)+D(IR,IC-1))*DZ(IL)

            IF(NLAYR.GT.1.AND.IL.GT.1.AND.FSM(IR,IC).GT.0.) 
     .        QZFLX(IR,IC,IL) = 0.5*
     .        (ANTIQZ(IR,IC,IL)*(CONCS(IR,IC,IL-1)+CONCS(IR,IC,IL))-
     .          ABS(ANTIQZ(IR,IC,IL))*(CONCS(IR,IC,IL-1)-
     .          CONCS(IR,IC,IL)))*XAZ(IR,IC)


 210  CONTINUE

 
C        TAKE CORRECTION STEP, PLACING RESULTS
C          BACK INTO CONCS NOW AT '(N+1)'
   
        IF (NLAYR.GT.1) THEN
c$doacross local(il,ic,ir,is,ie)
          DO 230 IL=1,NLAYR-1
            DO 230 IC=2,NCOL-1
              IS = IRS(IC)
              IE = IRE(IC)
              DO 230 IR=IS,IE
                IF(FSM(IR,IC).LE.0.)  GO TO 230
                CONCS(IR,IC,IL)=CONCS(IR,IC,IL)
     .             -DT*(QXFLX(IR+1,IC,IL)-QXFLX(IR,IC,IL)
     .                 +QYFLX(IR,IC+1,IL)-QYFLX(IR,IC,IL)
     .                 +QZFLX(IR,IC,IL)-QZFLX(IR,IC,IL+1))/
     .                                             BVOLN(IR,IC,IL)

 230      CONTINUE

          IL=NLAYR
c$doacross local(ic,ir,is,ie)
          DO 235 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 235 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 235
              CONCS(IR,IC,IL)=CONCS(IR,IC,IL)
     .           -DT*(QXFLX(IR+1,IC,IL)-QXFLX(IR,IC,IL)
     .               +QYFLX(IR,IC+1,IL)-QYFLX(IR,IC,IL)
     .               +QZFLX(IR,IC,IL))/BVOLN(IR,IC,IL)
 235      CONTINUE
        ELSE
          IL=1
          DO 240 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 240 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 240
              CONCS(IR,IC,IL)=CONCS(IR,IC,IL)
     .           -DT*(QXFLX(IR+1,IC,IL)-QXFLX(IR,IC,IL)
     .               +QYFLX(IR,IC+1,IL)-QYFLX(IR,IC,IL))/
     .                                             BVOLN(IR,IC,IL)
 240      CONTINUE
        ENDIF


C  CALCULATE HORIZONTAL DIFFUSION FLUXES
 
c$doacross local(il,ic,ir,is,ie)
        DO 250 IL=1,NLAYR
          DO 250 IC=2,NCOL
            DO 250 IR=2,NROW
              IF(FSM(IR,IC).GE.1 .OR. FSM(IR,IC).LE.-2)THEN
                IF(FSM(IR-1,IC).GE.1 .OR. FSM(IR-1,IC).LE.-2)THEN
                  RXFLX(IR,IC,IL)=RX(IR,IC,IL)*
     .             (CARAY(IR-1,IC,IL,ISYS)-CARAY(IR,IC,IL,ISYS))
                ENDIF
                IF(FSM(IR,IC-1).GE.1. OR. FSM(IR,IC-1).LE.-2)THEN
                  RYFLX(IR,IC,IL)=RY(IR,IC,IL)*
     .             (CARAY(IR,IC-1,IL,ISYS)-CARAY(IR,IC,IL,ISYS))
                ENDIF
              ENDIF
 250    CONTINUE
 

 
C  CALCULATE CONCENTRATIONS: HORIZONTAL DIFFUSION
 
c$doacross local(il,ic,ir,is,ie)
        DO 260 IL=1,NLAYR
          DO 260 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 260 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 260
              CONCS(IR,IC,IL)=CONCS(IR,IC,IL)+DT*
     .           (RXFLX(IR,IC,IL)-RXFLX(IR+1,IC,IL)+RYFLX(IR,IC,IL)
     .           -RYFLX(IR,IC+1,IL))/BVOLN(IR,IC,IL)
 260    CONTINUE

 
C  END OF SMOLARKIEWICZ METHOD
 
C**********************************************************************
 

C        COMPLETE REST OF INTEGRATION STEP

         IF (NLAYR.EQ.1)  GO TO 380
 
C  COMPLETE REST OF INTEGRATION STEP
C  I.E., THE IMPLICIT VERTICAL DIFFUSION STEP
 
c$doacross local(il,ic,ir,is,ie)
        DO 340 IL=1,NLAYR
         DO 340 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 340 IR=IS,IE
            MF(IR,IC,IL) = CONCS(IR,IC,IL)*BVOLN(IR,IC,IL)
 340    CONTINUE

C        SET UP DIAGONAL AND OFF-DIAGONAL ELEMENTS

c$doacross local(il,ic,ir,is,ie)
c       DO 350 IC=1,NCOL
        DO 350 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 350 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 350
          AL(IR,IC,1) = 0.
          AD(IR,IC,1) = BVOLN(IR,IC,1) + DT*RZ(IR,IC,2)
          AU(IR,IC,1) = -DT*RZ(IR,IC,2)
           DO 345 IL=2,NLAYR-1
            AL(IR,IC,IL) = -DT*RZ(IR,IC,IL)
            AD(IR,IC,IL) = BVOLN(IR,IC,IL) +
     .                        DT*(RZ(IR,IC,IL) + RZ(IR,IC,IL+1))
            AU(IR,IC,IL) = -DT*RZ(IR,IC,IL+1)
  345      CONTINUE
          AL(IR,IC,NLAYR) = -DT*RZ(IR,IC,NLAYR)
          AD(IR,IC,NLAYR) = BVOLN(IR,IC,NLAYR) +
     .                        DT*RZ(IR,IC,NLAYR)
          AU(IR,IC,NLAYR) = 0.0
  350   CONTINUE

C        FORWARD SWEEP OF TRIDIAGONAL SCHEME

c$doacross local(il,ic,ir,is,ie)
        DO 360 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 360 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 360
          BETA(IR,IC,1) = AD(IR,IC,1)
          GAMMA(IR,IC,1) = MF(IR,IC,1)/BETA(IR,IC,1)
          DO 355 IL=2,NLAYR
           BETA(IR,IC,IL) = AD(IR,IC,IL)
     .                 - AL(IR,IC,IL)*AU(IR,IC,IL-1)/BETA(IR,IC,IL-1)
           GAMMA(IR,IC,IL) = (MF(IR,IC,IL)
     .                 - AL(IR,IC,IL)*GAMMA(IR,IC,IL-1))/BETA(IR,IC,IL)
  355     CONTINUE
  360   CONTINUE

C        BACKWARD SWEEP FOR SOLUTION

c$doacross local(il,ic,ir,is,ie,ilyr)
        DO 370 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 370 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 370
          CONCS(IR,IC,NLAYR) = GAMMA(IR,IC,NLAYR)
          DO 365 ILYR=2,NLAYR
           IL = NLAYR+1-ILYR
           CONCS(IR,IC,IL) = GAMMA(IR,IC,IL)
     .             - AU(IR,IC,IL)*CONCS(IR,IC,IL+1)/BETA(IR,IC,IL)
  365     CONTINUE
  370   CONTINUE

 

C  BOUNDARY ELEMENTS ARE UNCHANGED

        DO 375 I=1,NOBCALL
          IR=IBCALL(1,I)
          IC=IBCALL(2,I)
          IL=IBCALL(3,I)
          CONCS(IR,IC,IL)=CARAY(IR,IC,IL,ISYS)
  375   CONTINUE
 

  380   CONTINUE

C  CHECK TO SEE IF NEGATIVE SOLUTIONS PERMITTED 

c$doacross local(il,ic,ir,is,ie,inegs,totmas) , share(negseg)
        DO 390 IL=1,NLAYR
          DO 390 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 390 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 390
 
              IF (NEGSLN.EQ.1)   THEN
                CARAY(IR,IC,IL,ISYS) = CONCS(IR,IC,IL)
 
              ELSE
 
C        IF NOT ... CHECK TO MAKE SURE SOLUTION IS NOT FORCED THRU ZERO
 
                IF(CONCS(IR,IC,IL).GE.0.0)   THEN
                  CARAY(IR,IC,IL,ISYS) = CONCS(IR,IC,IL)
                ELSE
C
C          PREVENT NEG SOLN BY QUARTERING PRESENT CONC AND PRINT WARNING
C
                  CARAY(IR,IC,IL,ISYS) = CARAY(IR,IC,IL,ISYS)/4.
                  INEGS = INEGS+1
                  IF(INEGS.LE.99)  THEN
                    NEGSEG(INEGS,1) = IR
                    NEGSEG(INEGS,2) = IC
                    NEGSEG(INEGS,3) = IL
                  ENDIF
                  TOTMAS = TOTMAS + (CARAY(IR,IC,IL,ISYS)-
     .                                 CONCS(IR,IC,IL))*BVOLN(IR,IC,IL)
                ENDIF
              ENDIF
 390    CONTINUE


        IF(INEGS.GT.0)  THEN
          CALL RCAMESS(2,0.001*TOTMAS)
          WRITE(OUT,1000)  ((NEGSEG(I,J),J=1,3),I=1,INEGS)
 1000     FORMAT(30X,12I5)
        ENDIF

  400 CONTINUE
  410 CONTINUE

C  PREPARE FOR NEXT TIME STEP
 
        TIME=TIME + DTSTEP
 
c$doacross local(il,ic,ir)
        DO 420 IL=1,NLAYR
          DO 420 IC=1,NCOL
            DO 420 IR=1,NROW
              BVOL(IR,IC,IL)=BVOLN(IR,IC,IL)
 420    CONTINUE
 
C        CHECK TO SEE IF IT IS TIME TO PRINT RESULTS
 
C  FIRST DETAILED DUMPS
       
      IF(PRNTD.EQ.0.0)  GO TO 435
      EPSLNP = DTSTEP/(PRNTD*2.)
      M=TIME/PRNTD+EPSLNP
      IF(M.LT.NND)  GO TO 435
 
C  DETAILED PRINT TIME
C  SET IDISK FOR DETAILED DUMPS
      
      IDISK=2
      NND=M+1
 
C  SECOND GLOBAL DUMPS
 
 435  EPSLNP=DTSTEP/(PRNTG*2.)
      M=TIME/PRNTG+EPSLNP
      IF (M.LT.NNG) GOTO 450
 
C  GLOBAL PRINT TIME
C  CALL PRINT ROUTINE
 
      CALL RCA09
      IDISK=IDISK+1
      NNG=M+1 

C  NOW PERFORM STABILITY CHECK

      DO 445 ISYS=1,NOSYS 
        IF(SYSBY(ISYS).EQ.1)   GO TO 445
        CKMAX = CMAX(ISYS)

        DO 440 IL=1,NLAYR
          DO 440 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 440 IR=IS,IE
              IF(FSM(IR,IC).LE.0)  GO TO 440
              IF(CARAY(IR,IC,IL,ISYS).GE.CKMAX)   GO TO 500
  440   CONTINUE
  445 CONTINUE
 
C        CHECK TO SEE IF NECESSARY TO UPDATE TIME-VARIABLE FUNCTIONS
 
C        FIRST POINT SOURCE LOADS 
  450 IF(IPSOPT.GT.1.AND.TIME.GE.NXPST)   
     .   CALL RCA10(BPS,NWK,NOPS,NXPST,22,SCALPS)
 
C        NONPOINT SOURCE LOADS 
      IF(INPSOPT.GT.1.AND.TIME.GE.NXNPST)   
     .   CALL RCA10(BNPS,NWK,NONPS,NXNPST,23,SCALNPS)

C        FALL-LINE LOADS 
      IF(IFLOPT.GT.1.AND.TIME.GE.NXFLT)   
     .   CALL RCA10(BFL,NWK,NOFL,NXFLT,24,SCALFL)

C        ATMOSPHERIC LOADS 
      IF(IATMOPT.GT.1.AND.TIME.GE.NXATMT)   
     .   CALL RCA10(BATM,1,NOATM,NXATMT,25,SCALATM)

C        NEXT BOUNDARY CONDITIONS 
      IF(IBCOPT.GT.1.AND.TIME.GE.NXBCT) CALL RCA11 

C        NEXT TRANSPORT FIELDS
      IF(TIME.GE.HYDBRK(ITIMHYD))  THEN
         CALL RCA03A
         CALL RCAEXP1
         ITIMHYD = ITIMHYD+1
      ENDIF
 

C        CHECK FOR END OF SIMULATION...
C           IF NOT GO BACK AND TAKE NEXT INTEGRATION STEP

      IF(TIME.LE.TEND)   GO TO 40 

      ITRAK = ITRAK+1 
      IF(ITRAK.LE.NSTEP)  THEN
         DT=STEP(ITRAK)
         TEND = TBRK(ITRAK) 
         IF(DT.EQ.0.0)   THEN
            CALL RCAMESS(6,DT)
            CALL EXIT
         ENDIF
      ELSE
         TEND = 0.0
      ENDIF 

      DTSTEP = DT 
      DT = SCALT*DT 
      IF(TEND.GT.0.)   GO TO 40 
 
C        FINISHED...CALL FOR FINAL DUMPS
 
      IDISK = 1 
      CALL TUNER
      RETURN

C         STABILITY CRITERIA VIOLATED...ABEND 

  500 CALL RCAMESS(1,CKMAX) 
      WRITE(OUT,9000)  BVOLN(IR,IC,IL),DIAG(IR,IC,IL)
 9000 FORMAT(10X,'VOLUME =',E13.4,'  ,DIAGONAL =',E13.4)
      IREC = IREC+1 
      IDISK = 1 
      CALL TUNER
      RETURN
      END 
