      SUBROUTINE  RCAIMP
C 
C        RCAIMP IS THE MASTER ROUTINE USED TO INTEGRATE THE DIFFERENTIAL
C               EQUATIONS OF THE WATER QUALITY MODEL.
C               TIME DERIVATIVE: IMPLICIT - GAUSS SEIDEL ITERATION
C               SPACE DERIVATIVE: UPWIND
C 
      INCLUDE  'RCACM'
      INTEGER  NEGSEG(99,3)
      REAL   CONCS(NROW,NCOL,0:NLAYR+1)
     .      ,CSTAR(NROW),FORCFN(NROW,NCOL,NLAYR),BVOLN(NROW,NCOL,NLAYR)
      LOGICAL   CHANGE
  
C        INITIAL SETUP

      ITRAK = 1 
      TIME = 0.0
      TIMEWQ = 0.0
      DTSTEP = DT
      DT = DT*SCALT 
      DTWQ = DTWQ*SCALT 
      INITB = 0
      NNG = 1 
      NND = 1 
      IF(TZERO.NE.0.0)  THEN
         CALL RCA08
         TIME = TZERO
         NNG = TIME/PRNTG + 0.5
         IF(PRNTD.NE.0.0)  NND = TIME/PRNTD + 0.5
      ENDIF

      DO 5 IL=0,NLAYR+1
      DO 5 IC=1,NCOL
      DO 5 IR=1,NROW
    5 CONCS(IR,IC,IL) = 0.0

      DO 7 IL=1,NLAYR
      DO 7 IC=1,NCOL
      DO 7 IR=1,NROW
    7 FORCFN(IR,IC,IL) = 0.0

C        SET UP -AVECT- ARRAYS FOR TIME = 0.0 (TZERO) 

      CALL RCAIMP1

C        PRINT INITIAL CONDITIONS 

      CALL RCA09
      IDISK = 3 

C        EVALUATE DERIVATIVES AT TIME LEVEL "N"

   10 IF(DTWQ.EQ.0.0 .OR. TIME.GE.TIMEWQ)   THEN
         CALL TUNER
         TIMEWQ = TIMEWQ+DTWQ
      ENDIF
      INITB = 1 
      IDISK = 0 
  

C             IMPLICIT STEP 

 
C        COMPUTE VOLUMES AT TIME LEVEL "N+1"
      DO 15 IL=1,NLAYR
      DO 15 IC=1,NCOL
      DO 15 IR=1,NROW
   15 BVOLN(IR,IC,IL) = BVOL(IR,IC,IL) + DT*VDER(IR,IC,IL)


      DO 100 ISYS=1,NOSYS 

C        CHECK FOR SYSTEM BYPASS
      IF(SYSBY(ISYS).EQ.1)  GO TO 100 

      INEGS = 0
      TOTMAS = 0.0

C        FORMULATE RHS (FORCING FUNCTION) OF 'MATRIX' 
C           RHS = CONC*BVOL + DT*SCALT*(CDER+BCLOAD+LOADS)
C        ALSO SET INITIAL ESTIMATE OF C AT N+1 TO C AT N

      DO 20 IL=1,NLAYR
      DO 20 IC=1,NCOL
      DO 20 IR=1,NROW
   20 CONCS(IR,IC,IL) = CARAY(IR,IC,IL,ISYS)

      DO 25 IL=1,NLAYR
      DO 25 IC=2,NCOL-1
      IS = IRS(IC)
      IE = IRE(IC)
      DO 25 IR=IS,IE
   25 FORCFN(IR,IC,IL) = CARAY(IR,IC,IL,ISYS)*BVOL(IR,IC,IL)
     .                                  + DT*CDARAY(IR,IC,IL,ISYS)

      IF(NOPS(ISYS).GT.0)  THEN
         DO 30 I=1,NOPS(ISYS) 
         IR = IPS(1,I,ISYS)
         IC = IPS(2,I,ISYS)
         IL = IPS(3,I,ISYS)
         FORCFN(IR,IC,IL) = FORCFN(IR,IC,IL) + DT*BPS(I,ISYS)
   30    CONTINUE
      ENDIF

      IF(NONPS(ISYS).GT.0)   THEN
         DO 35 I=1,NONPS(ISYS)
         IR = INPS(1,I,ISYS)
         IC = INPS(2,I,ISYS)
         IL = INPS(3,I,ISYS)
         FORCFN(IR,IC,IL) = FORCFN(IR,IC,IL) + DT*BNPS(I,ISYS)
   35    CONTINUE
      ENDIF

      IF(NOFL(ISYS).GT.0)   THEN
         DO 40 I=1,NOFL(ISYS)
         IR = IFL(1,I,ISYS)
         IC = IFL(2,I,ISYS)
         IL = IFL(3,I,ISYS)
         FORCFN(IR,IC,IL) = FORCFN(IR,IC,IL) + DT*BFL(I,ISYS)
   40    CONTINUE
      ENDIF

      IF(NOATM(ISYS).GT.0)   THEN
         DO 45 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 45 IR=IS,IE
         IF(FSM(IR,IC).EQ.1.)   FORCFN(IR,IC,1) = FORCFN(IR,IC,1) +
     .                                     DT*BATM(ISYS)*XAZ(IR,IC)
   45    CONTINUE
      ENDIF


C        ITERATE USING GAUSS-SEIDEL METHOD

      ITER = 0
      EPSLN = CMIN(ISYS)
   55 ITER = ITER + 1 

 
      IF (NLAYR.GT.1) THEN
      DO 75 IL=1,NLAYR
      DO 75 IC=2,NCOL-1
      IS = IRS(IC)
      IE = IRE(IC)
      DO 65 IR=IS,IE
      TERM = AVECT(IR,IC,IL,1)*CONCS(IR-1,IC,IL) +
     .        AVECT(IR,IC,IL,2)*CONCS(IR+1,IC,IL) + 
     .         AVECT(IR,IC,IL,3)*CONCS(IR,IC-1,IL) + 
     .          AVECT(IR,IC,IL,4)*CONCS(IR,IC+1,IL) + 
     .           AVECT(IR,IC,IL,5)*CONCS(IR,IC,IL-1) + 
     .            AVECT(IR,IC,IL,6)*CONCS(IR,IC,IL+1)

      CSTAR(IR) = (DT*TERM + FORCFN(IR,IC,IL))
     .                  /(BVOLN(IR,IC,IL) + DT*DIAG(IR,IC,IL)) 
       
 65   CONTINUE

      DO 70 IR=IS,IE
   70 CONCS(IR,IC,IL) = CSTAR(IR)
 
   75 CONTINUE
      ELSE
      IL=1
      DO 76 IC=2,NCOL-1
      IS = IRS(IC)
      IE = IRE(IC)
      DO 66 IR=IS,IE
      TERM = AVECT(IR,IC,IL,1)*CONCS(IR-1,IC,IL) +
     .        AVECT(IR,IC,IL,2)*CONCS(IR+1,IC,IL) + 
     .         AVECT(IR,IC,IL,3)*CONCS(IR,IC-1,IL) + 
     .          AVECT(IR,IC,IL,4)*CONCS(IR,IC+1,IL)

      CSTAR(IR) = (DT*TERM + FORCFN(IR,IC,IL))
     .                  /(BVOLN(IR,IC,IL) + DT*DIAG(IR,IC,IL)) 
 66   CONTINUE

      DO 71 IR=IS,IE
   71 CONCS(IR,IC,IL) = CSTAR(IR)
 
   76 CONTINUE
      ENDIF


      IF(MOD(ITER,5).NE.0)  GO TO 55
C        CHECK CONVERGENCE
      DO 85 IL=1,NLAYR
      DO 85 IC=2,NCOL-1
      IS = IRS(IC)
      IE = IRE(IC)
      DO 85 IR=IS,IE

C        INITIALIZE CHECKSUM

      CHECKSUM = (BVOLN(IR,IC,IL) + DT*DIAG(IR,IC,IL))
     .              * CONCS(IR,IC,IL)
      IF (NLAYR.GT.1) THEN
      CHECKSUM =
     .   CHECKSUM - DT * (AVECT(IR,IC,IL,1)*CONCS(IR-1,IC,IL) +
     .                     AVECT(IR,IC,IL,2)*CONCS(IR+1,IC,IL) + 
     .                      AVECT(IR,IC,IL,3)*CONCS(IR,IC-1,IL) + 
     .                       AVECT(IR,IC,IL,4)*CONCS(IR,IC+1,IL) + 
     .                        AVECT(IR,IC,IL,5)*CONCS(IR,IC,IL-1) + 
     .                         AVECT(IR,IC,IL,6)*CONCS(IR,IC,IL+1))
      ELSE
      CHECKSUM =
     .   CHECKSUM - DT * (AVECT(IR,IC,IL,1)*CONCS(IR-1,IC,IL) +
     .                     AVECT(IR,IC,IL,2)*CONCS(IR+1,IC,IL) + 
     .                      AVECT(IR,IC,IL,3)*CONCS(IR,IC-1,IL) + 
     .                       AVECT(IR,IC,IL,4)*CONCS(IR,IC+1,IL))
      ENDIF

C        CHECK DIFFERENCE IN MASS BALANCE AGAINST EPSILON 
c     IF(ABS(FORCFN(IR,IC,IL)/BVOL(IR,IC,IL)
c    .      -CHECKSUM/BVOLN(IR,IC,IL)).GT.EPSLN)  THEN
      IF(ABS((FORCFN(IR,IC,IL)-CHECKSUM)/(FORCFN(IR,IC,IL)+0.001))
     .       .GT.EPSLN)  THEN

C           UNSUCCESSFUL...

C              IF  -ITER- < 75, BRANCH FOR ANOTHER ITERATION
         IF(ITER.LT.75)  GO TO 55 

C              ELSE IF  -ITER- = 75, BRANCH TO ERROR MESSAGE ROUTINE
         GO TO 205

      ENDIF

   85 CONTINUE


C        CONVERGENCE SUCCESSFUL...
C           MOVE SOLUTION VECTOR INTO -CARAY-
      DO 95 IL=1,NLAYR
      DO 95 IC=2,NCOL-1
      IS = IRS(IC)
      IE = IRE(IC)
      DO 95 IR=IS,IE

      IF(FSM(IR,IC).NE.1)  GO TO 95
      CONC = CONCS(IR,IC,IL)

C        CHECK TO SEE IF NEGATIVE SOLUTIONS PERMITTED
      IF(NEGSLN.EQ.0)  THEN
C        IF NOT, CHECK TO MAKE SURE SOLUTION NOT FORCED THRU ZERO
         IF(CONC.LT.0.0)   THEN
            CONC = CARAY(IR,IC,IL,ISYS)/4.
            INEGS = INEGS+1
            IF(INEGS.LE.99)  THEN
               NEGSEG(INEGS,1) = IR
               NEGSEG(INEGS,2) = IC
               NEGSEG(INEGS,3) = IL
            ENDIF
            TOTMAS = TOTMAS + (CONC-CONCS(IR,IC,IL))*BVOLN(IR,IC,IL)
         ENDIF
      ENDIF

      CARAY(IR,IC,IL,ISYS) = AMAX1(CONC,1.0E-37)

   95 CONTINUE

      IF(INEGS.GT.0)   THEN
         CALL RCAMESS(2,0.001*TOTMAS)
         IF(INEGS.GT.99)  WRITE(OUT,3099)
 3099    FORMAT(30X,'WARNING!!!  MORE THAN 99 SEGMENTS WERE COMPUTED TO'
     .    ,' HAVE NEGATIVE CONCENTRATIONS'/30X,'ONLY FIRST 99 PRINTED')
         WRITE(OUT,1000)   ((NEGSEG(I,J),J=1,3),I=1,99)
 1000    FORMAT(10X,10(3I3,2X))
      ENDIF

  100 CONTINUE


      DO 110 IL=1,NLAYR
      DO 110 IC=1,NCOL
      DO 110 IR=1,NROW
  110 BVOL(IR,IC,IL) = BVOLN(IR,IC,IL)

  
C             STEP COMPLETE 
  
      TIME = TIME + DTSTEP

C        CHECK TO SEE IF IT IS TIME TO PRINT RESULTS

C        FIRST DETAILED DUMPS
      IF(PRNTD.EQ.0.0)  GO TO 115
      EPSLNP = DTSTEP/(PRNTD*2.) 
      M = TIME/PRNTD + EPSLNP
      IF(M.LT.NND)   GO TO 115
C        DETAILED PRINT TIME...
C        SET IDISK FOR DETAILED DUMPS
      IDISK = 2
      NND = M+1

C        SECOND GLOBAL DUMPS
 115  EPSLNP = DTSTEP/(PRNTG*2.) 
      M = TIME/PRNTG + EPSLNP
      IF(M.LT.NNG)   GO TO 130 
C        GLOBAL PRINT TIME...
C        CALL PRINT ROUTINE
      CALL RCA09
      IDISK = IDISK+1
      NNG = M+1

C          NOW PERFORM STABILITY CHECK
      DO 125 ISYS=1,NOSYS 
      IF(SYSBY(ISYS).EQ.1)   GO TO 125
      CKMAX = CMAX(ISYS)
         DO 120 IL=1,NLAYR
         DO 120 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 120 IR=IS,IE
         IF(CARAY(IR,IC,IL,ISYS).GE.CKMAX)   GO TO 200
  120    CONTINUE
  125 CONTINUE


C        CHECK TO SEE IF NECESSARY TO UPDATE TIME VARIABLE FUNCTIONS

C        FIRST POINT SOURCE LOADS 
  130 IF(IPSOPT.GT.1 .AND. TIME.GE.NXPST)   
     .   CALL RCA10(BPS,NWK,NOPS,NXPST,22,SCALPS)

C        NONPOINT SOURCE LOADS 
      IF(INPSOPT.GT.1 .AND. TIME.GE.NXNPST)   
     .   CALL RCA10(BNPS,NWK,NONPS,NXNPST,23,SCALNPS)

C        FALL-LINE LOADS 
      IF(IFLOPT.GT.1 .AND. TIME.GE.NXFLT)   
     .   CALL RCA10(BFL,NWK,NOFL,NXFLT,24,SCALFL)

C        ATMOSPHERIC LOADS 
      IF(IATMOPT.GT.1 .AND. TIME.GE.NXATMT)   
     .   CALL RCA10(BATM,1,NOATM,NXATMT,25,SCALATM)

C        NEXT BOUNDARY CONDITIONS 
      IF(IBCOPT.GT.1 .AND. TIME.GT.NXBCT)  CALL RCA11

C        NEXT TRANSPORT FIELDS
      IF(TIME.GE.HYDBRK(ITIMHYD))  THEN
         CALL RCA03A
         CALL RCAIMP1
         ITIMHYD = ITIMHYD+1
      ENDIF


C        CHECK FOR END OF SIMULATION...
C           IF NOT GO BACK AND TAKE NEXT INTEGRATION STEP
      IF(TIME.LE.TEND)   GO TO 10 

      ITRAK = ITRAK+1 
      IF(ITRAK.LE.NSTEP)  THEN
         DT=STEP(ITRAK)
         TEND = TBRK(ITRAK) 
         IF(DT.EQ.0.0)   THEN
            CALL RCAMESS(6,DT)
            CALL EXIT
         ENDIF
      ELSE
         TEND = 0.0
      ENDIF 

      DTSTEP = DT 
      DT = SCALT*DT 
      IF(TEND.GT.0.)   GO TO 10 

C        FINISHED...CALL FOR FINAL DUMPS
      IDISK = 1 
      CALL TUNER
      RETURN


C         STABILITY CRITERIA VIOLATED...ABEND 
  200 CALL RCAMESS(1,CKMAX) 
      GO TO 210
C        CONVERGENCE FAILURE
  205 CALL RCAMESS(3,TIME)
  210 PRINT *,'    FORCFN         BVOL         CHECKSUM         BVOLN'
      PRINT *, FORCFN(IR,IC,IL),BVOL(IR,IC,IL),CHECKSUM,BVOLN(IR,IC,IL)
      PRINT *,'     CONCS         CARAY        EPSLN'
      PRINT *, CONCS(IR,IC,IL),CARAY(IR,IC,IL,ISYS),EPSLN

      PRINT *,' '
      CHECKSUM = (BVOLN(IR,IC,IL) + DT*DIAG(IR,IC,IL))
     .                *CONCS(IR,IC,IL)
      PRINT *,'   CHECKSUM         DT       DIAG       CONCS'
      PRINT *,  CHECKSUM,DT,DIAG(IR,IC,IL),CONCS(IR,IC,IL)

      PRINT *,' '
      CHECKSUM = 
     .   CHECKSUM - DT * (AVECT(IR,IC,IL,1)*CONCS(IR-1,IC,IL) + 
     .                     AVECT(IR,IC,IL,2)*CONCS(IR+1,IC,IL) +
     .                      AVECT(IR,IC,IL,3)*CONCS(IR,IC-1,IL) +
     .                       AVECT(IR,IC,IL,4)*CONCS(IR,IC+1,IL) +
     .                        AVECT(IR,IC,IL,5)*CONCS(IR,IC,IL-1) +
     .                         AVECT(IR,IC,IL,6)*CONCS(IR,IC,IL+1))
      PRINT *,'    CHECKSUM      AVECT(IR-1)     CONCS'
      PRINT *, CHECKSUM,AVECT(IR,IC,IL,1),CONCS(IR-1,IC,IL) 
      PRINT *,'  AVECT(IR+1)     CONCS'
      PRINT *, AVECT(IR,IC,IL,2),CONCS(IR+1,IC,IL) 
      PRINT *,'  AVECT(IC-1)     CONCS'
      PRINT *, AVECT(IR,IC,IL,3),CONCS(IR,IC-1,IL) 
      PRINT *,'  AVECT(IC+1)     CONCS'
      PRINT *, AVECT(IR,IC,IL,4),CONCS(IR,IC+1,IL) 
      PRINT *,'  AVECT(IL-1)     CONCS'
      PRINT *, AVECT(IR,IC,IL,5),CONCS(IR,IC,IL-1) 
      PRINT *,'  AVECT(IL+1)     CONCS'
      PRINT *, AVECT(IR,IC,IL,6),CONCS(IR,IC,IL+1) 

      IF(IREC.EQ.1)   CALL EXIT 
      IREC = IREC+1 
      IDISK = 1 
      CALL TUNER
      RETURN
      END 
