      SUBROUTINE RCALF
C 
C        RCALF IS THE MASTER ROUTINE USED TO INTEGRATE THE DIFFERENTIAL
C              EQUATIONS OF THE WATER QUALITY MODEL.
C              TIME DERIVATIVE: EXPLICIT - LEAP-FROG
C              SPACE DERIVATIVE: CENTRAL
C
  
      INCLUDE 'RCACM' 
      INTEGER   NEGSEG(99,3)
      REAL   C(NROW,NCOL,NLAYR,NOSYS)
     .    ,CB(NROW,NCOL,NLAYR,NOSYS),CF(NROW,NCOL,NLAYR,NOSYS)
     .    ,DM(NROW,NCOL,NLAYR),MF(NROW,NCOL,NLAYR)
     .    ,BVOLB(NROW,NCOL,NLAYR),BVOLF(NROW,NCOL,NLAYR),TOTMAS
     .    ,AL(NROW,NCOL,NLAYR),AD(NROW,NCOL,NLAYR),AU(NROW,NCOL,NLAYR)
     .    ,BETA(NROW,NCOL,NLAYR),GAMMA(NROW,NCOL,NLAYR)
     .    ,BBCALL(NBC,NOSYS)
 
      EQUIVALENCE
     .       (MF(1,1,1),SCRATCH_TRAN(1,1,1,2))
     .    ,  (AL(1,1,1),SCRATCH_TRAN(1,1,1,3))
     .    ,  (AD(1,1,1),SCRATCH_TRAN(1,1,1,4))
     .    ,  (AU(1,1,1),SCRATCH_TRAN(1,1,1,5))
     .    ,  (BETA(1,1,1),SCRATCH_TRAN(1,1,1,6))
     .    ,  (GAMMA(1,1,1),SCRATCH_TRAN(1,1,1,7))
 
      EQUIVALENCE
     .       (DM(1,1,1),SCRATCH_TRAN(1,1,1,8))
     .    ,  (BVOLB(1,1,1),SCRATCH_TRAN(1,1,1,9))
     .    ,  (BVOLF(1,1,1),SCRATCH_TRAN(1,1,1,10))
 
 
C  INITIALIZATION

      ITRAK=1 
      TIME=0.0 
      TIMEWQ=0.0 
      INITB=0
      DTSTEP = DT
      DT = DT*SCALT
      DTWQ=DTWQ*SCALT
      NNG=1
      NND=1
 
      IF(TZERO.NE.0.)   THEN
         CALL RCA08
         TIME = TZERO
         NNG=TIME/PRNTG+0.5
         IF (PRNTD.NE.0.0) NND=TIME/PRNTD+0.5
      ENDIF

C        PUT ECOM COMPUTED SALINITY AS INITIAL CONDITIONS
      DO IL=1,NLAYR
      DO IC=1,NCOL
      DO IR=1,NROW
         CARAY(IR,IC,IL,1) = 10.
      ENDDO
      ENDDO
      ENDDO


      DO 30 ISYS=1,NOSYS
       DO 30 IL=1,NLAYR
        DO 30 IC=1,NCOL
         DO 30 IR=1,NROW
           CB(IR,IC,IL,ISYS) = CARAY(IR,IC,IL,ISYS)
           C(IR,IC,IL,ISYS) = CARAY(IR,IC,IL,ISYS)
           CF(IR,IC,IL,ISYS) = 0.0
   30 CONTINUE

       DO 35 IL=1,NLAYR
        DO 35 IC=1,NCOL
         DO 35 IR=1,NROW
          BVOLB(IR,IC,IL) = BVOL(IR,IC,IL)
   35 CONTINUE

C        PRINT INITIAL CONDITIONS

      CALL RCA09
      IDISK = 3 
 

C  BEGIN LEAP-FROG STEP

   40 CONTINUE

C        PUT ECOM COMPUTED SALINITY AT BOUNDARY
      DO I=1,NOBC(1)
       IR = IBC(1,I,1)
       IC = IBC(2,I,1)
       IL = IBC(3,I,1)
       CARAY(IR,IC,IL,1) =  10.
       BBC(I,1) =  10.
      ENDDO 


C        IF PIECEWISE LINEAR BOUNDARY CONDITIONS THEN
C           GET CONCENTRATIONS AT CURRENT TIME LEVEL

      IF(IBCPLOPT.EQ.1)   THEN
        DO 50 ISYS=1,NOSYS
          IF(NOBC(ISYS).EQ.0)  GO TO 50
          DELTBC = TIME - NXBCT
          DO 45 I=1,NOBC(ISYS)
            IR=IBC(1,I,ISYS)
            IC=IBC(2,I,ISYS)
            IL=IBC(3,I,ISYS)
            C(IR,IC,IL,ISYS) = DELTBC*SBC(I,ISYS) + BBC(I,ISYS)
   45     CONTINUE
   50  CONTINUE
      ENDIF

      IF(IBCPLOPT.EQ.0)   THEN
        DO 55 ISYS=1,NOSYS
          IF(NOBC(ISYS).EQ.0)  GO TO 55
          DO 54 I=1,NOBC(ISYS)
            IR=IBC(1,I,ISYS)
            IC=IBC(2,I,ISYS)
            IL=IBC(3,I,ISYS)
            C(IR,IC,IL,ISYS) =  BBC(I,ISYS)
   54     CONTINUE
   55  CONTINUE
      ENDIF


C        EVALUATE KINETIC DERIVATIVES 

      IF (DTWQ.EQ.0.0.OR.TIME.GE.TIMEWQ.OR.IDISK.EQ.3) THEN
        DO 60 ISYS=1,NOSYS
         DO 60 IL=1,NLAYR
          DO 60 IC=2,NCOL-1
           IS = IRS(IC)
           IE = IRE(IC)
           DO 60 IR=IS,IE
             CARAY(IR,IC,IL,ISYS) = MAX(CB(IR,IC,IL,ISYS),CMIN(ISYS))
   60   CONTINUE
        CALL TUNER
        TIMEWQ=TIMEWQ+DTWQ
      ENDIF
      INITB = 1 
      IDISK = 0 
 
C        COMPUTE VOLUMES AT TIME LEVEL 'N+1'
 
      DO 70 IL=1,NLAYR
        DO 70 IC=1,NCOL
        DO 70 IR=1,NROW
            BVOLF(IR,IC,IL) = BVOLB(IR,IC,IL) + 2.*DT*VDER(IR,IC,IL)
   70 CONTINUE

      DO 320 ISYS=1,NOSYS 
 
C        CHECK FOR SYSTEM BYPASS
      IF(SYSBY(ISYS).EQ.1)   GO TO 320

        INEGS=0
        TOTMAS=0.0
 
        DO 100 IL=1,NLAYR
         DO 100 IC=1,NCOL
          DO 100 IR=1,NROW
           DM(IR,IC,IL) = 0.0
           MF(IR,IC,IL) = 0.0
  100   CONTINUE

C     HORIZONTAL AND VERTICAL ADVECTION AND HORIZONTAL DISPERSION
 
C        HORIZONTAL AND VERTICAL ADVECTION
C        - CENTRAL DIFFERENCED AT TIME LEVEL "N"
 
        DO 110 IL=1,NLAYR
         DO 110 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 110 IR=IS,IE
           DM(IR,IC,IL) = 0.5 *
     .         (QX(IR,IC,IL)*(C(IR-1,IC,IL,ISYS)+C(IR,IC,IL,ISYS)) -
     .          QX(IR+1,IC,IL)*(C(IR,IC,IL,ISYS)+C(IR+1,IC,IL,ISYS)))
  110   CONTINUE


        DO 120 IL=1,NLAYR
         DO 120 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 120 IR=IS,IE
           DM(IR,IC,IL) = DM(IR,IC,IL) + 0.5 * 
     .         (QY(IR,IC,IL)*(C(IR,IC-1,IL,ISYS)+C(IR,IC,IL,ISYS)) -
     .          QY(IR,IC+1,IL)*(C(IR,IC,IL,ISYS)+C(IR,IC+1,IL,ISYS)))
  120   CONTINUE
 
       IF(NLAYR.GT.1)  THEN

C        LAYER 1
         DO 125 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 125 IR=IS,IE
           DM(IR,IC,1) = DM(IR,IC,1) + 0.5*QZ(IR,IC,2)*
     .         (C(IR,IC,1,ISYS)+C(IR,IC,2,ISYS))
  125    CONTINUE
C        LAYERS 2 THRU NLAYR-1
         DO 130 IL=2,NLAYR-1
          DO 130 IC=2,NCOL-1
           IS = IRS(IC)
           IE = IRE(IC)
           DO 130 IR=IS,IE
            DM(IR,IC,IL) = DM(IR,IC,IL) - 0.5 * 
     .          (QZ(IR,IC,IL)*(C(IR,IC,IL-1,ISYS)+C(IR,IC,IL,ISYS)) -
     .           QZ(IR,IC,IL+1)*(C(IR,IC,IL,ISYS)+C(IR,IC,IL+1,ISYS)))
  130    CONTINUE
C        LAYER NLAYR
         DO 135 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 135 IR=IS,IE
           DM(IR,IC,NLAYR) = DM(IR,IC,NLAYR) - 0.5*QZ(IR,IC,NLAYR)*
     .         (C(IR,IC,NLAYR,ISYS)+C(IR,IC,NLAYR-1,ISYS))
  135    CONTINUE

       ENDIF
 
C        USE UPWIND DIFFERENCES FOR RIVER/WALL BOUNDARY FLUXES
 
        DO 140 I=1,NOBCALL
          IR=IBCALL(1,I)
          IC=IBCALL(2,I)
          IL=IBCALL(3,I)
          IF(IR.LT.NROW .AND. FSM(IR+1,IC).EQ.1)  THEN
             IF(QX(IR+1,IC,IL).GE.0.0)  THEN
                DM(IR+1,IC,IL) = DM(IR+1,IC,IL) + 0.5*
     .            QX(IR+1,IC,IL)*(C(IR,IC,IL,ISYS)-C(IR+1,IC,IL,ISYS))
             ELSE
                DM(IR+1,IC,IL) = DM(IR+1,IC,IL) + 0.5*
     .            QX(IR+1,IC,IL)*(C(IR+1,IC,IL,ISYS)-C(IR,IC,IL,ISYS))
             ENDIF
          ENDIF 
          IF(IR.GT.1 .AND. FSM(IR-1,IC).EQ.1)  THEN
             IF(QX(IR,IC,IL).GE.0.0)  THEN
                DM(IR-1,IC,IL) = DM(IR-1,IC,IL) - 0.5*
     .            QX(IR,IC,IL)*(C(IR-1,IC,IL,ISYS)-C(IR,IC,IL,ISYS))
             ELSE
                DM(IR-1,IC,IL) = DM(IR-1,IC,IL) - 0.5*
     .            QX(IR,IC,IL)*(C(IR,IC,IL,ISYS)-C(IR-1,IC,IL,ISYS))
             ENDIF
          ENDIF
          IF(IC.LT.NCOL .AND. FSM(IR,IC+1).EQ.1)  THEN
             IF(QY(IR,IC+1,IL).GE.0.0)  THEN
                DM(IR,IC+1,IL) = DM(IR,IC+1,IL) + 0.5*
     .            QY(IR,IC+1,IL)*(C(IR,IC,IL,ISYS)-C(IR,IC+1,IL,ISYS))
             ELSE
                DM(IR,IC+1,IL) = DM(IR,IC+1,IL) + 0.5*
     .            QY(IR,IC+1,IL)*(C(IR,IC+1,IL,ISYS)-C(IR,IC,IL,ISYS))
             ENDIF
          ENDIF
          IF(IC.GT.1 .AND. FSM(IR,IC-1).EQ.1)  THEN
             IF(QY(IR,IC,IL).GE.0.0)  THEN
                DM(IR,IC-1,IL) = DM(IR,IC-1,IL) - 0.5*
     .            QY(IR,IC,IL)*(C(IR,IC-1,IL,ISYS)-C(IR,IC,IL,ISYS))
             ELSE
                DM(IR,IC-1,IL) = DM(IR,IC-1,IL) - 0.5*
     .            QY(IR,IC,IL)*(C(IR,IC,IL,ISYS)-C(IR,IC-1,IL,ISYS))
             ENDIF
          ENDIF
 140    CONTINUE

C        HORIZONTAL DISPERSION
C        - AT TIME LEVEL "N-1"
 
        DO 150 IL=1,NLAYR
         DO 150 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 150 IR=IS,IE
           DM(IR,IC,IL) =  DM(IR,IC,IL) +
     .         RX(IR,IC,IL)*(CB(IR-1,IC,IL,ISYS)-CB(IR,IC,IL,ISYS)) +
     .         RX(IR+1,IC,IL)*(CB(IR+1,IC,IL,ISYS)-CB(IR,IC,IL,ISYS))
  150   CONTINUE

        DO 155 IL=1,NLAYR
         DO 155 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 155 IR=IS,IE
           DM(IR,IC,IL) =  DM(IR,IC,IL) +
     .         RY(IR,IC,IL)*(CB(IR,IC-1,IL,ISYS)-CB(IR,IC,IL,ISYS)) +
     .         RY(IR,IC+1,IL)*(CB(IR,IC+1,IL,ISYS)-CB(IR,IC,IL,ISYS))
  155   CONTINUE

C        ZERO BOUNDARY DERIVATIVES
 
        DO 160 I=1,NOBCALL
          IR=IBCALL(1,I)
          IC=IBCALL(2,I)
          IL=IBCALL(3,I)
          DM(IR,IC,IL) = 0.0
 160    CONTINUE

C        SOURCES/SINKS DUE TO KINETICS

        DO 170 IL=1,NLAYR
         DO 170 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
           DO 170 IR=IS,IE
             DM(IR,IC,IL) = DM(IR,IC,IL) + CDARAY(IR,IC,IL,ISYS)
 170    CONTINUE
 
C     ADD LOADS
 
C        POINT SOURCE
        IF(NOPS(ISYS).GT.0)  THEN
          DO 180 I=1,NOPS(ISYS) 
            IR = IPS(1,I,ISYS)
            IC = IPS(2,I,ISYS)
            IL = IPS(3,I,ISYS)
            DM(IR,IC,IL) = DM(IR,IC,IL) + BPS(I,ISYS)
  180     CONTINUE
        ENDIF

C        NONPOINT SOURCE
        IF(NONPS(ISYS).GT.0)   THEN
          DO 190 I=1,NONPS(ISYS)
            IR = INPS(1,I,ISYS)
            IC = INPS(2,I,ISYS)
            IL = INPS(3,I,ISYS)
            DM(IR,IC,IL) = DM(IR,IC,IL) + BNPS(I,ISYS)
  190     CONTINUE
        ENDIF

C        FALL-LINE
        IF(NOFL(ISYS).GT.0)   THEN
          DO 200 I=1,NOFL(ISYS)
            IR = IFL(1,I,ISYS)
            IC = IFL(2,I,ISYS)
            IL = IFL(3,I,ISYS)
            DM(IR,IC,IL) = DM(IR,IC,IL) + BFL(I,ISYS)
  200     CONTINUE
        ENDIF

C        ATMOSPHERIC
        IF(NOATM(ISYS).GT.0)   THEN
          DO 210 IC=2,NCOL-1
           IS = IRS(IC)
           IE = IRE(IC)
           DO 210 IR=IS,IE
             DM(IR,IC,1) = DM(IR,IC,1) + BATM(ISYS)*XAZ(IR,IC)
  210     CONTINUE
        ENDIF
 
C  TAKE LEAP-FROG STEP
 
        DO 220 IL=1,NLAYR
         DO 220 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 220 IR=IS,IE
            MF(IR,IC,IL) = CB(IR,IC,IL,ISYS)*BVOLB(IR,IC,IL) +
     .          2.*DT*DM(IR,IC,IL)
 220    CONTINUE


C  IMPLICIT VERTICAL DIFFUSION STEP  (IF NLAYR > 1)
 
        IF(NLAYR.EQ.1)  GO TO 255

C        SET UP DIAGONAL AND OFF-DIAGONAL ELEMENTS

        DO 230 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 230 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 230
          AL(IR,IC,1) = 0.
          AD(IR,IC,1) = BVOLF(IR,IC,1) + 2.*DT*RZ(IR,IC,2)
          AU(IR,IC,1) = -2.*DT*RZ(IR,IC,2)
           DO 225 IL=2,NLAYR-1
            AL(IR,IC,IL) = -2.*DT*RZ(IR,IC,IL)
            AD(IR,IC,IL) = BVOLF(IR,IC,IL) +
     .                        2.*DT*(RZ(IR,IC,IL) + RZ(IR,IC,IL+1))
            AU(IR,IC,IL) = -2.*DT*RZ(IR,IC,IL+1)
  225      CONTINUE
          AL(IR,IC,NLAYR) = -2.*DT*RZ(IR,IC,NLAYR)
          AD(IR,IC,NLAYR) = BVOLF(IR,IC,NLAYR) +
     .                        2.*DT*RZ(IR,IC,NLAYR)
          AU(IR,IC,NLAYR) = 0.0
  230   CONTINUE

C        FORWARD SWEEP OF TRIDIAGONAL SCHEME

        DO 240 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 240 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 240
          BETA(IR,IC,1) = AD(IR,IC,1)
          GAMMA(IR,IC,1) = MF(IR,IC,1)/BETA(IR,IC,1)
          DO 235 IL=2,NLAYR
           BETA(IR,IC,IL) = AD(IR,IC,IL)
     .                 - AL(IR,IC,IL)*AU(IR,IC,IL-1)/BETA(IR,IC,IL-1)
           GAMMA(IR,IC,IL) = (MF(IR,IC,IL)
     .                 - AL(IR,IC,IL)*GAMMA(IR,IC,IL-1))/BETA(IR,IC,IL)
  235     CONTINUE
  240   CONTINUE

C        BACKWARD SWEEP FOR SOLUTION

        DO 250 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 250 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 250
          CF(IR,IC,NLAYR,ISYS) = GAMMA(IR,IC,NLAYR)
          DO 245 ILYR=2,NLAYR
           IL = NLAYR+1-ILYR
           CF(IR,IC,IL,ISYS) = GAMMA(IR,IC,IL)
     .               - AU(IR,IC,IL)*CF(IR,IC,IL+1,ISYS)/BETA(IR,IC,IL)
  245     CONTINUE
  250   CONTINUE

C        NO CHANGE IN BOUNDARY CONCENTRATION
 
  255   DO 270 I=1,NOBCALL
          IR=IBCALL(1,I)
          IC=IBCALL(2,I)
          IL=IBCALL(3,I)
          CF(IR,IC,IL,ISYS)=CB(IR,IC,IL,ISYS)
 270    CONTINUE

            
    
C  FILTER SOLUTIONS

        DO 300 IL=1,NLAYR
         DO 300 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 300 IR=IS,IE
           C(IR,IC,IL,ISYS) = C(IR,IC,IL,ISYS) + 0.05 *
     .        (CF(IR,IC,IL,ISYS)+CB(IR,IC,IL,ISYS)-2.*C(IR,IC,IL,ISYS))
  300   CONTINUE

  320 CONTINUE
 

C  PREPARE FOR NEXT TIME STEP
 
      TIME=TIME + DTSTEP
  
      DO 340 ISYS=1,NOSYS
       DO 340 IL=1,NLAYR
        DO 340 IC=1,NCOL
        DO 340 IR=1,NROW
          CB(IR,IC,IL,ISYS) = C(IR,IC,IL,ISYS)
          C(IR,IC,IL,ISYS) = CF(IR,IC,IL,ISYS)
  340 CONTINUE

       DO 345 IL=1,NLAYR
        DO 345 IC=1,NCOL
        DO 345 IR=1,NROW
          BVOLB(IR,IC,IL) = BVOL(IR,IC,IL)
          BVOL(IR,IC,IL) = BVOLF(IR,IC,IL)
  345 CONTINUE

C        CHECK TO SEE IF IT IS TIME TO PRINT RESULTS
C        FIRST DETAILED DUMPS
       
      IF(PRNTD.EQ.0.0)  GO TO 385
      EPSLNP = DTSTEP/(PRNTD*2.)
      M=TIME/PRNTD+EPSLNP
      IF(M.LT.NND)  GO TO 385
 
C        DETAILED PRINT TIME
C        SET IDISK FOR DETAILED DUMPS
      
      IDISK=2
      NND=M+1
 
C        SECOND GLOBAL DUMPS
 
 385  EPSLNP=DTSTEP/(PRNTG*2.)
      M=TIME/PRNTG+EPSLNP
      IF (M.LT.NNG) GOTO 400
 
C        GLOBAL PRINT TIME
C        CALL PRINT ROUTINE
 
      CALL RCA09
      IDISK=IDISK+1
      NNG=M+1 

C  NOW PERFORM STABILITY CHECK
  
      DO 420 ISYS=1,NOSYS 
        IF(SYSBY(ISYS).EQ.1)   GO TO 420
        CKMAX = CMAX(ISYS)
        DO 410 IL=1,NLAYR
         DO 410 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 410 IR=IS,IE
           IF(FSM(IR,IC).LE.0)  GO TO 410
           IF(CF(IR,IC,IL,ISYS).GE.CKMAX)   GO TO 430
  410   CONTINUE
  420 CONTINUE

C        CHECK TO SEE IF NECESSARY TO UPDATE TIME-VARIABLE FUNCTIONS
C        FIRST POINT SOURCE LOADS 
 
  400 IF(IPSOPT.GT.1.AND.TIME.GE.NXPST)   
     .   CALL RCA10(BPS,NWK,NOPS,NXPST,22,SCALPS)

C        NONPOINT SOURCE LOADS 
 
      IF(INPSOPT.GT.1.AND.TIME.GE.NXNPST)   
     .   CALL RCA10(BNPS,NWK,NONPS,NXNPST,23,SCALNPS)

C        FALL-LINE LOADS 
 
      IF(IFLOPT.GT.1.AND.TIME.GE.NXFLT)   
     .   CALL RCA10(BFL,NWK,NOFL,NXFLT,24,SCALFL)

C        ATMOSPHERIC LOADS 
 
      IF(IATMOPT.GT.1.AND.TIME.GE.NXATMT)   
     .   CALL RCA10(BATM,1,NOATM,NXATMT,25,SCALATM)

C        NEXT BOUNDARY CONDITIONS 
 
      IF((IBCOPT.EQ.2.OR.IBCOPT.EQ.4).AND.TIME.GT.NXBCT)  CALL RCA11 

C        NEXT TRANSPORT FIELDS
 
      IF(TIME.GE.HYDBRK(ITIMHYD))  THEN
         CALL RCA03A
         CALL RCAEXP1
         ITIMHYD = ITIMHYD+1
      ENDIF


C        CHECK FOR END OF SIMULATION...
C           IF NOT GO BACK AND TAKE NEXT INTEGRATION STEP
 
      IF(TIME.LE.TEND)   GO TO 40 

      ITRAK = ITRAK+1 
      IF(ITRAK.LE.NSTEP)  THEN
         DT=STEP(ITRAK)
         TEND = TBRK(ITRAK) 
         IF(DT.EQ.0.0)   THEN
            CALL RCAMESS(6,DT)
            CALL EXIT
         ENDIF
      ELSE
         TEND = 0.0
      ENDIF 

      DTSTEP = DT 
      DT = SCALT*DT 
      IF(TEND.GT.0.)   GO TO 40 
 
C        FINISHED...CALL FOR FINAL DUMPS
 
      IDISK = 1 
      CALL TUNER
      RETURN

C         STABILITY CRITERIA VIOLATED...ABEND 
 
  430 CALL RCAMESS(1,CKMAX) 
      WRITE(OUT,9000)  BVOLB(IR,IC,IL),DIAG(IR,IC,IL)
 9000 FORMAT(10X,'VOLUME =',E15.6,'  ,DIAGONAL =',E13.4)
      IREC = IREC+1 
      IDISK = 1 
      CALL TUNER
      RETURN
      END 
