      SUBROUTINE RCALFS

C 
C     RCALF IS THE MASTER SUBROUTINE USED TO INTEGRATE THE DIFFERENTIAL
C           EQUATIONS OF THE WATER QUALITY MODEL.
C           IT CONTROLS THE NUMERICAL INTEGRATION USING A CENTRAL
C           DIFFERENCING SCHEME FOR TIME AND A SMOLARKIEWICZ CORRECTED
C           UPWIND SCHEME FOR SPACE.
C
  
      INCLUDE 'RCACM' 
      INTEGER   NEGSEG(99,3)
      REAL   C(NROW,NCOL,NLAYR,NOSYS)
     .    ,CB(NROW,NCOL,NLAYR,NOSYS),CF(NROW,NCOL,NLAYR,NOSYS)
     .    ,DM(NROW,NCOL,NLAYR),MF(NROW,NCOL,NLAYR)
     .    ,BVOLB(NROW,NCOL,NLAYR),BVOLF(NROW,NCOL,NLAYR),TOTMAS
     .    ,AL(NROW,NCOL,NLAYR),AD(NROW,NCOL,NLAYR),AU(NROW,NCOL,NLAYR)
     .    ,BETA(NROW,NCOL,NLAYR),GAMMA(NROW,NCOL,NLAYR)
     .    ,BBCALL(NBC,NOSYS)
 
 
      REAL   CONCS(NROW,NCOL,NLAYR)
     .      ,DXBAR1(NROW,NCOL),DYBAR1(NROW,NCOL)
     .      ,DXBAR2(NROW,NCOL),DYBAR2(NROW,NCOL)
     .      ,D(NROW,NCOL)
     .      ,ANTIQX(NROW,NCOL,NLAYR),QXFLX(NROW,NCOL,NLAYR)
     .      ,ANTIQY(NROW,NCOL,NLAYR),QYFLX(NROW,NCOL,NLAYR)
     .      ,ANTIQZ(NROW,NCOL,NLAYR),QZFLX(NROW,NCOL,NLAYR)
     .      ,RXFLX(NROW,NCOL,NLAYR),RYFLX(NROW,NCOL,NLAYR)
     .      ,U(NROW,NCOL,NLAYR),V(NROW,NCOL,NLAYR),W(NROW,NCOL,NLAYR)
   
      EQUIVALENCE
     .       (MF(1,1,1),SCRATCH_TRAN(1,1,1,2))
     .    ,  (AL(1,1,1),SCRATCH_TRAN(1,1,1,3))
     .    ,  (AD(1,1,1),SCRATCH_TRAN(1,1,1,4))
     .    ,  (AU(1,1,1),SCRATCH_TRAN(1,1,1,5))
     .    ,  (BETA(1,1,1),SCRATCH_TRAN(1,1,1,6))
     .    ,  (GAMMA(1,1,1),SCRATCH_TRAN(1,1,1,7))

      EQUIVALENCE
     .        (DM(1,1,1),SCRATCH_TRAN(1,1,1,8))
     .      , (BVOLB(1,1,1),SCRATCH_TRAN(1,1,1,9))
     .      , (ANTIQX(1,1,1),SCRATCH_TRAN(1,1,1,10))
     .      , (ANTIQY(1,1,1),SCRATCH_TRAN(1,1,1,11))
     .      , (ANTIQZ(1,1,1),SCRATCH_TRAN(1,1,1,12))
     .      , (QXFLX(1,1,1),SCRATCH_TRAN(1,1,1,13))
     .      , (QYFLX(1,1,1),SCRATCH_TRAN(1,1,1,14))
     .      , (QZFLX(1,1,1),SCRATCH_TRAN(1,1,1,15))
 
C  INITIALIZATION


      IFLAG_TBRK=1
      IFLAG_TBC=1

      ITRAK=1 
      TIME=0.0 
      TIMEWQ=0.0 
      INITB=0
      DTSTEP = DT
      DT = DT*SCALT
      DTWQ=DTWQ*SCALT
      NNG=1
      NND=1
      R=1.E-15
C
      IF(TZERO.NE.0.)   THEN
         CALL RCA08
         TIME = TZERO
         NNG=TIME/PRNTG+0.5
         IF (PRNTD.NE.0.0) NND=TIME/PRNTD+0.5
      ENDIF

      DO 20 IL=1,NLAYR
        DO 20 IC=1,NCOL
          DO 20 IR=1,NROW
            CONCS(IR,IC,IL) = 0.0
            ANTIQX(IR,IC,IL) = 0.0
            ANTIQY(IR,IC,IL) = 0.0
            ANTIQZ(IR,IC,IL) = 0.0
            QXFLX(IR,IC,IL) = 0.0
            RXFLX(IR,IC,IL) = 0.0
            QYFLX(IR,IC,IL) = 0.0
            RYFLX(IR,IC,IL) = 0.0
            QZFLX(IR,IC,IL) = 0.0
 20   CONTINUE

      DO 25 ISYS=1,NOSYS
       DO 25 IL=1,NLAYR
        DO 25 IC=1,NCOL
         DO 25 IR=1,NROW
           IF(ISYS.EQ.1)CARAY(IR,IC,IL,ISYS) = HYDSAL(IR,IC,IL)
           CB(IR,IC,IL,ISYS) = CARAY(IR,IC,IL,ISYS)
           C(IR,IC,IL,ISYS) = CARAY(IR,IC,IL,ISYS)
           CF(IR,IC,IL,ISYS) = CARAY(IR,IC,IL,ISYS)
   25 CONTINUE


C        CALCULATE CONSTANTS USED FOR SMOLARKIEWICZ CALCULATIONS

      DO 30 IC=2,NCOL
       DO 30 IR=2,NROW
           DXBAR1(IR,IC) = 0.5*(DX(IR,IC)+DX(IR-1,IC))
           DXBAR2(IR,IC) = 0.5*(DX(IR,IC)+DX(IR,IC-1))
           DYBAR1(IR,IC) = 0.5*(DY(IR,IC)+DY(IR-1,IC))
           DYBAR2(IR,IC) = 0.5*(DY(IR,IC)+DY(IR,IC-1))
   30 CONTINUE

C        SET INITIAL VOLUMES AT TIME 'N-1'

       DO 35 IL=1,NLAYR
        DO 35 IC=1,NCOL
         DO 35 IR=1,NROW
          BVOLB(IR,IC,IL) = BVOL(IR,IC,IL)
   35 CONTINUE

C        SET UP -AVECT- ARRAYS FOR TIME = 0.0 (TZERO)

      CALL RCAEXP1


C        PRINT INITIAL CONDITIONS

      CALL RCA09
      IDISK = 3 
 

C  BEGIN LEAP-FROG STEP

  40  CONTINUE


C        IF PIECEWISE LINEAR BOUNDARY CONDITIONS THEN
C           GET CONCENTRATIONS AT CURRENT TIME LEVEL

      IF(IBCPLOPT.EQ.1)   THEN
        DO 50 ISYS=1,NOSYS
          IF(NOBC(ISYS).EQ.0)  GO TO 50
          DELTBC = TIME - NXBCT
          DO 45 I=1,NOBC(ISYS)
            IR=IBC(1,I,ISYS)
            IC=IBC(2,I,ISYS)
            IL=IBC(3,I,ISYS)
            CARAY(IR,IC,IL,ISYS) = DELTBC*SBC(I,ISYS) + BBC(I,ISYS)
            CB(IR,IC,IL,ISYS) = CARAY(IR,IC,IL,ISYS)
   45     CONTINUE
   50  CONTINUE
      ENDIF

C        COMPUTE DEPTHS AT TIME LEVEL N

      DO 55 IC=1,NCOL
        DO 55 IR=1,NROW
            D(IR,IC) = BVOL(IR,IC,1)/(XAZ(IR,IC)+1.0E-20)/DZ(1)
  55  CONTINUE

C        CALCULATE HORIZONTAL AND VERTICAL VELOCITIES AT TIME 'N'

      DO 60 IL=1,NLAYR
        DO 60 IC=2,NCOL
          DO 60 IR=2,NROW
            U(IR,IC,IL)=QX(IR,IC,IL)/(DZ(IL)*0.5*(D(IR,IC)+D(IR-1,IC))*
     .                  DYBAR1(IR,IC))
            V(IR,IC,IL)=QY(IR,IC,IL)/(DZ(IL)*0.5*(D(IR,IC)+D(IR,IC-1))*
     .                  DXBAR2(IR,IC))
            W(IR,IC,IL)=QZ(IR,IC,IL)/XAZ(IR,IC)
 60   CONTINUE


C        EVALUATE KINETIC DERIVATIVES (IF NECESSARY)

      IF (DTWQ.EQ.0.0.OR.TIME.GE.TIMEWQ.OR.IDISK.EQ.3) THEN

       DO 70 ISYS=1,NOSYS
         DO 70 IL=1,NLAYR
           DO 70 IC=1,NCOL
             DO 70 IR=1,NROW
             IF(FSM(IR,IC).LE.0.)  GO TO 70
             CARAY(IR,IC,IL,ISYS) = MAX(CB(IR,IC,IL,ISYS),CMIN(ISYS))
 70    CONTINUE


        CALL TUNER
        TIMEWQ=TIMEWQ+DTWQ

      ENDIF

      INITB = 1 
      IDISK = 0 
 
C        COMPUTE VOLUMES AT TIME LEVEL 'N+1'
 
      DO 80 IL=1,NLAYR
        DO 80 IC=1,NCOL
        DO 80 IR=1,NROW
            BVOLF(IR,IC,IL) = BVOLB(IR,IC,IL) + 2.*DT*VDER(IR,IC,IL)
  80  CONTINUE

      DO 320 ISYS=1,NOSYS 
 
C        CHECK FOR SYSTEM BYPASS
      IF(SYSBY(ISYS).EQ.1)   GO TO 320

        INEGS=0
        TOTMAS=0.0
 
        DO 100 IL=1,NLAYR
         DO 100 IC=1,NCOL
          DO 100 IR=1,NROW
           DM(IR,IC,IL) = 0.0
           MF(IR,IC,IL) = 0.0
  100   CONTINUE

C        THE 120 THROUGH 131 DO LOOPS TAKE AN EULER INTEGRATION STEP
C        STEP FOR HORIZONTAL AND VERTICAL ADVECTIVE TRANSPORT
C           (-DIAG- AND -AVECT- CONTAIN JUST  Q  TERMS)
 
C        COMPUTE C AT TIME LEVEL N+1

       IF (NLAYR.GT.1) THEN

C      SURFACE LAYER
         DO 120 IC=2,NCOL-1
           IS = IRS(IC)
           IE = IRE(IC)
           DO 120 IR=IS,IE
             IF(FSM(IR,IC).LE.0.)  GO TO 120
             DM(IR,IC,1) = - DIAG(IR,IC,1)*CB(IR,IC,1,ISYS)
     .        + AVECT(IR,IC,1,1)*CB(IR-1,IC,1,ISYS)
     .         + AVECT(IR,IC,1,2)*CB(IR+1,IC,1,ISYS)
     .          + AVECT(IR,IC,1,3)*CB(IR,IC-1,1,ISYS)
     .           + AVECT(IR,IC,1,4)*CB(IR,IC+1,1,ISYS)
     .            + AVECT(IR,IC,1,6)*CB(IR,IC,2,ISYS)
  120    CONTINUE


C      LAYERS 2 TO NLAYR-1
        DO 125 IL=2,NLAYR-1
          DO 125 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 125 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 125
              DM(IR,IC,IL) = - DIAG(IR,IC,IL)*CB(IR,IC,IL,ISYS)
     .         + AVECT(IR,IC,IL,1)*CB(IR-1,IC,IL,ISYS)
     .          + AVECT(IR,IC,IL,2)*CB(IR+1,IC,IL,ISYS)
     .           + AVECT(IR,IC,IL,3)*CB(IR,IC-1,IL,ISYS)
     .            + AVECT(IR,IC,IL,4)*CB(IR,IC+1,IL,ISYS)
     .             + AVECT(IR,IC,IL,5)*CB(IR,IC,IL-1,ISYS)
     .              + AVECT(IR,IC,IL,6)*CB(IR,IC,IL+1,ISYS)
  125   CONTINUE

C      BOTTOM LAYER
         DO 130 IC=2,NCOL-1
           IS = IRS(IC)
           IE = IRE(IC)
           DO 130 IR=IS,IE
             IF(FSM(IR,IC).LE.0.)  GO TO 130
             DM(IR,IC,NLAYR) =
     .        - DIAG(IR,IC,NLAYR)*CB(IR,IC,NLAYR,ISYS)
     .         + AVECT(IR,IC,NLAYR,1)*CB(IR-1,IC,NLAYR,ISYS)
     .          + AVECT(IR,IC,NLAYR,2)*CB(IR+1,IC,NLAYR,ISYS)
     .           + AVECT(IR,IC,NLAYR,3)*CB(IR,IC-1,NLAYR,ISYS)
     .            + AVECT(IR,IC,NLAYR,4)*CB(IR,IC+1,NLAYR,ISYS)
     .             + AVECT(IR,IC,NLAYR,5)*CB(IR,IC,NLAYR-1,ISYS)
  130   CONTINUE

       ELSE

        IL=1
        DO 131 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 131 IR=IS,IE
             IF(FSM(IR,IC).LE.0.)  GO TO 131
            DM(IR,IC,IL) = - DIAG(IR,IC,IL)*CB(IR,IC,IL,ISYS)
     .       + AVECT(IR,IC,IL,1)*CB(IR-1,IC,IL,ISYS)
     .        + AVECT(IR,IC,IL,2)*CB(IR+1,IC,IL,ISYS)
     .         + AVECT(IR,IC,IL,3)*CB(IR,IC-1,IL,ISYS)
     .          + AVECT(IR,IC,IL,4)*CB(IR,IC+1,IL,ISYS)
  131   CONTINUE

       ENDIF
 

C        ADD SOURCES/SINKS DUE TO KINETICS

        DO 135 IL=1,NLAYR
          DO 135 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 135 IR=IS,IE
              DM(IR,IC,IL) = DM(IR,IC,IL) + CDARAY(IR,IC,IL,ISYS)
 135    CONTINUE


C        ADD LOADS
 
        IF(NOPS(ISYS).GT.0)  THEN
          DO 140 I=1,NOPS(ISYS) 
            IR = IPS(1,I,ISYS)
            IC = IPS(2,I,ISYS)
            IL = IPS(3,I,ISYS)
            DM(IR,IC,IL) = DM(IR,IC,IL) + BPS(I,ISYS)
  140     CONTINUE
        ENDIF

        IF(NONPS(ISYS).GT.0)   THEN
          DO 145 I=1,NONPS(ISYS)
            IR = INPS(1,I,ISYS)
            IC = INPS(2,I,ISYS)
            IL = INPS(3,I,ISYS)
            DM(IR,IC,IL) = DM(IR,IC,IL) + BNPS(I,ISYS)
  145     CONTINUE
        ENDIF

        IF(NOFL(ISYS).GT.0)   THEN
          DO 150 I=1,NOFL(ISYS)
            IR = IFL(1,I,ISYS)
            IC = IFL(2,I,ISYS)
            IL = IFL(3,I,ISYS)
            DM(IR,IC,IL) = DM(IR,IC,IL) + BFL(I,ISYS)
  150     CONTINUE
        ENDIF

        IF(NOATM(ISYS).GT.0)   THEN
          DO 155 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 155 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 155
              DM(IR,IC,1) = DM(IR,IC,1) + BATM(ISYS)*XAZ(IR,IC)
  155     CONTINUE
        ENDIF


C  CALCULATE CONCENTRATIONS: UPWIND ADVECTION, KINETICS, LOADS 

 
        DO 170 IL=1,NLAYR
          DO 170 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 170 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 170
              CONCS(IR,IC,IL) = (BVOLB(IR,IC,IL)*CB(IR,IC,IL,ISYS)
     .                           + 2.*DT*DM(IR,IC,IL))/BVOLF(IR,IC,IL)

 170    CONTINUE

 
C  BOUNDARY ELEMENTS ARE UNCHANGED
 
      DO 175 I=1,NOBCALL
        IR=IBCALL(1,I)
        IC=IBCALL(2,I)
        IL=IBCALL(3,I)
        CONCS(IR,IC,IL)=CB(IR,IC,IL,ISYS)
 175  CONTINUE

C
C******************************************************************
C

C           SMOLARKIEWICZ METHOD FOR SECOND-ORDER ACCURACY
 
 
C        CALCULATE ANTIDIFFUSIION FLOWS ( = VELOCITIES * AREAS)
 

      DO 180 IL=1,NLAYR
        DO 180 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 180 IR=IS,IE

          IF(FSM(IR,IC).LE.0.) GO TO 180

            IF(CONCS(IR,IC,IL).LE.1.0E-09 .OR.
     .            CONCS(IR-1,IC,IL).LE.1.0E-09)  THEN
               ANTIQX(IR,IC,IL) = 0.0
            ELSE IF(ABS(U(IR,IC,IL)).LT.(U(IR,IC,IL)*U(IR,IC,IL)*
     .                 2.*DT/DXBAR1(IR,IC)))   THEN
               ANTIQX(IR,IC,IL) = 0.0
            ELSE
               ANTIQX(IR,IC,IL) = ABS(U(IR,IC,IL))*
     .          (1.-ABS(U(IR,IC,IL))*2.*DT/DXBAR1(IR,IC))*
     .               (CONCS(IR,IC,IL)-CONCS(IR-1,IC,IL))/
     .                    (CONCS(IR,IC,IL)+CONCS(IR-1,IC,IL)+R)
            ENDIF


            IF(CONCS(IR,IC,IL).LE.1.0E-09 .OR.
     .            CONCS(IR,IC-1,IL).LE.1.0E-09)  THEN
               ANTIQY(IR,IC,IL) = 0.0
            ELSE IF(ABS(V(IR,IC,IL)).LT.(V(IR,IC,IL)*V(IR,IC,IL)*
     .                 2.*DT/DYBAR2(IR,IC)))   THEN
               ANTIQY(IR,IC,IL) = 0.0
            ELSE
               ANTIQY(IR,IC,IL) = ABS(V(IR,IC,IL))*
     .          (1.-ABS(V(IR,IC,IL))*2.*DT/DYBAR2(IR,IC))*
     .               (CONCS(IR,IC,IL)-CONCS(IR,IC-1,IL))/
     .                    (CONCS(IR,IC,IL)+CONCS(IR,IC-1,IL)+R)
            ENDIF


            IF (IL.GT.1) THEN
               IF(CONCS(IR,IC,IL).LE.1.0E-09 .OR.
     .               CONCS(IR,IC,IL-1).LE.1.0E-09)  THEN
                  ANTIQZ(IR,IC,IL) = 0.0
               ELSE IF(ABS(W(IR,IC,IL)).LT.(W(IR,IC,IL)*W(IR,IC,IL)*
     .                    2.*DT/(D(IR,IC)*DZZ(IL-1))))   THEN
                  ANTIQZ(IR,IC,IL) = 0.0
               ELSE
                 ANTIQZ(IR,IC,IL) = ABS(W(IR,IC,IL))*
     .                 (1.-ABS(W(IR,IC,IL))*2.*DT/(D(IR,IC)*DZZ(IL-1)))*
     .                  (CONCS(IR,IC,IL-1)-CONCS(IR,IC,IL))/
     .                       (CONCS(IR,IC,IL-1)+CONCS(IR,IC,IL)+R)
               ENDIF
            ENDIF

 180  CONTINUE


C        CALCULATE ANTIDIFFUSION FLUXES
 
      DO 190 IL=1,NLAYR
        DO 190 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 190 IR=IS,IE

            IF(FSM(IR,IC).LE.0.) GO TO 190

            IF(FSM(IR,IC).EQ.1 .AND. FSM(IR-1,IC).EQ.1.) 
     .        QXFLX(IR,IC,IL) = 0.5*(ANTIQX(IR,IC,IL)*(CONCS(IR,IC,IL)+
     .           CONCS(IR-1,IC,IL))-ABS(ANTIQX(IR,IC,IL))*
     .           (CONCS(IR,IC,IL)-CONCS(IR-1,IC,IL)))
     .           *DYBAR1(IR,IC)*0.5*(D(IR,IC)+D(IR-1,IC))*DZ(IL)

            IF(FSM(IR,IC).EQ.1 .AND. FSM(IR,IC-1).EQ.1.) 
     .        QYFLX(IR,IC,IL) = 0.5*(ANTIQY(IR,IC,IL)*(CONCS(IR,IC,IL)+
     .          CONCS(IR,IC-1,IL))-ABS(ANTIQY(IR,IC,IL))*
     .          (CONCS(IR,IC,IL)-CONCS(IR,IC-1,IL)))      
     .           *DXBAR2(IR,IC)*0.5*(D(IR,IC)+D(IR,IC-1))*DZ(IL)

            IF(NLAYR.GT.1.AND.IL.GT.1.AND.FSM(IR,IC).GT.0.) 
     .        QZFLX(IR,IC,IL) = 0.5*
     .        (ANTIQZ(IR,IC,IL)*(CONCS(IR,IC,IL-1)+CONCS(IR,IC,IL))-
     .          ABS(ANTIQZ(IR,IC,IL))*(CONCS(IR,IC,IL-1)-
     .          CONCS(IR,IC,IL)))*XAZ(IR,IC)

 190  CONTINUE
 

C        TAKE CORRECTION STEP, PLACING RESULTS
C          BACK INTO CONCS NOW AT '(N+1)'
   
        IF (NLAYR.GT.1) THEN
          DO 200 IL=1,NLAYR-1
            DO 200 IC=2,NCOL-1
              IS = IRS(IC)
              IE = IRE(IC)
              DO 200 IR=IS,IE
                IF(FSM(IR,IC).LE.0.)  GO TO 200
                CONCS(IR,IC,IL)=CONCS(IR,IC,IL)
     .             -2.*DT*(QXFLX(IR+1,IC,IL)-QXFLX(IR,IC,IL)
     .                    +QYFLX(IR,IC+1,IL)-QYFLX(IR,IC,IL)
     .                    +QZFLX(IR,IC,IL)-QZFLX(IR,IC,IL+1))/
     .                                             BVOLF(IR,IC,IL)
 200      CONTINUE

C  
          IL=NLAYR
          DO 205 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 205 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 205
              CONCS(IR,IC,IL)=CONCS(IR,IC,IL)
     .           -2.*DT*(QXFLX(IR+1,IC,IL)-QXFLX(IR,IC,IL)
     .                  +QYFLX(IR,IC+1,IL)-QYFLX(IR,IC,IL)
     .                  +QZFLX(IR,IC,IL))/BVOLF(IR,IC,IL)
 205      CONTINUE
        ELSE
          IL=1
          DO 210 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 210 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 210
              CONCS(IR,IC,IL)=CONCS(IR,IC,IL)
     .           -2.*DT*(QXFLX(IR+1,IC,IL)-QXFLX(IR,IC,IL)
     .                  +QYFLX(IR,IC+1,IL)-QYFLX(IR,IC,IL))/
     .                                             BVOLF(IR,IC,IL)
 210      CONTINUE
        ENDIF

 
C  CALCULATE HORIZONTAL DIFFUSION FLUXES
C  - USING CONCENTRATIONS AT TIME LEVEL "N-1" -
 
        DO 220 IL=1,NLAYR
          DO 220 IC=2,NCOL
          DO 220 IR=2,NROW
              IF(FSM(IR,IC).GE.1. OR. FSM(IR,IC).LE.-2)THEN
                IF(FSM(IR-1,IC).GE.1. OR. FSM(IR-1,IC).LE.-2)THEN
                  RXFLX(IR,IC,IL)=RX(IR,IC,IL)*
     .             (CB(IR,IC,IL,ISYS)-CB(IR-1,IC,IL,ISYS))
                ENDIF
                IF(FSM(IR,IC-1).GE.1. OR. FSM(IR,IC-1).LE.-2)THEN
                  RYFLX(IR,IC,IL)=RY(IR,IC,IL)*
     .             (CB(IR,IC,IL,ISYS)-CB(IR,IC-1,IL,ISYS))
                ENDIF
              ENDIF
 220    CONTINUE
 
 
C  CALCULATE CONCENTRATIONS: HORIZONTAL DIFFUSION
 
        DO 230 IL=1,NLAYR
          DO 230 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 230 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)  GO TO 230
              CONCS(IR,IC,IL)=CONCS(IR,IC,IL)+2.*DT*
     .           (RXFLX(IR+1,IC,IL)-RXFLX(IR,IC,IL)+RYFLX(IR,IC+1,IL)
     .           -RYFLX(IR,IC,IL))/BVOLF(IR,IC,IL)
 230    CONTINUE
 

C  END OF SMOLARKIEWICZ METHOD
 
C**********************************************************************
 

C  COMPLETE REST OF INTEGRATION STEP
C  I.E., THE IMPLICIT VERTICAL DIFFUSION STEP
 
 
        DO 240 IL=1,NLAYR
         DO 240 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 240 IR=IS,IE
            MF(IR,IC,IL) = CONCS(IR,IC,IL)*BVOLF(IR,IC,IL)
 240    CONTINUE

C  IMPLICIT VERTICAL DIFFUSION STEP  (IF NLAYR > 1)
 
        IF(NLAYR.EQ.1)  GO TO 275

C        SET UP DIAGONAL AND OFF-DIAGONAL ELEMENTS

        DO 250 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 250 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 250
          AL(IR,IC,1) = 0.
          AD(IR,IC,1) = BVOLF(IR,IC,1) + 2.*DT*RZ(IR,IC,2)
          AU(IR,IC,1) = -2.*DT*RZ(IR,IC,2)
           DO 245 IL=2,NLAYR-1
            AL(IR,IC,IL) = -2.*DT*RZ(IR,IC,IL)
            AD(IR,IC,IL) = BVOLF(IR,IC,IL) +
     .                        2.*DT*(RZ(IR,IC,IL) + RZ(IR,IC,IL+1))
            AU(IR,IC,IL) = -2.*DT*RZ(IR,IC,IL+1)
  245      CONTINUE
          AL(IR,IC,NLAYR) = -2.*DT*RZ(IR,IC,NLAYR)
          AD(IR,IC,NLAYR) = BVOLF(IR,IC,NLAYR) +
     .                        2.*DT*RZ(IR,IC,NLAYR)
          AU(IR,IC,NLAYR) = 0.0
  250   CONTINUE

C        FORWARD SWEEP OF TRIDIAGONAL SCHEME

        DO 260 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 260 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 260
          BETA(IR,IC,1) = AD(IR,IC,1)
          GAMMA(IR,IC,1) = MF(IR,IC,1)/BETA(IR,IC,1)
          DO 255 IL=2,NLAYR
           BETA(IR,IC,IL) = AD(IR,IC,IL)
     .                 - AL(IR,IC,IL)*AU(IR,IC,IL-1)/BETA(IR,IC,IL-1)
           GAMMA(IR,IC,IL) = (MF(IR,IC,IL)
     .                 - AL(IR,IC,IL)*GAMMA(IR,IC,IL-1))/BETA(IR,IC,IL)
  255     CONTINUE
  260   CONTINUE

C        BACKWARD SWEEP FOR SOLUTION

        DO 270 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 270 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 270
          CF(IR,IC,NLAYR,ISYS) = GAMMA(IR,IC,NLAYR)
          DO 265 ILYR=2,NLAYR
           IL = NLAYR+1-ILYR
           CF(IR,IC,IL,ISYS) = GAMMA(IR,IC,IL)
     .               - AU(IR,IC,IL)*CF(IR,IC,IL+1,ISYS)/BETA(IR,IC,IL)
  265     CONTINUE
  270   CONTINUE

  275   CONTINUE


C  FILTER SOLUTIONS

        DO 300 IL=1,NLAYR
         DO 300 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 300 IR=IS,IE
           IF(FSM(IR,IC).LE.0.)  GO TO 300
           C(IR,IC,IL,ISYS) = C(IR,IC,IL,ISYS) + 0.025 *
     .        (CF(IR,IC,IL,ISYS)+CB(IR,IC,IL,ISYS)-2.*C(IR,IC,IL,ISYS))
  300   CONTINUE

  320 CONTINUE


C  PREPARE FOR NEXT TIME STEP
 
      TIME=TIME + DTSTEP
  
      DO 340 ISYS=1,NOSYS
        DO 340 IL=1,NLAYR
        DO 340 IC=1,NCOL
        DO 340 IR=1,NROW
            CB(IR,IC,IL,ISYS) = C(IR,IC,IL,ISYS)
            C(IR,IC,IL,ISYS) = CF(IR,IC,IL,ISYS)
  340 CONTINUE


       DO 345 IL=1,NLAYR
         DO 345 IC=1,NCOL
           DO 345 IR=1,NROW
          BVOLB(IR,IC,IL) = BVOL(IR,IC,IL)
          BVOL(IR,IC,IL) = BVOLF(IR,IC,IL)
  345 CONTINUE


C        CHECK TO SEE IF IT IS TIME TO PRINT RESULTS
C        FIRST DETAILED DUMPS
       
      IF(PRNTD.EQ.0.0)  GO TO 385
      EPSLNP = DTSTEP/(PRNTD*2.)
      M=TIME/PRNTD+EPSLNP
      IF(M.LT.NND)  GO TO 385
 
C        DETAILED PRINT TIME
C        SET IDISK FOR DETAILED DUMPS
      
      IDISK=2
      NND=M+1
 
C        SECOND GLOBAL DUMPS
 
 385  EPSLNP=DTSTEP/(PRNTG*2.)
      M=TIME/PRNTG+EPSLNP
      IF (M.LT.NNG) GOTO 400
 
C        GLOBAL PRINT TIME
C        CALL PRINT ROUTINE
 
      DO 395 ISYS=1,NOSYS
       DO 395 IL=1,NLAYR
        DO 395 IC=1,NCOL
         DO 395 IR=1,NROW
           CARAY(IR,IC,IL,ISYS) = CB(IR,IC,IL,ISYS)
  395 CONTINUE

      CALL RCA09
      IDISK=IDISK+1
      NNG=M+1 

C  NOW PERFORM STABILITY CHECK
  
      DO 420 ISYS=1,NOSYS 
        IF(SYSBY(ISYS).EQ.1)   GO TO 420
        CKMAX = CMAX(ISYS)

        DO 410 IL=1,NLAYR
         DO 410 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 410 IR=IS,IE
           IF(FSM(IR,IC).LE.0)  GO TO 410
           IF(CF(IR,IC,IL,ISYS).GE.CKMAX)   GO TO 430
  410   CONTINUE
  420 CONTINUE

C        CHECK TO SEE IF NECESSARY TO UPDATE TIME-VARIABLE FUNCTIONS
C        FIRST POINT SOURCE LOADS 
 
  400 IF(IPSOPT.GT.1.AND.TIME.GE.NXPST)   
     .   CALL RCA10(BPS,NWK,NOPS,NXPST,22,SCALPS)
C
C        NONPOINT SOURCE LOADS 
C
      IF(INPSOPT.GT.1.AND.TIME.GE.NXNPST)   
     .   CALL RCA10(BNPS,NWK,NONPS,NXNPST,23,SCALNPS)

C
C        FALL-LINE LOADS 
C
      IF(IFLOPT.GT.1.AND.TIME.GE.NXFLT)   
     .   CALL RCA10(BFL,NWK,NOFL,NXFLT,24,SCALFL)

C
C        ATMOSPHERIC LOADS 
C
      IF(IATMOPT.GT.1.AND.TIME.GE.NXATMT)   
     .   CALL RCA10(BATM,1,NOATM,NXATMT,25,SCALATM)

C
C        NEXT BOUNDARY CONDITIONS 
C
      IFLAG_TBC=1
      IF((IBCOPT.EQ.2.OR.IBCOPT.EQ.4).AND.TIME.GE.NXBCT)  THEN
         IFLAG_TBC=0
         CALL RCA11 
      ENDIF

      IF(IBCPLOPT.NE.1)  THEN
        DO 425 ISYS=1,NOSYS
        DO 425 I=1,NOBCALL
          IR=IBCALL(1,I)
          IC=IBCALL(2,I)
          IL=IBCALL(3,I)
          CB(IR,IC,IL,ISYS)=CARAY(IR,IC,IL,ISYS) !! stepwise
 425    CONTINUE
      ENDIF


C        NEXT TRANSPORT FIELDS
C
         IFLAG_TBRK=1
      IF(TIME.GE.HYDBRK(ITIMHYD))  THEN
         IFLAG_TBRK=0
         CALL RCA03A

         CALL RCAEXP1
         ITIMHYD = ITIMHYD+1

      ENDIF

   
       IF(IFLAG_TBRK.EQ.0)THEN

       DO 346 IL=1,NLAYR
         DO 346 IC=1,NCOL
         DO 346 IR=1,NROW
          BVOLB(IR,IC,IL) = BVOL(IR,IC,IL)
  346 CONTINUE

       ENDIF



 
C        CHECK FOR END OF SIMULATION...
C           IF NOT GO BACK AND TAKE NEXT INTEGRATION STEP
C
      IF(TIME.LE.TEND)   GO TO 40 

      ITRAK = ITRAK+1 
      IF(ITRAK.LE.NSTEP)  THEN
         DT=STEP(ITRAK)
         TEND = TBRK(ITRAK) 
         IF(DT.EQ.0.0)   THEN
            CALL RCAMESS(6,DT)
            CALL EXIT
         ENDIF
      ELSE
         TEND = 0.0
      ENDIF 

      DTSTEP = DT 
      DT = SCALT*DT 
      IF(TEND.GT.0.)   GO TO 40 
C
C        FINISHED...CALL FOR FINAL DUMPS
C
      IDISK = 1 
      CALL TUNER
      RETURN

C
C         STABILITY CRITERIA VIOLATED...ABEND 
C
  430 CALL RCAMESS(1,CKMAX) 
      WRITE(OUT,9000)  BVOLB(IR,IC,IL),DIAG(IR,IC,IL)
 9000 FORMAT(10X,'VOLUME =',E15.6,'  ,DIAGONAL =',E13.4)
      IREC = IREC+1 
      IDISK = 1 
      CALL TUNER
      RETURN
      END 
