      SUBROUTINE RCASPLT
C 
C        RCASPLT IS THE MASTER SUBROUTINE USED FOR INTEGRATING THE DIFFERENTIAL
C                EQUATIONS OF THE WATER QUALITY MODEL.
C                TIME DERIVATIVE: EXPLICIT - SPLIT TIMESTEP
C                SPACE DERIVATIVE: UPWIND
C
C************************************************************************
C 
      INCLUDE 'RCACM' 
      INTEGER  NEGSEG(99,3)
      REAL   CONCS(NROW,NCOL,0:NLAYR)
     .      ,DM(NROW,NCOL,NLAYR,NOSYS),BVOLN(NROW,NCOL,NLAYR),TOTMAS
     .      ,AL(NROW,NCOL,NLAYR),AD(NROW,NCOL,NLAYR),AU(NROW,NCOL,NLAYR)
     .      ,BETA(NROW,NCOL,NLAYR),GAMMA(NROW,NCOL,NLAYR)
     .      ,MF(NROW,NCOL,NLAYR),CDARAYS(NROW,NCOL,NLAYR,NOSYS)

      EQUIVALENCE
     .       (MF(1,1,1),SCRATCH_TRAN(1,1,1,2))
     .    ,  (AL(1,1,1),SCRATCH_TRAN(1,1,1,3))
     .    ,  (AD(1,1,1),SCRATCH_TRAN(1,1,1,4))
     .    ,  (AU(1,1,1),SCRATCH_TRAN(1,1,1,5))
     .    ,  (BETA(1,1,1),SCRATCH_TRAN(1,1,1,6))
     .    ,  (GAMMA(1,1,1),SCRATCH_TRAN(1,1,1,7))
 
C        INITIALIZATION

      ITRAK=1 
      TIME=0.0 
      TIMEWQ=0.0 
      INITB=0
      NSPLIT = DTFULL/DTSPLIT
      DTSTEP = DTFULL
      DTFULL = DTFULL*SCALT
      DT = DTFULL
      DTSPLIT = DT/NSPLIT
      DTWQ=DTWQ*SCALT
      NNG=1
      NND=1
C
      R=1.E-15
C
      IF(TZERO.NE.0.)   THEN
         CALL RCA08
         TIME = TZERO
         NNG=TIME/PRNTG+0.5
         IF (PRNTD.NE.0.0) NND=TIME/PRNTD+0.5
      ENDIF

      DO 25 ISYS=1,NOSYS
       DO 25 IL=1,NLAYR
        DO 25 IC=1,NCOL
          DO 25 IR=1,NROW
            CDARAYS(IR,IC,IL,ISYS) = 0.0
   25 CONTINUE

      DO 30 IL=1,NLAYR
       DO 30 IC=1,NCOL
        DO 30 IR=1,NROW
         CONCS(IR,IC,IL) = 0.0
   30 CONTINUE

C        SET UP -AVECT- ARRAYS FOR TIME = 0.0 (TZERO)
      CALL RCASPLT1

C        PRINT INITIAL CONDITIONS
      CALL RCA09
      IDISK = 3 
 
   40 CONTINUE

C        IF PIECEWISE LINEAR BOUNDARY CONDITIONS THEN
C           GET CONCENTRATIONS AT CURRENT TIME LEVEL

      IF(IBCPLOPT.EQ.1)   THEN
        DO 50 ISYS=1,NOSYS
          IF(NOBC(ISYS).EQ.0)  GO TO 50
          DELTBC = TIME - NXBCT
          DO 45 I=1,NOBC(ISYS)
            IR=IBC(1,I,ISYS)
            IC=IBC(2,I,ISYS)
            IL=IBC(3,I,ISYS)
            CARAY(IR,IC,IL,ISYS) = DELTBC*SBC(I,ISYS) + BBC(I,ISYS)
   45     CONTINUE
   50  CONTINUE
      ENDIF

C        EVALUATE KINETIC DERIVATIVES 
      IF (DTWQ.EQ.0.0.OR.TIME.GE.TIMEWQ.OR.IDISK.EQ.3) THEN
        CALL TUNER
        IDISK = 0 
        TIMEWQ=TIMEWQ+DTWQ
        INITB = 1 
        ICHANGE = 1
      ENDIF

C        IF TUNER CALLED OR IF LOADS CHANGED THEN UPDATE -CDARAYS-
      IF(ICHANGE.EQ.1)  THEN

       DO 80 ISYS=1,NOSYS

C      SOURCES/SINKS DUE TO KINETICS

        DO 55 IL=1,NLAYR
         DO 55 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 55 IR=IS,IE
            CDARAYS(IR,IC,IL,ISYS) = CDARAY(IR,IC,IL,ISYS)
  55    CONTINUE
 
C  ADD LOADS

        IF(NOPS(ISYS).GT.0)  THEN
          DO 60 I=1,NOPS(ISYS) 
           IR = IPS(1,I,ISYS)
           IC = IPS(2,I,ISYS)
           IL = IPS(3,I,ISYS)
           CDARAYS(IR,IC,IL,ISYS) = CDARAYS(IR,IC,IL,ISYS)+BPS(I,ISYS)
   60     CONTINUE
        ENDIF

        IF(NONPS(ISYS).GT.0)   THEN
          DO 65 I=1,NONPS(ISYS)
           IR = INPS(1,I,ISYS)
           IC = INPS(2,I,ISYS)
           IL = INPS(3,I,ISYS)
           CDARAYS(IR,IC,IL,ISYS) = CDARAYS(IR,IC,IL,ISYS)+BNPS(I,ISYS)
   65     CONTINUE
        ENDIF

        IF(NOFL(ISYS).GT.0)   THEN
          DO 70 I=1,NOFL(ISYS)
           IR = IFL(1,I,ISYS)
           IC = IFL(2,I,ISYS)
           IL = IFL(3,I,ISYS)
           CDARAYS(IR,IC,IL,ISYS) = CDARAYS(IR,IC,IL,ISYS)+BFL(I,ISYS)
   70     CONTINUE
        ENDIF
  
        IF(NOATM(ISYS).GT.0)   THEN
          DO 75 IC=2,NCOL-1
           IS = IRS(IC)
           IE = IRE(IC)
           DO 75 IR=IS,IE
            CDARAYS(IR,IC,1,ISYS) = CDARAYS(IR,IC,1,ISYS)
     .                                   + BATM(ISYS)*XAZ(IR,IC)
   75     CONTINUE
        ENDIF

   80  CONTINUE
       ICHANGE = 0
      ENDIF

C        EULER INTEGRATION STEP

      DO 100 ISYS=1,NOSYS
       DO 100 IL=1,NLAYR
        DO 100 IC=1,NCOL
          IS = IRS(IC)
          IE = IRE(IC)
          DO 100 IR=IS,IE
            DM(IR,IC,IL,ISYS) = 0.0
  100 CONTINUE
      
      IF(NSEGSPLT.EQ.0)  GO TO 200

C     PERFORM SPLIT TIMESTEP COMPUTATION
C        FIRST INTEGRATING CRITICAL TIMESTEP SEGMENTS

      DO 180 ISYS=1,NOSYS 
 
C        CHECK FOR SYSTEM BYPASS
 
       IF(SYSBY(ISYS).EQ.1)   GO TO 180

       INEGS=0
       TOTMAS=0.0

       DO 170 ISPLIT=1,NSPLIT

        DO 150 ISEG=1,NSEGSPLT
          IR = ISEGSPLT(ISEG,1)
          IC = ISEGSPLT(ISEG,2)
          IL = ISEGSPLT(ISEG,3)
C        COMPUTE VOLUMES AT TIME LEVEL 'N+1'
          BVOLN(IR,IC,IL) = BVOL(IR,IC,IL)
     .                           + ISPLIT*DTSPLIT*VDERSPLT(ISEG)
 
C        COMPUTE CHANGE IN MASS FOR CRITICAL SEGMENT
          DMSPLT = CDARAYS(IR,IC,IL,ISYS)
     .        - DIAGSPLT(ISEG)*CARAY(IR,IC,IL,ISYS)
     .         + AVECTSPLT(ISEG,1)*CARAY(IR-1,IC,IL,ISYS)
     .          + AVECTSPLT(ISEG,2)*CARAY(IR+1,IC,IL,ISYS)
     .           + AVECTSPLT(ISEG,3)*CARAY(IR,IC-1,IL,ISYS)
     .            + AVECTSPLT(ISEG,4)*CARAY(IR,IC+1,IL,ISYS)
          IF(IL.GT.1)   
     .      DMSPLT = DMSPLT + AVECTSPLT(ISEG,5)*CARAY(IR,IC,IL-1,ISYS)
          IF(IL.LT.NLAYR)   
     .      DMSPLT = DMSPLT + AVECTSPLT(ISEG,6)*CARAY(IR,IC,IL+1,ISYS)

C        COMPUTE CHANGE IN MASS IN SEGMENTS ADJOINING CRITICAL SEGMENT
          DM(IR-1,IC,IL,ISYS) = DM(IR-1,IC,IL,ISYS) +
     .         AVECTSPLT(ISEG,7)*CARAY(IR,IC,IL,ISYS)/NSPLIT
          DM(IR+1,IC,IL,ISYS) = DM(IR+1,IC,IL,ISYS) +
     .         AVECTSPLT(ISEG,8)*CARAY(IR,IC,IL,ISYS)/NSPLIT
          DM(IR,IC-1,IL,ISYS) = DM(IR,IC-1,IL,ISYS) +
     .         AVECTSPLT(ISEG,9)*CARAY(IR,IC,IL,ISYS)/NSPLIT
          DM(IR,IC+1,IL,ISYS) = DM(IR,IC+1,IL,ISYS) +
     .         AVECTSPLT(ISEG,10)*CARAY(IR,IC,IL,ISYS)/NSPLIT
          IF(IL.GT.1)  DM(IR,IC,IL-1,ISYS) = DM(IR,IC,IL-1,ISYS) +
     .         AVECTSPLT(ISEG,11)*CARAY(IR,IC,IL,ISYS)/NSPLIT
          IF(IL.LT.NLAYR)  DM(IR,IC,IL+1,ISYS) = DM(IR,IC,IL+1,ISYS) +
     .         AVECTSPLT(ISEG,12)*CARAY(IR,IC,IL,ISYS)/NSPLIT

C        NOW COMPUTE CHANGE IN CONCENTRATION FOR CRITICAL SEGMENT
          CONCS(IR,IC,IL) = ((BVOL(IR,IC,IL)+(ISPLIT-1)*DTSPLIT
     .         *VDERSPLT(ISEG))*CARAY(IR,IC,IL,ISYS) + DTSPLIT*DMSPLT)
     .                /BVOLN(IR,IC,IL)
C        PREVENT NEGATIVE SOLUTIONS
          IF(CONCS(IR,IC,IL).LT.0.0)
     .           CONCS(IR,IC,IL) = CARAY(IR,IC,IL,ISYS)/4.
  
  150   CONTINUE

        DO 160 ISEG=1,NSEGSPLT
          IR = ISEGSPLT(ISEG,1)
          IC = ISEGSPLT(ISEG,2)
          IL = ISEGSPLT(ISEG,3)
          DM(IR,IC,IL,ISYS) = 0.0
          CARAY(IR,IC,IL,ISYS) = CONCS(IR,IC,IL)
  160   CONTINUE

  170  CONTINUE

  180 CONTINUE

C        UPDATE VOLUMES FOR CRITICAL SEGMENTS
      DO 190 ISEG=1,NSEGSPLT
        IR = ISEGSPLT(ISEG,1)
        IC = ISEGSPLT(ISEG,2)
        IL = ISEGSPLT(ISEG,3)
        BVOL(IR,IC,IL) = BVOL(IR,IC,IL) + DT*VDERSPLT(ISEG)
  190 CONTINUE

  200 CONTINUE


C        NOW TAKE FULL EXPLICIT STEP FOR REMAINING WATER SEGMENTS

C        COMPUTE VOLUMES AT TIME LEVEL 'N+1'
 
      DO 210 IL=1,NLAYR
        DO 210 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 210 IR=IS,IE
            BVOLN(IR,IC,IL) = BVOL(IR,IC,IL) + DT*VDER(IR,IC,IL)
 210  CONTINUE
 
C        UPDATE -DM- TO INCLUDE KINETICS AND LOADS

      DO 215 ISYS=1,NOSYS
       DO 215 IL=1,NLAYR
        DO 215 IC=2,NCOL-1
         IS = IRS(IC)
         IE = IRE(IC)
         DO 215 IR=IS,IE
          DM(IR,IC,IL,ISYS) = DM(IR,IC,IL,ISYS)
     .       + (FSM(IR,IC)-FSMSPLT(IR,IC,IL))*CDARAYS(IR,IC,IL,ISYS)
 215  CONTINUE
 

      DO 500 ISYS=1,NOSYS 
C
C        CHECK FOR SYSTEM BYPASS
C
      IF(SYSBY(ISYS).EQ.1)   GO TO 500

      INEGS=0
      TOTMAS=0.0

C  CALCULATE CHANGE IN CONCENTRATION DUE TO ADVECTIVE TRANSPORT
C    AND HORIZONTAL MIXING PROCESSES

      IF (NLAYR.GT.1) THEN

C      SURFACE LAYER
        DO 220 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 220 IR=IS,IE
            IF(FSM(IR,IC).LE.0.)GO TO 220
            DM(IR,IC,1,ISYS) = DM(IR,IC,1,ISYS)
     .        - DIAG(IR,IC,1)*CARAY(IR,IC,1,ISYS)
     .         + AVECT(IR,IC,1,1)*CARAY(IR-1,IC,1,ISYS)
     .          + AVECT(IR,IC,1,2)*CARAY(IR+1,IC,1,ISYS)
     .           + AVECT(IR,IC,1,3)*CARAY(IR,IC-1,1,ISYS)
     .            + AVECT(IR,IC,1,4)*CARAY(IR,IC+1,1,ISYS)
     .              + AVECT(IR,IC,1,6)*CARAY(IR,IC,2,ISYS)

            CONCS(IR,IC,1) = (BVOL(IR,IC,1)*CARAY(IR,IC,1,ISYS)
     .                          + DT*DM(IR,IC,1,ISYS))/BVOLN(IR,IC,1)
  220   CONTINUE
C      LAYERS 2 TO NLAYR-1
        DO 225 IL=2,NLAYR-1
          DO 225 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 225 IR=IS,IE
              IF(FSM(IR,IC).LE.0.)GO TO 225
              DM(IR,IC,IL,ISYS) = DM(IR,IC,IL,ISYS)
     .         - DIAG(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .          + AVECT(IR,IC,IL,1)*CARAY(IR-1,IC,IL,ISYS)
     .           + AVECT(IR,IC,IL,2)*CARAY(IR+1,IC,IL,ISYS)
     .            + AVECT(IR,IC,IL,3)*CARAY(IR,IC-1,IL,ISYS)
     .             + AVECT(IR,IC,IL,4)*CARAY(IR,IC+1,IL,ISYS)
     .              + AVECT(IR,IC,IL,5)*CARAY(IR,IC,IL-1,ISYS)
     .               + AVECT(IR,IC,IL,6)*CARAY(IR,IC,IL+1,ISYS)

              CONCS(IR,IC,IL) = (BVOL(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .                           + DT*DM(IR,IC,IL,ISYS))/BVOLN(IR,IC,IL)
  225   CONTINUE
C      BOTTOM LAYER
        DO 230 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 230 IR=IS,IE
            IF(FSM(IR,IC).LE.0.)GO TO 230
            DM(IR,IC,NLAYR,ISYS) = DM(IR,IC,NLAYR,ISYS)
     .        - DIAG(IR,IC,NLAYR)*CARAY(IR,IC,NLAYR,ISYS)
     .         + AVECT(IR,IC,NLAYR,1)*CARAY(IR-1,IC,NLAYR,ISYS)
     .          + AVECT(IR,IC,NLAYR,2)*CARAY(IR+1,IC,NLAYR,ISYS)
     .           + AVECT(IR,IC,NLAYR,3)*CARAY(IR,IC-1,NLAYR,ISYS)
     .            + AVECT(IR,IC,NLAYR,4)*CARAY(IR,IC+1,NLAYR,ISYS)
     .             + AVECT(IR,IC,NLAYR,5)*CARAY(IR,IC,NLAYR-1,ISYS)

            CONCS(IR,IC,NLAYR) =
     .             (BVOL(IR,IC,NLAYR)*CARAY(IR,IC,NLAYR,ISYS)
     .                   + DT*DM(IR,IC,NLAYR,ISYS))/BVOLN(IR,IC,NLAYR)
  230   CONTINUE
 
      ELSE
 
        IL=1
        DO 231 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 231 IR=IS,IE
            IF(FSM(IR,IC).LE.0.)GO TO 231
            DM(IR,IC,IL,ISYS) = - DIAG(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .       + AVECT(IR,IC,IL,1)*CARAY(IR-1,IC,IL,ISYS)
     .        + AVECT(IR,IC,IL,2)*CARAY(IR+1,IC,IL,ISYS)
     .         + AVECT(IR,IC,IL,3)*CARAY(IR,IC-1,IL,ISYS)
     .          + AVECT(IR,IC,IL,4)*CARAY(IR,IC+1,IL,ISYS)

            CONCS(IR,IC,IL) = (BVOL(IR,IC,IL)*CARAY(IR,IC,IL,ISYS)
     .                         + DT*DM(IR,IC,IL,ISYS))/BVOLN(IR,IC,IL)
  231   CONTINUE

       ENDIF
 
C
C  BOUNDARY ELEMENTS ARE UNCHANGED
C
      DO 255 I=1,NOBCALL
        IR=IBCALL(1,I)
        IC=IBCALL(2,I)
        IL=IBCALL(3,I)
        CONCS(IR,IC,IL)=CARAY(IR,IC,IL,ISYS)
 255  CONTINUE

 
C  COMPLETE REST OF INTEGRATION STEP
C        IMPLICIT VERTICAL DIFFUSION STEP  (IF NLAYR > 1)

      IF (NLAYR.EQ.1)  GO TO 380
C  I.E., THE IMPLICIT VERTICAL DIFFUSION STEP
 
      DO 340 IL=1,NLAYR
        DO 340 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 340 IR=IS,IE
            MF(IR,IC,IL) = CONCS(IR,IC,IL)*BVOLN(IR,IC,IL)
 340    CONTINUE

C        SET UP DIAGONAL AND OFF-DIAGONAL ELEMENTS

      DO 350 IC=1,NCOL
        IS = IRS(IC)
        IE = IRE(IC)
        DO 350 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 350
          AL(IR,IC,1) = 0.
          AD(IR,IC,1) = BVOLN(IR,IC,1) + DT*RZ(IR,IC,2)
          AU(IR,IC,1) = -DT*RZ(IR,IC,2)
          DO 345 IL=2,NLAYR-1
            AL(IR,IC,IL) = -DT*RZ(IR,IC,IL)
            AD(IR,IC,IL) = BVOLN(IR,IC,IL) +
     .                        DT*(RZ(IR,IC,IL) + RZ(IR,IC,IL+1))
            AU(IR,IC,IL) = -DT*RZ(IR,IC,IL+1)
  345     CONTINUE
          AL(IR,IC,NLAYR) = -DT*RZ(IR,IC,NLAYR)
          AD(IR,IC,NLAYR) = BVOLN(IR,IC,NLAYR) + DT*RZ(IR,IC,NLAYR)
          AU(IR,IC,NLAYR) = 0.0
  350 CONTINUE

C        FORWARD SWEEP OF TRIDIAGONAL SCHEME

      DO 360 IC=2,NCOL-1
        IS = IRS(IC)
        IE = IRE(IC)
        DO 360 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 360
          BETA(IR,IC,1) = AD(IR,IC,1)
          GAMMA(IR,IC,1) = MF(IR,IC,1)/BETA(IR,IC,1)
          DO 355 IL=2,NLAYR
            BETA(IR,IC,IL) = AD(IR,IC,IL)
     .                 - AL(IR,IC,IL)*AU(IR,IC,IL-1)/BETA(IR,IC,IL-1)
            GAMMA(IR,IC,IL) = (MF(IR,IC,IL)
     .                 - AL(IR,IC,IL)*GAMMA(IR,IC,IL-1))/BETA(IR,IC,IL)
  355     CONTINUE
  360 CONTINUE

C        BACKWARD SWEEP FOR SOLUTION

      DO 370 IC=2,NCOL-1
        IS = IRS(IC)
        IE = IRE(IC)
        DO 370 IR=IS,IE
          IF(FSM(IR,IC).LE.0)  GO TO 370
          CONCS(IR,IC,NLAYR) = GAMMA(IR,IC,NLAYR)
          DO 365 ILYR=2,NLAYR
            IL = NLAYR+1-ILYR
            CONCS(IR,IC,IL) = GAMMA(IR,IC,IL)
     .               - AU(IR,IC,IL)*CONCS(IR,IC,IL+1)/BETA(IR,IC,IL)
  365     CONTINUE
  370 CONTINUE

C  BOUNDARY ELEMENTS ARE UNCHANGED

      DO 375 I=1,NOBCALL
        IR=IBCALL(1,I)
        IC=IBCALL(2,I)
        IL=IBCALL(3,I)
        CONCS(IR,IC,IL)=CARAY(IR,IC,IL,ISYS)
  375 CONTINUE
 

  380 CONTINUE

      DO 470 IL=1,NLAYR
        DO 470 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 470 IR=IS,IE
            IF(FSM(IR,IC).LE.0.)GO TO 470

C        CHECK TO SEE IF NEGATIVE SOLUTIONS PERMITTED 

            IF (NEGSLN.EQ.1)   THEN
              CARAY(IR,IC,IL,ISYS) = CONCS(IR,IC,IL)
            ELSE

C        IF NOT ... CHECK TO MAKE SURE SOLUTION IS NOT FORCED THRU ZERO

              IF(CONCS(IR,IC,IL).GE.0.0)   THEN
                CARAY(IR,IC,IL,ISYS) = CONCS(IR,IC,IL)
              ELSE

C          PREVENT NEG SOLN BY QUARTERING PRESENT CONC AND PRINT WARNING

                CARAY(IR,IC,IL,ISYS) = CARAY(IR,IC,IL,ISYS)/4.
                INEGS = INEGS+1
                IF(INEGS.LE.99)  THEN
                  NEGSEG(INEGS,1) = IR
                  NEGSEG(INEGS,2) = IC
                  NEGSEG(INEGS,3) = IL
                ENDIF
                TOTMAS = TOTMAS+(CARAY(IR,IC,IL,ISYS)-CONCS(IR,IC,IL))
     +                      *BVOLN(IR,IC,IL)
              ENDIF
            ENDIF
  470 CONTINUE

      IF(INEGS.GT.0)  THEN
        CALL RCAMESS(2,0.001*TOTMAS)
        WRITE(OUT,1000)  ((NEGSEG(I,J),J=1,3),I=1,INEGS)
 1000   FORMAT(30X,12I5)
      ENDIF

  500 CONTINUE

      DO 515 IL=1,NLAYR
        DO 515 IC=2,NCOL-1
          IS = IRS(IC)
          IE = IRE(IC)
          DO 515 IR=IS,IE
            BVOL(IR,IC,IL) = BVOLN(IR,IC,IL)
  515 CONTINUE

C  PREPARE FOR NEXT TIME STEP

      TIME=TIME + DTSTEP
  
C        CHECK TO SEE IF IT IS TIME TO PRINT RESULTS
 
C  FIRST DETAILED DUMPS
       
      IF(PRNTD.EQ.0.0)  GO TO 535
      EPSLNP = DTSTEP/(PRNTD*2.)
      M=TIME/PRNTD+EPSLNP
      IF(M.LT.NND)  GO TO 535
 
C  DETAILED PRINT TIME
C  SET IDISK FOR DETAILED DUMPS
      
      IDISK=2
      NND=M+1
 
C  SECOND GLOBAL DUMPS
 
 535  EPSLNP=DTSTEP/(PRNTG*2.)
      M=TIME/PRNTG+EPSLNP
      IF (M.LT.NNG) GOTO 550
 
C  GLOBAL PRINT TIME
C  CALL PRINT ROUTINE
 
      CALL RCA09
      IDISK=IDISK+1
      NNG=M+1 

       
C  NOW PERFORM STABILITY CHECK
  
      DO 545 ISYS=1,NOSYS 
        IF(SYSBY(ISYS).EQ.1)   GO TO 545
        CKMAX = CMAX(ISYS)

        DO 540 IL=1,NLAYR
          DO 540 IC=2,NCOL-1
            IS = IRS(IC)
            IE = IRE(IC)
            DO 540 IR=IS,IE
              IF(FSM(IR,IC).LE.0)  GO TO 540
              IF(CARAY(IR,IC,IL,ISYS).GE.CKMAX)   GO TO 600
  540   CONTINUE
  545 CONTINUE

C        CHECK TO SEE IF NECESSARY TO UPDATE TIME-VARIABLE FUNCTIONS
C        FIRST POINT SOURCE LOADS 

  550 ICHANGE = 0
      IF(IPSOPT.GT.1.AND.TIME.GE.NXPST)   THEN
        CALL RCA10(BPS,NWK,NOPS,NXPST,22,SCALPS)
        ICHANGE = 1
      ENDIF

C        NONPOINT SOURCE LOADS 

      IF(INPSOPT.GT.1.AND.TIME.GE.NXNPST)   THEN
        CALL RCA10(BNPS,NWK,NONPS,NXNPST,23,SCALNPS)
        ICHANGE = 1
      ENDIF

C        FALL-LINE LOADS 

      IF(IFLOPT.GT.1.AND.TIME.GE.NXFLT)   THEN
        CALL RCA10(BFL,NWK,NOFL,NXFLT,24,SCALFL)
        ICHANGE = 1
      ENDIF

C        ATMOSPHERIC LOADS 

      IF(IATMOPT.GT.1.AND.TIME.GE.NXATMT)   THEN
        CALL RCA10(BATM,1,NOATM,NXATMT,25,SCALATM)
        ICHANGE = 1
      ENDIF

C        NEXT BOUNDARY CONDITIONS 

      IF(IBCOPT.GT.1.AND.TIME.GT.NXBCT) CALL RCA11 

C        NEXT TRANSPORT FIELDS

      IF(TIME.GE.HYDBRK(ITIMHYD))  THEN
         CALL RCA03A
         CALL RCASPLT1
         ITIMHYD = ITIMHYD+1
      ENDIF


C        CHECK FOR END OF SIMULATION...
C           IF NOT GO BACK AND TAKE NEXT INTEGRATION STEP

      IF(TIME.LE.TEND)   GO TO 40 

      ITRAK = ITRAK+1 
      IF(ITRAK.LE.NSTEP)  THEN
         DT=STEP(ITRAK)
         TEND = TBRK(ITRAK) 
         IF(DT.EQ.0.0)   THEN
            CALL RCAMESS(6,DT)
            CALL EXIT
         ENDIF
      ELSE
         TEND = 0.0
      ENDIF 

      DTSTEP = DT 
      DT = SCALT*DT 
      IF(TEND.GT.0.)   GO TO 40 

C        FINISHED...CALL FOR FINAL DUMPS

      IDISK = 1 
      CALL TUNER
      RETURN

C         STABILITY CRITERIA VIOLATED...ABEND 

  600 CALL RCAMESS(1,CKMAX) 
      WRITE(OUT,9000)  BVOLN(IR,IC,IL),DIAG(IR,IC,IL)
 9000 FORMAT(10X,'VOLUME =',E13.4,'  ,DIAGONAL =',E13.4)
      IREC = IREC+1 
      IDISK = 1 
      CALL TUNER
      RETURN
      END 
