      SUBROUTINE STOICH(NUTR,KADNUTR,PTAM,PC1,PC2,CNUTR1,CNUTR2
     .   ,ANUTRC1,ANUTRC2)

C        -STOICH- DETERMINES THE NUTRIENT TO CARBON RATIOS GIVEN ...
C  INPUTS
C     NUTR    =  TOTAL INORGANIC NUTRIENT CONCENTRATION
C     KADNUTR =  PARTITION COEFFICIENT
C     PTAM    =  PARTICULATE TOTAL ACTIVE METAL
C     PC1     =  BIOMASS CONCENTRATION OF PHYTOPLANKTON GROUP 1
C     PC2     =  BIOMASS CONCENTRATION OF PHYTOPLANKTON GROUP 2
C     CNUTR1  =  CARBON TO NUTRIENT COEFFICIENTS FOR PHYTOPLANKTON GROUP 1
C     CNUTR2  =  CARBON TO NUTRIENT COEFFICIENTS FOR PHYTOPLANKTON GROUP 2
C  OUTPUTS
C     ANUTRC1 =  NUTRIENT TO CARBON RATIO FOR PHYTOPLANKTON GROUP 1
C     ANUTRC2 =  NUTRIENT TO CARBON RATIO FOR PHYTOPLANKTON GROUP 2

      REAL   NUTR,KADNUTR,CNUTR1(3),CNUTR2(3)

      CN11=CNUTR1(1)
      CN12=CNUTR1(2)
      CN13=CNUTR1(3)
      CN21=CNUTR2(1)
      CN22=CNUTR2(2)
      CN23=CNUTR2(3)

      XL     = 0.0
      XR     = NUTR
      XMOLD  = NUTR+1.E-30
      ALGCAR = PC1+PC2

      IF (ALGCAR.GT.0.0.AND.NUTR.GT.0.0) THEN
        DF = 1./(1.+KADNUTR*PTAM)
        DO 100 I=1,10
         FXL = XL + PC1/(CN11+CN12*EXP(-MIN(10.,CN13*XL*DF)))
     .            + PC2/(CN21+CN22*EXP(-MIN(10.,CN23*XL*DF)))
     .            - NUTR
         FXR = XR + PC1/(CN11+CN12*EXP(-MIN(10.,CN13*XR*DF)))
     .            + PC2/(CN21+CN22*EXP(-MIN(10.,CN23*XR*DF)))
     .            - NUTR
         XM  = (XL*FXR-XR*FXL)/(FXR-FXL)
         FXM = XM + PC1/(CN11+CN12*EXP(-MIN(10.,CN13*XM*DF)))
     .            + PC2/(CN21+CN22*EXP(-MIN(10.,CN23*XM*DF)))
     .            - NUTR

          PCTDF = ABS((XM-XMOLD)/XMOLD)
          IF (PCTDF.LT.0.001.OR.FXM.EQ.0.) GO TO 110
          IF (FXM*FXL.LT.0.) THEN
            XR = XM
          ELSE
            XL = XM
          ENDIF

          XMOLD = XM

  100   CONTINUE

  110   CONTINUE
        IF(PC1.GE.PC2)THEN
          ANUTRC1 = (NUTR-XM
     .               -PC2/(CN21+CN22*EXP(-MIN(20.,CN23*XM*DF))))/PC1
          ANUTRC1 = MAX(ANUTRC1,1./(CN11+CN12))
          ANUTRC1 = MIN(ANUTRC1,1./CN11)
  
          ANUTRC2 = (NUTR-XM-PC1*ANUTRC1)/PC2
          ANUTRC2 = MAX(ANUTRC2,1./(CN21+CN22))
          ANUTRC2 = MIN(ANUTRC2,1./CN21)
        ELSE
          ANUTRC2 = (NUTR-XM
     .               -PC1/(CN11+CN12*EXP(-MIN(20.,CN13*XM*DF))))/PC2
          ANUTRC2 = MAX(ANUTRC2,1./(CN21+CN22))
          ANUTRC2 = MIN(ANUTRC2,1./CN21)
  
          ANUTRC1 = (NUTR-XM-PC2*ANUTRC2)/PC1
          ANUTRC1 = MAX(ANUTRC1,1./(CN11+CN12))
          ANUTRC1 = MIN(ANUTRC1,1./CN11)
        ENDIF

 
      ELSE
 
        ANUTRC1 = 0.0
        ANUTRC2 = 0.0

      ENDIF

      RETURN
      END
