cOPTION CROSSREF ON
cOPTION SYMTABLE ON
cPAGEWIDTH 132
      SUBROUTINE SED_READ
      SAVE
C
C*******************************************************************************
C
C             VARIABLE STOICHIOMETRY SEDIMENT SUBMODEL - VSSEDMODL
C             ----------------------------------------------------
C
C                      ORIGINALLY DEVELOPED FOR THE CHESAPEAKE BAY
C
C   ORGANIC FLUX TO SEDIMENT ==> DIAGENESIS ==> INORGANIC FLUX TO WATER COLUMN
C             3-G MODEL - G1=LABILE, G2=REFRACTORY, G3=CONSERVATIVE
C
C*******************************************************************************
C
C  INPUTS
C  ------
C             REQUIRED INPUTS FOR SEDIMENT SUB-MODEL
C
C  A.  PASSED TO SEDIMENT SUBROUTINE FROM WATER QUALITY SUBROUTINE
C
C    1.  OVERLYING WATER COLUMN CONCENTRATIONS OF AMMONIA, NITRATE,
C        SALINITY, DISSOLVED OXYGEN, PHOSPHORUS AND SILICA
C    2.  OVERLYING WATER COLUMN SEGMENT VOLUME (V1)
C    3.  OVERLYING WATER COLUMN SEGMENT DEPTH (BL(ISEG,3))
C    4.  OVERLYING WATER COLUMN SEGMENT TEMPERATURE
C    8.  ALGAL PHOSPHORUS TO CARBON, NITROGEN TO CARBON, AND SILICA
C        TO CARBON RATIOS
C    9.  DEPTH OF OVERLYING WATER COLUMN
C
C  B.  INPUTS SUPPLIED VIA DIRECT INPUT TO THE SEDIMENT SUBROUTINE
C
C     VARIABLE NAMES        DESCRIPTION                            UNITS
C     --------------   ----------------------------------------    -----
C           HSED       DEPTH OF SEDIMENT LAYER                        CM
C          DIFFT       WATER COLUMN-SEDIMENT LAYER DIFFUSION     CM2/SEC
C                      COEFFICIENT FOR TEMPERATURE
C         SALTSW       SALINITY CONCENTRATION FOR DETERMINING        PPT
C                      WHETHER METHANE OR SULFIDE SOD FORMULATION
C                      IS TO BE USED
C         XXSNET       NET SETTLING VELOCITY FROM WATER COLUMN
C
C  POM FLUX RELATED
C                      FRACTIONS OF G1, G2, AND G3 CONTAINED IN ...
C        FRPPH1(3)     ALGAL GROUP NO.1 PHOSPHORUS
C        FRPPH2(3)     ALGAL GROUP NO.2 PHOSPHORUS
C        FRPOP(3)      NON-ALGAL PARTICULATE ORGANIC PHOSPHORUS
C        FRNPH1(3)     ALGAL GROUP NO.1 NITROGEN
C        FRNPH2(3)     ALGAL GROUP NO.2 NITROGEN
C        FRPON(3)      NON-ALGAL PARTICULATE ORGANIC NITROGEN
C        FRCPH1(3)     ALGAL GROUP NO.1 CARBON
C        FRCPH2(3)     ALGAL GROUP NO.2 CARBON
C        FRPOC(3)      NON-ALGAL PARTICULATE ORGANIC CARBON
C
C  DIAGENESIS RELATED
C        KPDIAG(3)     REACTION RATES FOR POP G1, G2, AND G3        /DAY
C        DPTHTA(3)     TEMPERATURE THETAS FOR POP G1, G2, AND G3
C        KNDIAG(3)     REACTION RATES FOR PON G1, G2, AND G3        /DAY
C        DNTHTA(3)     TEMPERATURE THETAS FOR PON G1, G2, AND G3
C        KCDIAG(3)     REACTION RATES FOR POC G1, G2, AND G3        /DAY
C        DCTHTA(3)     TEMPERATURE THETAS FOR POC G1, G2, AND G3
C                      
C
C  FLUX-FLUX RELATED
C        VPMIX(NSED)   SOLID-PHASE MIXING RATE                  M**2/DAY
C        VDMIX(NSED)   DISSOLVED-PHASE MIXING RATE              M**2/DAY
C           M1         CONCENTRATION OF SS IN LAYER 1            KG SS/L
C           M2         CONCENTRATION OF SS IN LAYER 2            KG SS/L
C          PIE1S       PARTITION COEFFICIENT ASSOCIATED             L/KG
C                      SULFIDE SORPTION IN LAYER 1
C          PIE2S       PARTITION COEFFICIENT ASSOCIATED             L/KG
C                      SULFIDE SORPTION IN LAYER 2
C          KLN2        NITRATE TRANSFER COEFFICIENT BETWEEN        M/DAY
C                        SEDIMENT LAYERS 1 AND 2
C        REACTION VELOCITIES FOR...
C          KAPPD1      DISSOLVED SULFIDE IN LAYER 1                M/DAY
C          KAPPP1      PARTICULATE SULFIDE IN LAYER 1              M/DAY
C          KAPPNH4     AMMONIA NITROGEN                            M/DAY
C          KP1NO3      AEROBIC DENITRIFICATION                     M/DAY
C          KP2NO3      ANAEROBIC DENITRIFICATION                   M/DAY
C          KAPPC       METHANE                                     M/DAY
C          KAPPD       METHANE DIFFUSION MASS TRANSFER             M/DAY
C
C
C  OUTPUT
C  ------
C     THE SUBROUTINE RETURNS FLUXES FOR....
C      JSOD, JAQSOD, JCH4AQ AND JCH4G [GM O2*/M2-DAY]
C      JNH4, JPO4, JNO3 AND JSI [MG/M2-DAY]
C
C***********************************************************************
C
      INCLUDE  'RCACM'
      INCLUDE  'SEDCM'

      CHARACTER   SSNAME(17)*27,FRNAME(9)*24,SEDFILNA*40
      INTEGER  ISEDPRNT(3)
      REAL  PGFRAC(3),NGFRAC(3),CGFRAC(3)

      EXTERNAL  SEDF

C        STATE-VARIABLES
      REAL
     .      SAL(NROW,NCOL,NLAYR)   , PHYT1(NROW,NCOL,NLAYR)
     .    , PHYT2(NROW,NCOL,NLAYR) , RPOP(NROW,NCOL,NLAYR)
     .    , LPOP(NROW,NCOL,NLAYR)  , RDOP(NROW,NCOL,NLAYR)
     .    , LDOP(NROW,NCOL,NLAYR)  , PO4T(NROW,NCOL,NLAYR)
     .    , RPON(NROW,NCOL,NLAYR)  , LPON(NROW,NCOL,NLAYR)
     .    , RDON(NROW,NCOL,NLAYR)  , LDON(NROW,NCOL,NLAYR)
     .    , NH3T(NROW,NCOL,NLAYR)  , NO23(NROW,NCOL,NLAYR)
     .    , BSI(NROW,NCOL,NLAYR)   , SIT(NROW,NCOL,NLAYR)
     .    , RPOC(NROW,NCOL,NLAYR)  , LPOC(NROW,NCOL,NLAYR)
     .    , RDOC(NROW,NCOL,NLAYR)  , LDOC(NROW,NCOL,NLAYR)
     .    , REDOC(NROW,NCOL,NLAYR) , EXDOC(NROW,NCOL,NLAYR)
     .    , O2EQ(NROW,NCOL,NLAYR)  , DO(NROW,NCOL,NLAYR)
     .    , TAM(NROW,NCOL,NLAYR)
 

C        REAL AND LABELED COMMON FOR LINKING WATER COLUMN AND SEDIMENT SUBMODELS
      REAL   NCRB1,NCRB2,NH3
      REAL   SODS(NROW,NCOL),FLXHS(NROW,NCOL),FLXNH3(NROW,NCOL)
     .      ,FLXPO4(NROW,NCOL),FLXNO3(NROW,NCOL),FLXSI(NROW,NCOL)
     .      ,FLXCH4AQ(NROW,NCOL),FLXCH4G(NROW,NCOL)
      COMMON /FLUXES/
     .   PO4(NROW,NCOL,NLAYR),NH3(NROW,NCOL,NLAYR),SI(NROW,NCOL,NLAYR)
     .  ,PCRB1(NROW,NCOL,NLAYR),PCRB2(NROW,NCOL,NLAYR)
     .  ,NCRB1(NROW,NCOL,NLAYR),NCRB2(NROW,NCOL,NLAYR)
     .  ,SCRB1(NROW,NCOL,NLAYR),SCRB2(NROW,NCOL,NLAYR)
     .  ,DEPFLUX(NROW,NCOL,12),CFLUXS(NROW,NCOL,8)
      EQUIVALENCE
     .   (CFLUXS(1,1,1),SODS(1,1))     , (CFLUXS(1,1,2),FLXHS(1,1))  
     .  ,(CFLUXS(1,1,3),FLXCH4AQ(1,1)) , (CFLUXS(1,1,4),FLXCH4G(1,1))
     .  ,(CFLUXS(1,1,5),FLXNH3(1,1))   , (CFLUXS(1,1,6),FLXNO3(1,1)) 
     .  ,(CFLUXS(1,1,7),FLXPO4(1,1))   , (CFLUXS(1,1,8),FLXSI(1,1))

C 
      EQUIVALENCE
     .   (CARAY(1,1,1,1),SAL(1,1,1))    , (CARAY(1,1,1,2),PHYT1(1,1,1))
     . , (CARAY(1,1,1,3),PHYT2(1,1,1))  , (CARAY(1,1,1,4),RPOP(1,1,1))
     . , (CARAY(1,1,1,5),LPOP(1,1,1))   , (CARAY(1,1,1,6),RDOP(1,1,1))
     . , (CARAY(1,1,1,7),LDOP(1,1,1))   , (CARAY(1,1,1,8),PO4T(1,1,1))
     . , (CARAY(1,1,1,9),RPON(1,1,1))   , (CARAY(1,1,1,10),LPON(1,1,1))
     . , (CARAY(1,1,1,11),RDON(1,1,1))  , (CARAY(1,1,1,12),LDON(1,1,1))
     . , (CARAY(1,1,1,13),NH3T(1,1,1))  , (CARAY(1,1,1,14),NO23(1,1,1))
     . , (CARAY(1,1,1,15),BSI(1,1,1))   , (CARAY(1,1,1,16),SIT(1,1,1))
     . , (CARAY(1,1,1,17),RPOC(1,1,1))  , (CARAY(1,1,1,18),LPOC(1,1,1))
     . , (CARAY(1,1,1,19),RDOC(1,1,1))  , (CARAY(1,1,1,20),LDOC(1,1,1))
     . , (CARAY(1,1,1,21),REDOC(1,1,1)) , (CARAY(1,1,1,22),EXDOC(1,1,1))
     . , (CARAY(1,1,1,23),O2EQ(1,1,1))  , (CARAY(1,1,1,24),DO(1,1,1))
     . , (CARAY(1,1,1,25),TAM(1,1,1))

      DATA  SSNAME/
     .  '    Temperature [Deg C]    ' , '     G1 POP [mg P/m**3]    ' ,
     .  '     G1 PON [mg N/m**3]    ' , '     G1 POC [mg C/m**3]    ' ,
     .  '     G2 POP [mg P/m**3]    ' , '     G2 PON [mg N/m**3]    ' ,
     .  '     G2 POC [mg C/m**3]    ' , '     G3 POP [mg P/m**3]    ' ,
     .  '     G3 PON [mg N/m**3]    ' , '     G3 POC [mg C/m**3]    ' ,
     .  '  Biogenic Si [mg Si/m**3] ' , ' Dissolved PO4 [mg P/m**3] ' ,
     .  ' Dissolved NH4 [mg N/m**3] ' , ' Dissolved NO3 [mg N/m**3] ' ,
     .  'Dissolved H2S [mg O2*/m**3]' , ' Dissolved Si [mg Si/m**3] ' ,
     .  '      Benthic Stress       ' /

      DATA   FRNAME/
     .    'WINTER PHYTO PHOSPHORUS ','SUMMER PHYTO PHOSPHORUS '
     .   ,'DETRITAL ORG PHOSPHORUS '
     .   ,'WINTER PHYTO NITROGEN   ','SUMMER PHYTO NITROGEN   '
     .   ,'DETRITAL ORG NITROGEN   '
     .   ,'WINTER PHYTO CARBON     ','SUMMER PHYTO CARBON     '
     .   ,'DETRITAL ORG CARBON     '/
C             INITIALIZATION

C        OPEN FILES FOR SEDIMENT DUMPS

      OPEN(14,FILE='RCAF14',FORM='UNFORMATTED')
      OPEN(16,FILE='RCAFICSED',FORM='UNFORMATTED')

C        WRITE -FSM- (LAND-MASK) TO RCAF14
      WRITE(14)  FSM

C        SET NUMBER OF LAYERS FOR SEDIMENT MODEL => 2
C        (AEROBIC + 1 ANAEROBIC)
      
      nsedlyr = 2
      xnsedlyr = nsedlyr

      READ(IN,1000)  SEDFILNA
 1000 FORMAT(A40)
      OPEN(30,FILE=SEDFILNA,FORM='FORMATTED')

C        PRINT CONTROLS
      READ(30,1100)   ISEDPRNT
 1100 FORMAT(3I5)

C             GET MODEL COEFFICIENTS REQUIRED FOR SEDIMENT MODEL

C        SEDIMENT DEPTH
      DO 10 IC=1,NCOL
       READ(30,1200,ERR=900) (HSED(IR,IC),IR=1,NROW)
 1200 FORMAT(8F10.0)
   10 CONTINUE

C        COMPUTE SEDIMENT VOLUMES ==> M**3

      DO 20 IC=1,NCOL
       DO 20 IR=1,NROW

C        SEDIMENT VOLUME (M**3)
        BSVOL(IR,IC) = 0.01*HSED(IR,IC)*XAZ(IR,IC)

C        SET BFORMAX
        BFORMAXS(IR,IC) = 0.0
        ISWBNTHS(IR,IC) = 0

   20 CONTINUE

      WRITE(OUT,1010)
 1010 FORMAT(///34X,
     .'SEDIMENT-WATER COLUMN LINKAGES AND SEDIMENT DEPTHS AND VOLUMES'/)
       WRITE(OUT,1020)
 1020    FORMAT(10X,'SEDIMENT SEGMENT DEPTHS (CM)')
c      CALL RCAPRNT(HSED,1,1)
       WRITE(OUT,1021)
 1021    FORMAT(10X,'SEDIMENT VOLUMES (M**3)')
c      CALL RCAPRNT(BSVOL,1,1)

C        CONVERT DEPTHS TO METERS

      DO 30 IC=1,NCOL
       DO 30 IR=1,NROW
        HSED(IR,IC) = 0.01*HSED(IR,IC)
   30 CONTINUE
 
C        READ SEDIMENT INITIAL CONDITIONS
      WRITE(OUT,1040)
 1040 FORMAT(////38X,
     .  'S E D I M E N T   I N I T I A L   C O N D I T I O N S'/)

C        INITIAL CONDITIONS

      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CTEMP,SSNAME(1))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPOP(1,1,1),SSNAME(2))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPON(1,1,1),SSNAME(3))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPOC(1,1,1),SSNAME(4))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPOP(1,1,2),SSNAME(5))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPON(1,1,2),SSNAME(6))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPOC(1,1,2),SSNAME(7))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPOP(1,1,3),SSNAME(8))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPON(1,1,3),SSNAME(9))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPOC(1,1,3),SSNAME(10))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),CPOS,SSNAME(11))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),PO4T2TM1S,SSNAME(12))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),NH4T2TM1S,SSNAME(13))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),NO3T2TM1S,SSNAME(14))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),HST2TM1S,SSNAME(15))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),SIT2TM1S,SSNAME(16))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),BNTHSTR1S,SSNAME(17))

      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),PO4T1TM1S,SSNAME(12))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),NH4T1TM1S,SSNAME(13))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),NO3T1TM1S,SSNAME(14))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),SIT1TM1S,SSNAME(16))
      CALL ICREAD(NROW,NCOL,OUT,ISEDPRNT(1),HST1TM1S,SSNAME(15))

      if(cycle.eq.1)    then
         rewind(16)
         read(16)   ctemp,cpop,cpon,cpoc,cpos
     .           ,po4t2tm1s,nh4t2tm1s,no3t2tm1s
     .           ,hst2tm1s,sit2tm1s,bnthstr1s
     .           ,po4t1tm1s,nh4t1tm1s,no3t1tm1s
     .           ,sit1tm1s,hst1tm1s
      endif

C        TRANSFER I.C. TO APPROPRIATE VECTORS
      CALL LOADSED

C        TEMPERATURE DIFFUSION COEFFICIENT

      READ(30,1200,ERR=900)  DIFFT
      DIFFT = 0.0001*86400.*DIFFT
      READ(30,1200,ERR=900)  SALTSW
      READ(30,1200,ERR=900)  FRPPH1
      READ(30,1200,ERR=900)  FRPPH2
c     READ(30,1200,ERR=900)  FRPPH3
      READ(30,1200,ERR=900)  FRPOP
      READ(30,1200,ERR=900)  FRNPH1
      READ(30,1200,ERR=900)  FRNPH2
c     READ(30,1200,ERR=900)  FRNPH3
      READ(30,1200,ERR=900)  FRPON
      READ(30,1200,ERR=900)  FRCPH1
      READ(30,1200,ERR=900)  FRCPH2
c     READ(30,1200,ERR=900)  FRCPH3
      READ(30,1200,ERR=900)  FRPOC
      READ(30,1200,ERR=900)  (KPDIAG(I),DPTHTA(I),I=1,3)
      READ(30,1200,ERR=900)  (KNDIAG(I),DNTHTA(I),I=1,3)
      READ(30,1200,ERR=900)  (KCDIAG(I),DCTHTA(I),I=1,3)
      READ(30,1200,ERR=900)  KSI,THTASI

      WRITE(OUT,1210)  DIFFT
      WRITE(OUT,1215)  SALTSW
      WRITE(OUT,1220)
      WRITE(OUT,1230)  FRNAME(1),FRPPH1
      WRITE(OUT,1230)  FRNAME(2),FRPPH2
      WRITE(OUT,1230)  FRNAME(3),FRPOP
      WRITE(OUT,1230)  FRNAME(4),FRNPH1
      WRITE(OUT,1230)  FRNAME(5),FRNPH2
      WRITE(OUT,1230)  FRNAME(6),FRPON
      WRITE(OUT,1230)  FRNAME(7),FRCPH1
      WRITE(OUT,1230)  FRNAME(8),FRCPH2
      WRITE(OUT,1230)  FRNAME(9),FRPOC
      WRITE(OUT,1240)  (KPDIAG(I),DPTHTA(I),I=1,3),(KNDIAG(I),
     .   DNTHTA(I),I=1,3),(KCDIAG(I),DCTHTA(I),I=1,3),KSI,THTASI

 1210 FORMAT(
     .   //30X,'TEMPERATURE DIFFUSION COEFFICIENT ',E10.3,' CM**2/SEC')
 1215 FORMAT(
     .   //31X,'IF SALINITY LESS THAN ',F10.3,' PPT, METHANE FORMED')
 1220 FORMAT(//30X,'PARTICULATE ORGANIC MATTER G-MODEL SPLITS'/ 10X,
     .   'FRACTION OF....',5X,'RECYCLED TO',5X,'G1',5X,'G2',5X,'G3')
 1230 FORMAT(6X,A24,11X,3F7.2)
 1240 FORMAT(//30X,'DIAGENESIS RATES (/DAY)  TEMP CORR FACTOR'/
     .   30X,'PHOSPHORUS'/
     .   39X,'G1',E11.3,5X,F7.3/39X,'G2',E11.3,5X,F7.3/
     .   39X,'G3',E11.3,5X,F7.3/
     .   30X,'NITROGEN'/
     .   39X,'G1',E11.3,5X,F7.3/39X,'G2',E11.3,5X,F7.3/
     .   39X,'G3',E11.3,5X,F7.3/
     .   30X,'CARBON'/
     .   39X,'G1',E11.3,5X,F7.3/39X,'G2',E11.3,5X,F7.3/
     .   39X,'G3',E11.3,5X,F7.3/
     .   30X,'SILICA'/
     .   41X,E11.3,5X,F7.3)

C        READ SEDIMENTATION AND PARTICLE MIXING RATES
      WRITE(OUT,1050)
 1050 FORMAT(////38X,
     .  'S E D I M E N T A T I O N   A N D   P A R T I C L E   M I X I N
     . G   R A T E S'//)

      CALL VREAD(NROW,NCOL,OUT,ISEDPRNT(1),VSED,1)
      CALL VREAD(NROW,NCOL,OUT,ISEDPRNT(1),VPMIX,2)
      CALL VREAD(NROW,NCOL,OUT,ISEDPRNT(1),VDMIX,3)

C        COMPUTE G FRACTION SPLITS FOR REFRACTORY POM
      DO 35 I=2,3
       PGFRAC(I) = FRPOP(I)/(FRPOP(2)+FRPOP(3))
       NGFRAC(I) = FRPON(I)/(FRPON(2)+FRPON(3))
       CGFRAC(I) = FRPOC(I)/(FRPOC(2)+FRPOC(3))
   35 CONTINUE

C        CONVERT CM/YEAR TO M/DAY

      DO 37 IC=1,NCOL
       DO 37 IR=1,NROW
        VSED(IR,IC) = 2.73791E-5*VSED(IR,IC)
   37 CONTINUE

      READ(30,1200,ERR=900)  m1,m2,thtaDp,thtaDd
      WRITE(OUT,1270)   M1,M2,THTADP,THTADD
 1270 FORMAT(//35X,'ADDITIONAL CONSTANTS'/
     .  /30X,'m1........',F8.2,' KG/L'
     .  /30X,'m2........',F8.2,' KG/L'
     .  /30X,'thtaDp....',F8.3,
     .  /30X,'thtaDd....',F8.3)
      READ(30,1200,ERR=900)  Dd0,thtaDd0
      WRITE(OUT,1271)   DD0,THTADD0
 1271 FORMAT(
     .  /30X,'Dd0.......',F8.3,' M/DAY'
     .  /30X,'thtaDd0...',F8.3)
      READ(30,1200,ERR=900)  kappnh4s,pienh4,thtanh4s,kmnh4,thtakmnh4,
     .     kmnh4o2
      WRITE(OUT,1272)  kappnh4s,pienh4,thtanh4s,kmnh4,thtakmnh4,kmnh4o2
 1272 FORMAT(
     .  /30X,'Saltwater values'
     .  /30X,'kappnh4s..',F8.3,' M/DAY'
     .  /30X,'pienh4....',F8.3,' L/KG'
     .  /30X,'thtanh4s..',F8.3,
     .  /30X,'kmnh4.....',F8.3,' MG N/M**3'
     .  /30X,'thtakmnh4.',F8.3,
     .  /30X,'kmnh4o2...',F8.3,' MG O2/L')
      READ(30,1200,ERR=900)  kappnh4f,thtanh4f
      WRITE(OUT,1273)  kappnh4f,thtanh4f
 1273 FORMAT(
     .  /30X,'Freshwater values'
     .  /30X,'kappnh4f..',F8.3,' M/DAY'
     .  /30X,'thtanh4f..',F8.3)
      READ(30,1200,ERR=900)  kapp1no3s,k2no3s,thtano3s
      WRITE(OUT,1274)  kapp1no3s,k2no3s,thtano3s
 1274 FORMAT(
     .  /30X,'Saltwater values'
     .  /30X,'kapp1no3s.',F8.3,' M/DAY'
     .  /30X,'k2no3s....',F8.3,' /DAY'
     .  /30X,'thtano3s..',F8.3)
      READ(30,1200,ERR=900)  kapp1no3f,k2no3f,thtano3f
      WRITE(OUT,1275)  kapp1no3f,k2no3f,thtano3f
 1275 FORMAT(
     .  /30X,'Freshwater values'
     .  /30X,'kapp1no3f.',F8.3,' M/DAY'
     .  /30X,'k2no3f....',F8.3,' /DAY'
     .  /30X,'thtano3f..',F8.3)
      READ(30,1200,ERR=900)  kappd1,kappp1,pie1s,pie2s,thtapd1,kmhso2
      WRITE(OUT,1276)  kappd1,kappp1,pie1s,pie2s,thtapd1,kmhso2
 1276 FORMAT(
     .  /30X,'kappd1....',F8.3,' M/DAY'
     .  /30X,'kappp1....',F8.3,' M/DAY'
     .  /30X,'pie1s.....',F8.3,' L/KG'
     .  /30X,'pie2s.....',F8.3,' L/KG'
     .  /30X,'thtapd1...',F8.3,
     .  /30X,'kmhso2....',F8.3,' MG O2/L')
      READ(30,1200,ERR=900)  csisat,pie1si,pie2si,ksi,thtasi,kmpsi,
     .     o2critsi,jsidetr
      WRITE(OUT,1278)  csisat,pie1si,pie2si,ksi,thtasi,kmpsi,
     .     o2critsi,jsidetr
 1278 FORMAT(
     .  /30X,'csisat....',F8.3,' MG SI/M**3'
     .  /30X,'pie1si....',F8.3,' L/KG'
     .  /30X,'pie2si....',F8.3,' L/KG'
     .  /30X,'ksi.......',F8.3,' /DAY'
     .  /30X,'thtasi....',F8.3,
     .  /30X,'kmpsi.....',F8.3,' MG SI/M**3'
     .  /30X,'o2critsi..',F8.3,' MG O2/L'
     .  /30X,'jsidetr...',F8.1,' MG SI/M**2')
      READ(30,1200,ERR=900)  pie1po4m,pie1po4n,o2crit,kmo2Dp
      WRITE(OUT,1280)  pie1po4m,pie1po4n,o2crit,kmo2Dp
 1280 FORMAT(
     .  /30X,'pie1po4m..',F8.3,' L/KG'
     .  /30X,'pie1po4n..',F8.3,' L/KG'
     .  /30X,'o2crit....',F8.3,' MG O2/L'
     .  /30X,'kmo2Dp....',F8.3,' MG O2/L')
      READ(30,1200,ERR=900)  tempbnth,kbnthstr,klbnth,Dpmin
      WRITE(OUT,1282)  tempbnth,kbnthstr,klbnth,Dpmin
 1282 FORMAT(
     .  /30X,'tempbnth..',F8.3,' DEG C'
     .  /30X,'kbnthstr..',F8.3,' /DAY'
     .  /30X,'klbnth....',F8.3,
     .  /30X,'Dpmin.....',F8.3,' M**2/DAY')
      READ(30,1200,ERR=900)  kappch4,thtach4
      WRITE(OUT,1284)  kappch4,thtach4
 1284 FORMAT(
     .  /30X,'kappch4...',F8.3,' M/DAY'
     .  /30X,'thtach4...',F8.3)

C        SET MIN AND MAX RANGES FOR ROOT FINDER

C     SODMIN = 0.0001
      SODMIN = 0.100E-20
      SODMAX = 100.
      ITER = 50
      EPS = 0.00005

      DO 46 IC=1,NCOL
       DO 46 IR=1,NROW
        IF(FSM(IR,IC).EQ.1)  THEN
C        SET UP REACTION RATES IN TABLE LOOK-UP FORM
C        (ASSUMES ALL SEDIMENT DEPTHS AND MIXING RATES ARE THE SAME)
         H20 = HSED(IR,IC)
         DELTAZ = HSED(IR,IC)
         w2 = VSED(IR,IC)
         Dp = VPMIX(IR,IC)
         Dd = VDMIX(IR,IC)
         CALL SEDTS2TH
C        RETURN TO MAIN PROGRAM AFTER INITIALIZATION
         RETURN
        ENDIF
   46 CONTINUE

   47 RETURN


      ENTRY  SED_FLUX


C        PASS WQ TIME-STEP (IN DAYS) TO SEDIMENT SUBR

      IF(DTWQ.NE.0.0)  THEN
        DELTAT = DTWQ
      ELSE
        DELTAT = DT
      ENDIF

C      COMPUTE TEMPERATURE AND BENTHIC-STRESS DEPENDENT DEPOSITION RATES

      DO 55 IC=1,NCOL
      DO 55 IR=1,NROW
      IF(FSM(IR,IC).NE.1) GO TO 55
      TEMPD = CTEMP(IR,IC)
      ITEMP = 10.*TEMPD+1
      TERM = (1.-KBNTHSTR*BNTHSTR1S(IR,IC))*ZW12NOM(ITEMP)
   55 CONTINUE

C             PARTICULATE ORGANIC FLUX TO SEDIMENT

c  c$doacross local(ic,ir,term,ppflx,pnflx,pcflx,sarea)
      DO 100 IC=1,NCOL
      DO 100 IR=1,NROW
      IF(FSM(IR,IC).NE.1) GO TO 100

C        FLUX RATE FROM WATER COLUMN = DEPOSITION RATE * CONC
C                    M/DAY * MG/L * 1000 L/M**3 ==> MG/M2-DAY 

      TERM = 1000./XAZ(IR,IC)
      SSFWS(IR,IC) = 0.
      PCFWS(IR,IC) = 0.
      PNFWS(IR,IC) = 0.
      PPFWS(IR,IC) = 0.
      PSFWS(IR,IC) = 0.

      DO 60 I=1,3

C USE LABILE PARTICULATES AS G1, SPLIT REFRACTORY BETWEEN G2, G3

      IF (I .EQ. 1) THEN
        PPFLX = TERM*DEPFLUX(IR,IC,4)
        PNFLX = TERM*DEPFLUX(IR,IC,6)
        PCFLX = TERM*DEPFLUX(IR,IC,9)
      ELSE
        PPFLX = TERM*DEPFLUX(IR,IC,3)*PGFRAC(I)
        PNFLX = TERM*DEPFLUX(IR,IC,5)*NGFRAC(I)
        PCFLX = TERM*DEPFLUX(IR,IC,8)*CGFRAC(I)
      END IF

C          PHOSPHORUS

      FLXPOP(IR,IC,I)
     .        = TERM*PCRB1(IR,IC,NLAYR)*FRPPH1(I)*DEPFLUX(IR,IC,1)
     .        + TERM*PCRB2(IR,IC,NLAYR)*FRPPH2(I)*DEPFLUX(IR,IC,2)
     .        + PPFLX

C        NITROGEN

      FLXPON(IR,IC,I)
     .        = TERM*NCRB1(IR,IC,NLAYR)*FRNPH1(I)*DEPFLUX(IR,IC,1)
     .        + TERM*NCRB2(IR,IC,NLAYR)*FRNPH2(I)*DEPFLUX(IR,IC,2)
     .        + PNFLX

C        CARBON

      FLXPOC(IR,IC,I) = TERM*FRCPH1(I)*DEPFLUX(IR,IC,1)
     .                + TERM*FRCPH2(I)*DEPFLUX(IR,IC,2)
     .                + PCFLX

   60 CONTINUE

C        SILICA

      FLXPOS(IR,IC) = TERM*SCRB1(IR,IC,NLAYR)*DEPFLUX(IR,IC,1)
     .              + TERM*SCRB2(IR,IC,NLAYR)*DEPFLUX(IR,IC,2)
     .              + TERM*DEPFLUX(IR,IC,7)

C ADD ADSORBED PHOSPHATE AND SILICA TO LABILE POOL

      FLXPOP(IR,IC,1) = FLXPOP(IR,IC,1)+TERM*DEPFLUX(IR,IC,10)
      FLXPOS(IR,IC) = FLXPOS(IR,IC)+TERM*DEPFLUX(IR,IC,11)

C SUM PARTICULATE FLUXES TO SEDIMENTS, NEGATIVE INTO SEDIMENTS

      SAREA=1./XAZ(IR,IC)
c     SAREA=XAZ(IR,IC)/1000.
      DO 62 I = 1,3
        PPFWS(IR,IC) = PPFWS(IR,IC) - FLXPOP(IR,IC,I)*SAREA
        PNFWS(IR,IC) = PNFWS(IR,IC) - FLXPON(IR,IC,I)*SAREA
        PCFWS(IR,IC) = PCFWS(IR,IC) - FLXPOC(IR,IC,I)*SAREA
 62   CONTINUE
        PSFWS(IR,IC) = PSFWS(IR,IC) - FLXPOS(IR,IC)*SAREA

100   CONTINUE

      DO 200 IC=1,NCOL
      DO 200 IR=1,NROW
      IF(FSM(IR,IC).NE.1)  GO TO 200

C        LOAD CONCS AT TIME LEVEL   T - DELTAT

      CALL LOADTM1

C       -DIAGX- UNITS ARE [GM/M**3-DAY]
C       -JDIAGX- UNITS ARE [MG/M2/DAY]

C        SEDIMENT DEPTH

      H2 = HSED(IR,IC)
      DELTAZ = H2

C        SEDIMENTATION AND MIXING RATES

      w2 = VSED(IR,IC)
      Dp = VPMIX(IR,IC)
      Dd = VDMIX(IR,IC)

C        SEDIMENT TEMPERATURE

      TEMPD = CTEMP(IR,IC)
      STP20 = TEMPD - 20.

C        CONVERT OVERLYING WATER COLUMN CONCENTRATIONS INTO MG/M**3

      PO40 = PO4(IR,IC,NLAYR)*1000.
      NH40 = NH3(IR,IC,NLAYR)*1000.
      NO30 = NO23(IR,IC,NLAYR)*1000.
      SI0  = SI(IR,IC,NLAYR)*1000.

C        OVERLYING WATER COLUMN OXYGEN (WITH 0.001 MG/L FLOOR)

      O20 = AMAX1(DO(IR,IC,NLAYR),0.001)

C        OVERLYING WATER COLUMN SALINITY

      SAL0 = SAL(IR,IC,NLAYR)

C        DEPTH OF OVERLYING WATER COLUMN

      OWCDPH = HBAR(IR,IC)
c     OWCDPH = ZD(IR,IC,NLAYR)+HBAR(IR,IC)

C        METHANE SATURATION

      CH4SAT = 0.099*(1.+(OWCDPH+HSED(IR,IC))/10.)*0.9759**STP20

c-- evaluate the temperature dependent coefficients
      error=sedf(sodmin,0)
      call diagen

c-- solve the nh4,no3,sod equations
      sod=zbrent(sedf,sodmin,sodmax,eps,ierr)
      if(ierr.ne.0)  then
         write(out,9000)   time,ierr,ir,ic,sodmin,sodmax
 9000    format(/
     .      5x,'Zbrent failure at time =',f8.3,' with ierr=',i2/
     .      5x,'in sediment segment ir=',i5,' ic=',i5/
     .      5x,'(sodmin,sodmax=',f6.4,f6.1,')'/
     .      5x,'Program termination follows diagnostic dumps')
         if(ierr.eq.2)  then
         write(out,9900)  time,ctemp(ir,ic),pop1r,pop2r,pop3r,popr
 9900    format(/1x,' time,ctemp(ir,ic),pop1r,pop2r,pop3r,popr'/8e10.3)
         write(out,9901)  pon1r,pon2r,pon3r,ponr,poc1r,poc2r,poc3r,pocr
 9901    format(/1x,' pon1r,pon2r,pon3r,ponr,poc1r,poc2r,poc3r,pocr'/
     .          8e10.3)
         write(out,9902)  po4t2r,hst2r,sit2r,psiavr
 9902    format(/1x,' po4t2r,hst2r,sit2r,psiavr'/8e10.3)
         write(out,9903)
     .        (flxpop(ir,ic,1)+flxpop(ir,ic,2)+flxpop(ir,ic,3))
     .       ,(flxpon(ir,ic,1)+flxpon(ir,ic,2)+flxpon(ir,ic,3))
     .       ,(flxpoc(ir,ic,1)+flxpoc(ir,ic,2)+flxpoc(ir,ic,3))
 9903    format(/1x,' flxpop,flxpon,flxpoc'/8e10.3)
         write(out,9904)  o20,csod,sod,s,h1,h2,h1dot,h2dot
 9904    format(/1x,' o20,csod,sod,s,h1,h2,h1dot,h2dot'/8e10.3)
         write(out,9905)  jp,jn,jc,jo2nh4,xjco2av,xjc1av
 9905    format(/1x,' jp,jn,jc,jo2nh4,xjco2av,xjc1av'/8e10.3)
         write(out,9906)  jpo4,jnh4,jno3,jhs,jsi,jch4aq,jch4g,bnthstr
 9906    format(/1x,' jpo4,jnh4,jno3,jhs,jsi,jch4aq,jch4g,bnthstr'/
     .          8e10.3)
         write(out,9907)  po40,po41,po42,po4t2,nh40,nh41,nh42,nh4t2
 9907    format(/1x,' po40,po41,po42,po4t2,nh40,nh41,nh42,nh4t2'/8e10.3)
         write(out,9908)  no30,no31,no32,no3t2,hs1,hs2,hst2
 9908    format(/1x,' no30,no31,no32,no3t2,hs1,hs2,hst2'/8e10.3)
         write(out,9909)  si0,si1,si2,sit2
 9909    format(/1x,' si0,si1,si2,sit2'/8e10.3)
         call exit
         else
         error=sedf(sodmin,1)
         write(out,9889)  time,sodmin,error
 9889    format(/5x,'Zbrent diagnostics at time =',f8.3,
     .     ' for sod =',f8.4,' error =',e12.3/)
         write(out,9900)  time,ctemp(ir,ic),pop1r,pop2r,pop3r,popr
         write(out,9901)  pon1r,pon2r,pon3r,ponr,poc1r,poc2r,poc3r,pocr
         write(out,9902)  po4t2r,hst2r,sit2r,psiavr
         write(out,9903)
     .        (flxpop(ir,ic,1)+flxpop(ir,ic,2)+flxpop(ir,ic,3))
     .       ,(flxpon(ir,ic,1)+flxpon(ir,ic,2)+flxpon(ir,ic,3))
     .       ,(flxpoc(ir,ic,1)+flxpoc(ir,ic,2)+flxpoc(ir,ic,3))
         write(out,9904)  o20,csod,sod,s,h1,h2,h1dot,h2dot
         write(out,9905)  jp,jn,jc,jo2nh4,xjco2av,xjc1av
         write(out,9906)  jpo4,jnh4,jno3,jhs,jsi,jch4aq,jch4g,bnthstr
         write(out,9907)  po40,po41,po42,po4t2,nh40,nh41,nh42,nh4t2
         write(out,9908)  no30,no31,no32,no3t2,hs1,hs2,hst2
         write(out,9909)  si0,si1,si2,sit2
         error=sedf(sodmax,1)
         write(out,9889)  time,sodmax,error
         write(out,9900)  time,ctemp(ir,ic),pop1r,pop2r,pop3r,popr
         write(out,9901)  pon1r,pon2r,pon3r,ponr,poc1r,poc2r,poc3r,pocr
         write(out,9902)  po4t2r,hst2r,sit2r,psiavr
         write(out,9903)
     .        (flxpop(ir,ic,1)+flxpop(ir,ic,2)+flxpop(ir,ic,3))
     .       ,(flxpon(ir,ic,1)+flxpon(ir,ic,2)+flxpon(ir,ic,3))
     .       ,(flxpoc(ir,ic,1)+flxpoc(ir,ic,2)+flxpoc(ir,ic,3))
         write(out,9904)  o20,csod,sod,s,h1,h2,h1dot,h2dot
         write(out,9905)  jp,jn,jc,jo2nh4,xjco2av,xjc1av
         write(out,9906)  jpo4,jnh4,jno3,jhs,jsi,jch4aq,jch4g,bnthstr
         write(out,9907)  po40,po41,po42,po4t2,nh40,nh41,nh42,nh4t2
         write(out,9908)  no30,no31,no32,no3t2,hs1,hs2,hst2
         write(out,9909)  si0,si1,si2,sit2
         call exit
         endif
      endif
c-- and evaluate the po4,si equations
      error=sedf(sod,2)

c-- replace the t minus 1 concentrations
      call storetm1

C ASSIGN FLUX-FLUX RESULTS TO WQM ARRAYS

      SODS(IR,IC) = SOD
      FLXHS(IR,IC) = JHS
      FLXCH4AQ(IR,IC) = JCH4AQ
      FLXCH4G(IR,IC) = JCH4G
      FLXNH3(IR,IC) = JNH4/1000.
      FLXPO4(IR,IC) = JPO4/1000.
      FLXNO3(IR,IC) = JNO3/1000.
      FLXSI(IR,IC) = JSI/1000.

      if(idisk.ge.2)    then

c- output solid phase concentrations in mg/g (O2) mg/kg (po4) mg/g (Si)
         hst2r=hst2av/1000./m2
         po4t2r=po4t2av/1000./m2
         sit2r=(psiav+sit2av)/1.0e6/m2
         psiavr=(psiav)/1.0e6/m2
c- output POC,PON,etc in mg/g
         pon1r=pon1av/1.0e6/m2
         pon2r=pon2av/1.0e6/m2
         pon3r=pon3av/1.0e6/m2
         ponr=pon1r+pon2r+pon3r
         poc1r=poc1av/1.0e6/m2
         poc2r=poc2av/1.0e6/m2
         poc3r=poc3av/1.0e6/m2
         pocr=poc1r+poc2r+poc3r
         pop1r=pop1av/1.0e6/m2
         pop2r=pop2av/1.0e6/m2
         pop3r=pop3av/1.0e6/m2
         popr=pop1r+pop2r+pop3r
c- output dissolved concentrations in mg/L
         nh40z=nh40/1000.
         nh41z=nh41/1000.
         nh42z=nh42av/1000.
         nh4t2z=nh4t2av/1000.
         no30z=no30/1000.
         no31z=no31/1000.
         no32z=no32av/1000.
         no3t2z=no3t2av/1000.
         si0z=si0/1000.
         si1z=si1/1000.
         si2z=si2av/1000.
         sit2z=sit2av/1000.
         po40z=po40/1000.
         po41z=po41/1000.
         po42z=po42av/1000.
         po4t2z=po4t2av/1000.
         hs1z=hs1
         hs2z=hs2av
         hst2z=hst2av
c- denitrification flux
      itemp = 10*tempd+1
      if(sal0.gt.saltsw)  then
         xapp1no3=zhtano3s(itemp)
         xk2no3=zhta2no3s(itemp)*deltaz
      else
         xapp1no3=zhtano3f(itemp)
         xk2no3=zhta2no3f(itemp)*deltaz
      endif
      bnthden = (xapp1no3*xapp1no3/s*no31+xk2no3*no32)

c1012 format(60e9.3)
c        write(14,1012)  time,ctemp(ir,ic),pop1r,pop2r,pop3r,popr
         write(14)  time,ctemp(ir,ic),pop1r,pop2r,pop3r,popr
     .    ,pon1r,pon2r,pon3r,ponr,poc1r,poc2r,poc3r,pocr
     .    ,po4t2r,hst2r,sit2r,psiavr
     .    ,(flxpop(ir,ic,1)+flxpop(ir,ic,2)+flxpop(ir,ic,3))
     .    ,(flxpon(ir,ic,1)+flxpon(ir,ic,2)+flxpon(ir,ic,3))
     .    ,(flxpoc(ir,ic,1)+flxpoc(ir,ic,2)+flxpoc(ir,ic,3))
     .    ,o20,csod,sod,s,jp,jn,jc,jo2nh4,xjco2av,xjc1av
     .    ,jpo4,jnh4,jno3,jhs,jsi,jch4aq,jch4g,bnthstr
     .    ,po40,po41,po42,po4t2,nh40,nh41,nh42,nh4t2
     .    ,no30,no31,no32,no3t2,hs1,hs2,hst2,si0,si1,si2,sit2,bnthden

      call storesed

      endif

  200 CONTINUE

C             TAKE TEMPERATURE INTEGRATION STEP
      DO 300 IC=1,NCOL
      DO 300 IR=1,NROW
      IF(FSM(IR,IC).NE.1)  GO TO 300
         CTEMP(IR,IC) = CTEMP(IR,IC)+DELTAT*DIFFT/HSED(IR,IC)/
     .      HSED(IR,IC)*(HYDTEMP(IR,IC,NLAYR)-CTEMP(IR,IC))
  300 CONTINUE


      if(idisk.ge.1)    then
         rewind(16)
         write(16)   ctemp,cpop,cpon,cpoc,cpos
     .           ,po4t2tm1s,nh4t2tm1s,no3t2tm1s
     .           ,hst2tm1s,sit2tm1s,bnthstr1s
     .           ,po4t1tm1s,nh4t1tm1s,no3t1tm1s
     .           ,sit1tm1s,hst1tm1s
      endif

      IF(TIME.GE.303.9948.AND.TIME.LT.304.0052) THEN
       OPEN(UNIT=162,file='RCAFICSEDAUG',form='UNFORMATTED')
       WRITE(162)   ctemp,cpop,cpon,cpoc,cpos
     .           ,po4t2tm1s,nh4t2tm1s,no3t2tm1s
     .           ,hst2tm1s,sit2tm1s,bnthstr1s
     .           ,po4t1tm1s,nh4t1tm1s,no3t1tm1s
     .           ,sit1tm1s,hst1tm1s
       CLOSE(162)
      ENDIF

      RETURN

  900 WRITE(OUT,9005)
 9005 FORMAT(/' READ ERROR IN SEDIMENT INPUT DECK')
      STOP
      END
