cOPTION CROSSREF ON
cOPTION SYMTABLE ON
cPAGEWIDTH 132
C
C***********************************************************************
C 
C 
      SUBROUTINE   TUNER
C 
C 
C***********************************************************************
C 
C 
C               LONG ISLAND SOUND  EUTROPHICATION MODEL - LISEM
C               -------------------------------------------------
C                                          VARIABLE STOICHIOMETRY
C                                   INTERACTIVE SEDIMENT SUBMODEL
C                                   modification on global dump
C                                   stepwise sunlight
C 
C***********************************************************************
C 
C 
C     SYSTEMS                                                      UNITS
C 
C     -------                                                      -----
C        1 - SALINITY (SAL)                                          PPT
C
C        2 - PHYTOPLANKTON - WINTER DIATOMS (PHYT1)               MG C/L
C        3 - PHYTOPLANKTON - SUMMER ASSEMBLAGE (PHYT2)            MG C/L
C
C        4 - PARTICULATE ORGANIC PHOSPHOROUS - REFRACTORY (RPOP)  MG P/L
C        5 - PARTICULATE ORGANIC PHOSPHOROUS - LABILE (LPOP)      MG P/L
C        6 - DISSOLVED ORGANIC PHOSPHOROUS - REFRACTORY (RDOP)    MG P/L
C        7 - DISSOLVED ORGANIC PHOSPHOROUS - LABILE (LDOP)        MG P/L
C        8 - TOTAL DISSOLVED INORGANIC PHOSPHOROUS (PO4T)         MG P/L
C
C        9 - PARTICULATE ORGANIC NITROGEN - REFRACTORY (RPON)     MG N/L
C       10 - PARTICULATE ORGANIC NITROGEN - LABILE (LPON)         MG N/L
C       11 - DISSOLVED ORGANIC NITROGEN - REFRACTORY (RDON)       MG N/L
C       12 - DISSOLVED ORGANIC NITROGEN - LABILE (LDON)           MG N/L
C       13 - TOTAL AMMONIA (NH3T)                                 MG N/L
C       14 - NITRITE + NITRATE (NO23)                             MG N/L
C
C       15 - BIOGENIC SILICA - UNAVAILABLE (BSI)                 MG SI/L
C       16 - TOTAL SILICA - AVAILABLE (SIT)                      MG SI/L
C
C       17 - PARTICULATE ORGANIC CARBON - REFRACTORY (RPOC)       MG C/L
C       18 - PARTICULATE ORGANIC CARBON - LABILE (LPOC)           MG C/L
C       19 - DISSOLVED ORGANIC CARBON - REFRACTORY (RDOC)         MG C/L
C       20 - DISSOLVED ORGANIC CARBON - LABILE (LDOC)             MG C/L
C       21 - DISSOLVED ORGANIC CARBON - REACTIVE (REDOC)          MG C/L
C       22 - DISSOLVED ORGANIC CARBON - ALGAL EXUDATE (EXDOC)     MG C/L
C
C       23 - O2*  - AQUEOUS SOD (O2EQ)                           MG O2/L
C       24 - DISSOLVED OXYGEN (DO)                               MG O2/L
C 
C       25 - TOTAL ACTIVE METAL (TAM)                             MMOL/L
C       25 - TEMPERATURE  (RCATEMP)                               oC     
C 
C***********************************************************************
C 
C     CONSTANTS 
C     --------- 
C                  NAMES AND DESCRIPTIONS OF CONSTANTS
C 
C   NO   NAME      DESCRIPTION                                     UNITS
C 
C    1   K1C     SATURATED PHYTOPLANKTON GROWTH RATE                /DAY
C                (AT TEMPERATURE = TOPT1)
C    2   K1T     TEMPERATURE COEFFICIENT 
C    3   TOPT1   OPTIMAL GROWTH TEMPERATURE FOR DIATOMS            DEG C
C    4   IS1     SATURATING ALGAL LIGHT INTENSITY                 LY/DAY
C    5   KMN1    HALF SATURATION CONSTANT FOR NITROGEN            MG N/L
C    6   KMP1    HALF SATURATION CONSTANT FOR PHOSPHOROUS         MG P/L
C    7   KMS1    HALF SATURATION CONSTANT FOR SILICA             MG SI/L
C    8   K1RC    ENDOGENOUS RESPIRATION RATE AT 20 DEG C            /DAY
C    9   K1RT    TEMPERATURE COEFFICIENT 
C   10   K1GRZC  DEATH RATE DUE TO GRAZING                          /DAY
C   11   K1GRZT  TEMPERATURE COEFFICIENT 
C   12   CCHL1   CARBON TO CHLOROPHYLL RATIO                MG C/MG CHLA
C   13   CRBP11  CARBON TO PHOSPHORUS RATIO - NON-P LIMITED    MG C/MG P
C   14   CRBP12  CARBON TO PHOSPHORUS RATIO - P LIMITED        MG C/MG P
C   15   CRBP13  COEFFICIENT DETERMINING RANGE OF P LIMITATION    L/MG P
C   16   CRBN11  CARBON TO NITROGEN RATIO - NON-N LIMITED      MG C/MG N
C   17   CRBN12  CARBON TO NITROGEN RATIO - N LIMITED          MG C/MG N
C   18   CRBN13  COEFFICIENT DETERMINING RANGE OF N LIMITATION    L/MG N
C   19   CRBS11  CARBON TO SILICA RATIO - NON-SI LIMITED      MG C/MG SI
C   20   CRBS12  CARBON TO SILICA RATIO - SI LIMITED          MG C/MG SI
C   21   CRBS13  COEFFICIENT DETERMINING RANGE OF SI LIMITATION  L/MG SI
C
C   22   K2C     SATURATED PHYTOPLANKTON GROWTH RATE                /DAY
C                (AT TEMPERATURE = TOPT2)
C   23   K2T     TEMPERATURE COEFFICIENT 
C   24   TOPT2   OPTIMAL GROWTH TEMPERATURE FOR SUMMER ASSEMBLAGE  DEG C
C   25   IS2     SATURATING ALGAL LIGHT INTENSITY                 LY/DAY
C   26   KMN2    HALF SATURATION CONSTANT FOR NITROGEN            MG N/L
C   27   KMP2    HALF SATURATION CONSTANT FOR PHOSPHOROUS         MG P/L
C   28   KMS2    HALF SATURATION CONSTANT FOR SILICA             MG SI/L
C   29   K2RC    ENDOGENOUS RESPIRATION RATE AT 20 DEG C            /DAY
C   30   K2RT    TEMPERATURE COEFFICIENT 
C   31   K2GRZC  DEATH RATE DUE TO GRAZING                          /DAY
C   32   K2GRZT  TEMPERATURE COEFFICIENT 
C   33   CCHL2   CARBON TO CHLOROPHYLL RATIO                MG C/MG CHLA
C   34   CRBP21  CARBON TO PHOSPHORUS RATIO - NON-P LIMITED    MG C/MG P
C   35   CRBP22  CARBON TO PHOSPHORUS RATIO - P LIMITED        MG C/MG P
C   36   CRBP23  COEFFICIENT DETERMINING RANGE OF P LIMITATION    L/MG P
C   37   CRBN21  CARBON TO NITROGEN RATIO - NON-N LIMITED      MG C/MG N
C   38   CRBN22  CARBON TO NITROGEN RATIO - N LIMITED          MG C/MG N
C   39   CRBN23  COEFFICIENT DETERMINING RANGE OF N LIMITATION    L/MG N
C   40   CRBS21  CARBON TO SILICA RATIO - NON-SI LIMITED      MG C/MG SI
C   41   CRBS22  CARBON TO SILICA RATIO - SI LIMITED          MG C/MG SI
C   42   CRBS23  COEFFICIENT DETERMINING RANGE OF SI LIMITATION  L/MG SI
C
C   43   XKC     CHLOROPHYLL SELF-SHADING EXT.COEFF.          M2/MG CHLA
C   44   KMPHYT  HALF SATURATION CONSTANT FOR PHYTOPLANKTON       MG C/L
C
C                   RECYCLE FRACTIONS
C   45   FRPOP   REFRACTORY PARTICULATE ORGANIC PHOSPHOROUS 
C   46   FLPOP   LABILE PARTICULATE ORGANIC PHOSPHOROUS 
C   47   FRDOP   REFRACTORY DISSOLVED ORGANIC PHOSPHOROUS 
C   48   FLDOP   LABILE DISSOLVED ORGANIC PHOSPHOROUS 
C   49   FPO4    DISSOLVED INORGANIC PHOSPHOROUS 
C   50   FRPON   REFRACTORY PARTICULATE ORGANIC NITROGEN
C   51   FLPON   LABILE PARTICULATE ORGANIC NITROGEN
C   52   FRDON   REFRACTORY DISSOLVED ORGANIC NITROGEN
C   53   FLDON   LABILE DISSOLVED ORGANIC NITROGEN
C   54   FNH3    AMMONIA 
C   55   FRPOC   REFRACTORY PARTICULATE ORGANIC CARBON
C   56   FLPOC   LABILE PARTICULATE ORGANIC CARBON
C   57   FRDOC   REFRACTORY DISSOLVED ORGANIC CARBON
C   58   FLDOC   LABILE DISSOLVED ORGANIC CARBON
C 
C            PHOSPHORUS HYDROLYSIS/MINERALIZATION RATES AT 20 DEG C 
C   59   K46C    HYDROLYSIS RATE OF RPOP TO RDOP                    /DAY
C   60   K46T    TEMPERATURE COEFFICIENT 
C   61   K57C    HYDROLYSIS RATE OF LPOP TO LDOP                    /DAY
C   62   K57T    TEMPERATURE COEFFICIENT 
C   63   K68C    MINERALIZATION RATE OF RDOP TO PO4                 /DAY
C   64   K68T    TEMPERATURE COEFFICIENT 
C   65   K78C    MINERALIZATION RATE OF LDOP TO PO4                 /DAY
C   66   K78T    TEMPERATURE COEFFICIENT 
C
C             NITROGEN HYDROLYSIS/MINERALIZATION RATES AT 20 DEG C 
C   67   K911C   HYDROLYSIS RATE OF RPON TO RDON                    /DAY
C   68   K911T   TEMPERATURE COEFFICIENT 
C   69   K1012C  HYDROLYSIS RATE OF LPON TO LDON                    /DAY
C   70   K1012T  TEMPERATURE COEFFICIENT 
C   71   K1113C  MINERAL
C   72   K1113T  TEMPERATURE COEFFICIENT 
C   73   K1213C  MINERALIZATION RATE OF LDON TO NH3                 /DAY
C   74   K1213T  TEMPERATURE COEFFICIENT 
C                  NITRIFICATION/DENITIFICATION RATES
C   75   K1314C  NITRIFICATION RATE AT 20 DEG C                     /DAY
C   76   K1314T  TEMPERATURE COEFFICIENT 
C   77   KNIT    HALF SATURATION CONSTANT FOR NITRIF-            MG O2/L
C                ICATION OXYGEN LIMITATION 
C   78   K140C   DENITRIFICATION RATE AT 20 DEG C                   /DAY
C   79   K140T   TEMPERATURE COEFFICIENT 
C   80   KNO3    MICHAELIS CONSTANT FOR DENITRIFICATION          MG O2/L
C                OXYGEN LIMITATION 
C 
C                   SILICA MINERALIZATION RATES AT 20 DEG C 
C   81   K1516C  MINERALIZATION RATE OF BIOGENIC SI TO AVAIL SI     /DAY
C   82   K1516T  TEMPERATURE COEFFICIENT 
C
C                   CARBON HYDROLYSIS/OXIDATION RATES AT 20 DEG C 
C   83   K1719C  HYDROLYSIS RATE OF RPOC TO RDOC                    /DAY
C   84   K1719T  TEMPERATURE COEFFICIENT 
C   85   K1820C  HYDROLYSIS RATE OF LPOC TO LDOC                    /DAY
C   86   K1820T  TEMPERATURE COEFFICIENT 
C   87   K190C   OXIDATION RATE OF RDOC                             /DAY
C   88   K190T   TEMPERATURE COEFFICIENT 
C   89   K200C   OXIDATION RATE OF LDOC                             /DAY
C   90   K200T   TEMPERATURE COEFFICIENT 
C   91   KMLDOC  MICHAELIS CONSTANT FOR LDOC                      MG C/L
C   92   KDOC    HALF SATURATION CONSTANT FOR ORG CARBON         MG O2/L
C   93   K210C   REACTIVE DOC OXIDATION RATE                        /DAY
C   94   K210T   TEMPERATURE COEFFICIENT
C   95   FLOCEX  FRACTION OF PRIMARY PRODUCTIVITY GOING TO
C                LABILE ORGANIC CARBON VIA EXUDATION
C   96   K220C   ALGAL EXUDATE DOC OXIDATION RATE                   /DAY
C   97   K220T   TEMPERATURE COEFFICIENT
C
C   98   K230C   OXIDATION RATE FOR AQUEOUS SOD                     /DAY
C   99   K230T   TEMPERATURE COEFFICIENT
C  100   KO2EQ   HALF SATURATION CONSTANT FOR O2*                MG O2/L
C 
C  101   KAT     TEMPERATURE CORRECTION COEFFICIENT FOR ATMOSPHERIC
C                  REAERATION
C
C  102   VSBASE  BASE ALGAL SETTLING RATE                          M/DAY
C  103   VSNUTR  NUTRIENT STRESSED ALGAL SETTLING RATE             M/DAY
C  104   VSBAST  TEMPERATURE CORRECTION
C  105   VSPOM   PARTICULATE ORGANIC MATTER SETTLING RATE          M/DAY
C  106   VSPIM   PARTIC INORG MATTER (TAM) SETTLING RATE           M/DAY
C  107   VSPMT   TEMPERATURE CORRECTION
C  108   KADPO4  PARTITION COEFFICIENT FOR SORBED PHOSPHORUS  L/MMOL TAM
C  109   KADSI   PARTITION COEFFICIENT FOR SORBED SILICA      L/MMOL TAM
C  110   VSSEDT  TEMPERATURE CORRECTION FOR DEPOSITION TO SEDIMENT
C  111   DTAMMX  SOLUBILITY OF TAM UNDER ANOXIC CONDITIONS         MOL/L
C  112   KDOTAM  CONSTANT THAT RELATES TAM SOLUBILITY TO DO      L/MG DO
C  113   JTAMC   ANOXIC TAM RELEASE RATE AT REFERENCE TEMP    MOL/M2-DAY
C  114   JTAMT   TEMPERATURE COEFFICIENT
C  115   KTAM    HALF SATURATION CONSTANT FOR TAM RELEASE        MG DO/L
C  116   DIFUS   DIFFUSIVITY OF OXYGEN ACROSS THE AIR-WATER      M^2/DAY
C                INTERFACE
C  117   KLMIN   MINIMUM KL FOR REAERATION                         M/DAY
C  118   K2CNL   SATURATED PHYTOPLANKTON GROWTH RATE                /DAY
C                (AT TEMPERATURE = TOPT2) FOR NUTRIENT DEFICIENT AREAS 
C  119   K2CNNL  SATURATED PHYTOPLANKTON GROWTH RATE                /DAY
C                (AT TEMPERATURE = TOPT2) FOR NUTRIENT RICH AREAS 
C  120   KK2C    CONSTANT FOR SATURATED GROWTH RATE SELECTION     MG N/L
C 
C***********************************************************************
C 
C     2-D PARAMETERS
C     --------------
C                       NAMES AND DESCRIPTIONS OF PARAMETERS
C 
C       NO  NAME           DESCRIPTION                             UNITS
C 
C        1 KL      TRANSFER COEFFICIENT FOR REAERATION             M/DAY
C        2 VSNET1  NET SETTLING RATE FROM WATER COLUMN             M/DAY
C                  TO THE BED FOR ALGAL GROUP NO. 1
C        3 VSNET2  NET SETTLING RATE FROM WATER COLUMN             M/DAY
C                  TO THE BED FOR ALGAL GROUP NO. 2
C        4 VSNET3  NET SETTLING RATE FROM WATER COLUMN             M/DAY
C                  TO THE BED FOR NON-LIVING POM AND TAM
C        5 KEBASE  BASE (CHL-A CORRECTED) EXTINCTION COEFF        /METER
C                  (SINCE KE'S ARE TIME-VARIABLE, THEY WILL BE
C                   READ FROM A SERIES OF INPUT FILES)
C 
C 
C***********************************************************************
C 
C     TIME-VARIABLE FUNCTIONS 
C     ----------------------- 
C        1  ITOTSF  TOTAL DAILY SOLAR RADIATION                   LY/DAY
C        2  F       FRACTION OF DAYLIGHT                             DAY
C        3  WTRTMP  WATER COLUMN TEMP.                               oC 
C 
C***********************************************************************
C  
C     THE SEDIMENT SUBROUTINE RETURNS FLUXES BACK....
C      SODS                               [GM O2*/M2-DAY]
C      JHS                                [GM O2*/M2-DAY]
C      JCH4AQ                             [GM O2*/M2-DAY]
C      JCH4G                              [GM O2*/M2-DAY]
C      JNH3                               [MG N/M2-DAY]
C      JPO4                               [MG P/M2-DAY]
C      JNO3                               [MG N/M2-DAY]
C      JSI                                [MG SI/M2-DAY]
C
C 
C***********************************************************************
      SAVE

      INCLUDE 'RCACM'

      CHARACTER  FILNA*6,FILKE*5,NUMBER(32)*2
      CHARACTER   DDNAMES(5,NOSYS)*8

C        STATE-VARIABLES
      REAL
     .      SAL(NROW,NCOL,NLAYR)   , CDSAL(NROW,NCOL,NLAYR)
     .    , PHYT1(NROW,NCOL,NLAYR) , CDPHYT1(NROW,NCOL,NLAYR)
     .    , PHYT2(NROW,NCOL,NLAYR) , CDPHYT2(NROW,NCOL,NLAYR)
     .    , RPOP(NROW,NCOL,NLAYR)  , CDRPOP(NROW,NCOL,NLAYR)
     .    , LPOP(NROW,NCOL,NLAYR)  , CDLPOP(NROW,NCOL,NLAYR)
     .    , RDOP(NROW,NCOL,NLAYR)  , CDRDOP(NROW,NCOL,NLAYR)
     .    , LDOP(NROW,NCOL,NLAYR)  , CDLDOP(NROW,NCOL,NLAYR)
     .    , PO4T(NROW,NCOL,NLAYR)  , CDPO4T(NROW,NCOL,NLAYR)
      REAL
     .      RPON(NROW,NCOL,NLAYR)  , CDRPON(NROW,NCOL,NLAYR)
     .    , LPON(NROW,NCOL,NLAYR)  , CDLPON(NROW,NCOL,NLAYR)
     .    , RDON(NROW,NCOL,NLAYR)  , CDRDON(NROW,NCOL,NLAYR)
     .    , LDON(NROW,NCOL,NLAYR)  , CDLDON(NROW,NCOL,NLAYR)
     .    , NH3T(NROW,NCOL,NLAYR)  , CDNH3T(NROW,NCOL,NLAYR)
     .    , NO23(NROW,NCOL,NLAYR)  , CDNO23(NROW,NCOL,NLAYR)
     .    , BSI(NROW,NCOL,NLAYR)   , CDBSI(NROW,NCOL,NLAYR)
     .    , SIT(NROW,NCOL,NLAYR)   , CDSIT(NROW,NCOL,NLAYR)
      REAL
     .      RPOC(NROW,NCOL,NLAYR)  , CDRPOC(NROW,NCOL,NLAYR)
     .    , LPOC(NROW,NCOL,NLAYR)  , CDLPOC(NROW,NCOL,NLAYR)
     .    , RDOC(NROW,NCOL,NLAYR)  , CDRDOC(NROW,NCOL,NLAYR)
     .    , LDOC(NROW,NCOL,NLAYR)  , CDLDOC(NROW,NCOL,NLAYR)
     .    , REDOC(NROW,NCOL,NLAYR) , CDREDOC(NROW,NCOL,NLAYR)
     .    , EXDOC(NROW,NCOL,NLAYR) , CDEXDOC(NROW,NCOL,NLAYR)
     .    , O2EQ(NROW,NCOL,NLAYR)  , CDO2EQ(NROW,NCOL,NLAYR)
     .    , DO(NROW,NCOL,NLAYR)    , CDDO(NROW,NCOL,NLAYR)
     .    , RCATEMP(NROW,NCOL,NLAYR), CDRCATEMP(NROW,NCOL,NLAYR)
      REAL
     .     K1C,K1T,IS1,ISAT1,KMN1,KMP1,KMS1,K1RC,K1RT,K1GRZC,K1GRZT
     .    ,K2C,K2T,IS2,ISAT2,KMN2,KMP2,KMS2,K2RC,K2RT,K2GRZC,K2GRZT
     .    ,XKC,KMPHYT,K46C,K46T,K57C,K57T,K68C,K68T,K78C,K78T
     .    ,K911C,K911T,K1012C,K1012T,K1113C,K1113T,K1213C,K1213T
     .    ,K1314C,K1314T,KNIT,K140C,K140T,KNO3,K1516C,K1516T
     .    ,K1719C,K1719T,K1820C,K1820T,K190C,K190T,K200C,K200T
     .    ,KMLDOC,KDOC,K210C,K210T,K220C,K220T,K230C,K230T,KO2EQ
     .    ,KADPO4,KADSI,ITOTSF,ITOT,KDOTAM,JTAMC,JTAMT,KTAM,ISURF
     .    ,KAT,KLMIN,KLVEL,KAVEL,K2CX,K2CNL,K2CNNL,KK2C
      REAL
     .     EXPVALS(2000)
      REAL
     .    KEBASE(NROW,NCOL),KL(NROW,NCOL),KA20(NROW,NCOL),CS(NROW,NCOL)
     .   ,VSNET1(NROW,NCOL),VSNET2(NROW,NCOL),VSNET3(NROW,NCOL)
     .   ,TEMP1(NROW,NCOL),TEMP2(NROW,NCOL)
      REAL
     .    CRBP1(3),CRBN1(3),CRBS1(3),CRBP2(3),CRBN2(3),CRBS2(3)
      REAL  
     .    SKE(NROW,NCOL,NLAYR)       ,ATTENL(NROW,NCOL,NLAYR)
     .   ,RLGHT1(NROW,NCOL,NLAYR)    ,RLGHT2(NROW,NCOL,NLAYR)
     .   ,RNUTR1(NROW,NCOL,NLAYR)    ,RNUTR2(NROW,NCOL,NLAYR)
     .   ,PNH31(NROW,NCOL,NLAYR)     ,PNH32(NROW,NCOL,NLAYR)
     .   ,GPP1(NROW,NCOL,NLAYR)      ,GPP2(NROW,NCOL,NLAYR) 
     .   ,DPP1(NROW,NCOL,NLAYR)      ,DPP2(NROW,NCOL,NLAYR) 
     .   ,RESP(NROW,NCOL,NLAYR)      ,GRAZ(NROW,NCOL,NLAYR) 
     .   ,ALG1VS(NROW,NCOL,NLAYR)    ,ALG2VS(NROW,NCOL,NLAYR) 
      REAL  
     .    TPOP(NROW,NCOL,NLAYR)      ,TDOP(NROW,NCOL,NLAYR)
     .   ,TPON(NROW,NCOL,NLAYR)      ,TDON(NROW,NCOL,NLAYR)
     .   ,TPOC(NROW,NCOL,NLAYR)      ,TDOC(NROW,NCOL,NLAYR)
     .   ,TPSI(NROW,NCOL,NLAYR)      ,TGPDP(NROW,NCOL,NLAYR)
     .   ,TRESP(NROW,NCOL,NLAYR)     ,BOTBOD(NROW,NCOL,NLAYR)
     .   ,BOTBODHS(NROW,NCOL,NLAYR)
      REAL
     .      SAL_DDA(NROW,NCOL,NLAYR)    , PHYT1_DDA(NROW,NCOL,NLAYR)
     .    , PHYT2_DDA(NROW,NCOL,NLAYR)  , RPOP_DDA(NROW,NCOL,NLAYR) 
     .    , LPOP_DDA(NROW,NCOL,NLAYR)   , RDOP_DDA(NROW,NCOL,NLAYR) 
     .    , LDOP_DDA(NROW,NCOL,NLAYR)   , DPO4_DDA(NROW,NCOL,NLAYR) 
     .    , RPON_DDA(NROW,NCOL,NLAYR)   , LPON_DDA(NROW,NCOL,NLAYR) 
     .    , RDON_DDA(NROW,NCOL,NLAYR)   , LDON_DDA(NROW,NCOL,NLAYR) 
     .    , DNH3_DDA(NROW,NCOL,NLAYR)   , NO23_DDA(NROW,NCOL,NLAYR) 
      REAL
     .      BSI_DDA(NROW,NCOL,NLAYR)    , DSI_DDA(NROW,NCOL,NLAYR)  
     .    , RPOC_DDA(NROW,NCOL,NLAYR)   , LPOC_DDA(NROW,NCOL,NLAYR) 
     .    , RDOC_DDA(NROW,NCOL,NLAYR)   , LDOC_DDA(NROW,NCOL,NLAYR) 
     .    , REDOC_DDA(NROW,NCOL,NLAYR)  , EXDOC_DDA(NROW,NCOL,NLAYR)
     .    , O2EQ_DDA(NROW,NCOL,NLAYR)   , DO_DDA(NROW,NCOL,NLAYR)   
     .    , TEM_DDA(NROW,NCOL,NLAYR)  
     .    , DO_DMIN(NROW,NCOL,NLAYR)    , DO_DMAX(NROW,NCOL,NLAYR)   
     .    , ECMSAL_DDA(NROW,NCOL,NLAYR) , ECMTEM_DDA(NROW,NCOL,NLAYR)  
     .    , PO4T_DDA(NROW,NCOL,NLAYR)   , NH3T_DDA(NROW,NCOL,NLAYR) 
     .    , SIT_DDA(NROW,NCOL,NLAYR)
      REAL
     .      SAL_GDA(NROW,NCOL,NLAYR)    , PHYT1_GDA(NROW,NCOL,NLAYR)
     .    , PHYT2_GDA(NROW,NCOL,NLAYR)  , TPOP_GDA(NROW,NCOL,NLAYR)
     .    , TDOP_GDA(NROW,NCOL,NLAYR)   , DPO4_GDA(NROW,NCOL,NLAYR)
     .    , TPON_GDA(NROW,NCOL,NLAYR)   , TDON_GDA(NROW,NCOL,NLAYR)
     .    , DNH3_GDA(NROW,NCOL,NLAYR)   , NO23_GDA(NROW,NCOL,NLAYR)
      REAL
     .      TPSI_GDA(NROW,NCOL,NLAYR)   , DSI_GDA(NROW,NCOL,NLAYR)
     .    , TPOC_GDA(NROW,NCOL,NLAYR)   , TDOC_GDA(NROW,NCOL,NLAYR)
     .    , O2EQ_GDA(NROW,NCOL,NLAYR)   , TGPDP_GDA(NROW,NCOL,NLAYR)
     .    , RCATMP_GDA(NROW,NCOL,NLAYR) , SKE_GDA(NROW,NCOL,NLAYR)
     .    , ECMTMP_GDA(NROW,NCOL,NLAYR) , ECMSAL_GDA(NROW,NCOL,NLAYR)
     .    , TRESP_GDA(NROW,NCOL,NLAYR)  , BOTBODHS_GDA(NROW,NCOL,NLAYR)
      REAL
     .    SK1314(NROW,NCOL,NLAYR)    ,SK140(NROW,NCOL,NLAYR)
     .   ,SK190(NROW,NCOL,NLAYR)     ,SK200(NROW,NCOL,NLAYR)
     .   ,SK2014(NROW,NCOL,NLAYR)    ,SK210(NROW,NCOL,NLAYR)
     .   ,SK220(NROW,NCOL,NLAYR)     ,SK230(NROW,NCOL,NLAYR)
     .   ,SR240(NROW,NCOL,NLAYR)     ,SK2413(NROW,NCOL,NLAYR)
     .   ,SK24OC(NROW,NCOL,NLAYR)    ,SK14P(NROW,NCOL,NLAYR)
      REAL
     .    PO4TAM(NROW,NCOL,NLAYR)    ,SITAM(NROW,NCOL,NLAYR)
     .   ,XEMPRC(NROW,NCOL,NLAYR)    ,PTAM(NROW,NCOL,NLAYR)
     .   ,STP20(NROW,NCOL,NLAYR)
      REAL
     .    CHLAVEG(NROW,NCOL,NLAYR)    ,DOAVEG(NROW,NCOL,NLAYR)
     .   ,CHLMING(NROW,NCOL,NLAYR)    ,DOMING(NROW,NCOL,NLAYR)
     .   ,CHLMAXG(NROW,NCOL,NLAYR)    ,DOMAXG(NROW,NCOL,NLAYR)
      INTEGER
     .    ISTP20(NROW,NCOL,NLAYR)
      REAL 
     .    K1RCT(350),K1GRZCT(350),K2RCT(350),K2GRZCT(350)
     .   ,K46CT(350),K57CT(350),K68CT(350),K78CT(350)
     .   ,K911CT(350),K1012CT(350),K1113CT(350),K1213CT(350)
     .   ,K1314CT(350),K140CT(350),K1516CT(350),K1719CT(350)
     .   ,K1820CT(350),K190CT(350),K200CT(350),K210CT(350)
     .   ,K220CT(350),K230CT(350),KACT(350)

      REAL
     .    HALF(NROW,NCOL,NLAYR),ONE(NROW,NCOL,NLAYR)
     .,ONEHALF(NROW,NCOL,NLAYR),TWO(NROW,NCOL,NLAYR)
     .,TWOHALF(NROW,NCOL,NLAYR),THREE(NROW,NCOL,NLAYR)
     .,THREEHALF(NROW,NCOL,NLAYR),FOUR(NROW,NCOL,NLAYR)
     .,FOURHALF(NROW,NCOL,NLAYR),FIVE(NROW,NCOL,NLAYR)
     .,FIVEHALF(NROW,NCOL,NLAYR),SIX(NROW,NCOL,NLAYR)
     .,GTESIX(NROW,NCOL,NLAYR)
      REAL
     .    HALFday(NROW,NCOL,NLAYR),ONEday(NROW,NCOL,NLAYR)
     .,ONEHALFday(NROW,NCOL,NLAYR),TWOday(NROW,NCOL,NLAYR)
     .,TWOHALFday(NROW,NCOL,NLAYR),THREEday(NROW,NCOL,NLAYR)
     .,THREEHALFday(NROW,NCOL,NLAYR),FOURday(NROW,NCOL,NLAYR)
     .,FOURHALFday(NROW,NCOL,NLAYR),FIVEday(NROW,NCOL,NLAYR)
     .,FIVEHALFday(NROW,NCOL,NLAYR),SIXday(NROW,NCOL,NLAYR)
     .,GTESIXday(NROW,NCOL,NLAYR)

C        LABELED COMMON FOR SEGMENT KES
      COMMON /WCKE/ IKE,NKE,TKE(25)

C        REAL AND LABELED COMMON FOR LINKING WATER COLUMN AND SEDIMENT SUBMODELS
      REAL   NCRB1,NCRB2
      REAL   SODS(NROW,NCOL),JHS(NROW,NCOL),JNH3(NROW,NCOL)
     .      ,JPO4(NROW,NCOL),JNO3(NROW,NCOL),JSI(NROW,NCOL)
     .      ,JCH4AQ(NROW,NCOL),JCH4G(NROW,NCOL)
      COMMON /FLUXES/
     .   DPO4(NROW,NCOL,NLAYR),DNH3(NROW,NCOL,NLAYR)
     .  ,DSI(NROW,NCOL,NLAYR)
     .  ,PCRB1(NROW,NCOL,NLAYR),PCRB2(NROW,NCOL,NLAYR)
     .  ,NCRB1(NROW,NCOL,NLAYR),NCRB2(NROW,NCOL,NLAYR)
     .  ,SCRB1(NROW,NCOL,NLAYR),SCRB2(NROW,NCOL,NLAYR)
     .  ,DEPFLUX(NROW,NCOL,12),CFLUXS(NROW,NCOL,8)
      EQUIVALENCE
     .   (CFLUXS(1,1,1),SODS(1,1))   , (CFLUXS(1,1,2),JHS(1,1))
     .  ,(CFLUXS(1,1,3),JCH4AQ(1,1)) , (CFLUXS(1,1,4),JCH4G(1,1))
     .  ,(CFLUXS(1,1,5),JNH3(1,1))   , (CFLUXS(1,1,6),JNO3(1,1))
     .  ,(CFLUXS(1,1,7),JPO4(1,1))   , (CFLUXS(1,1,8),JSI(1,1))

      EQUIVALENCE
     .  (CARAY(1,1,1,1),SAL(1,1,1))   ,(CDARAY(1,1,1,1),CDSAL(1,1,1))
     . ,(CARAY(1,1,1,2),PHYT1(1,1,1)) ,(CDARAY(1,1,1,2),CDPHYT1(1,1,1))
     . ,(CARAY(1,1,1,3),PHYT2(1,1,1)) ,(CDARAY(1,1,1,3),CDPHYT2(1,1,1))
     . ,(CARAY(1,1,1,4),RPOP(1,1,1))  ,(CDARAY(1,1,1,4),CDRPOP(1,1,1))
     . ,(CARAY(1,1,1,5),LPOP(1,1,1))  ,(CDARAY(1,1,1,5),CDLPOP(1,1,1))
     . ,(CARAY(1,1,1,6),RDOP(1,1,1))  ,(CDARAY(1,1,1,6),CDRDOP(1,1,1))
     . ,(CARAY(1,1,1,7),LDOP(1,1,1))  ,(CDARAY(1,1,1,7),CDLDOP(1,1,1))
     . ,(CARAY(1,1,1,8),PO4T(1,1,1))  ,(CDARAY(1,1,1,8),CDPO4T(1,1,1))
      EQUIVALENCE
     .  (CARAY(1,1,1,9),RPON(1,1,1))  ,(CDARAY(1,1,1,9),CDRPON(1,1,1))
     . ,(CARAY(1,1,1,10),LPON(1,1,1)) ,(CDARAY(1,1,1,10),CDLPON(1,1,1))
     . ,(CARAY(1,1,1,11),RDON(1,1,1)) ,(CDARAY(1,1,1,11),CDRDON(1,1,1))
     . ,(CARAY(1,1,1,12),LDON(1,1,1)) ,(CDARAY(1,1,1,12),CDLDON(1,1,1))
     . ,(CARAY(1,1,1,13),NH3T(1,1,1)) ,(CDARAY(1,1,1,13),CDNH3T(1,1,1))
     . ,(CARAY(1,1,1,14),NO23(1,1,1)) ,(CDARAY(1,1,1,14),CDNO23(1,1,1))
     . ,(CARAY(1,1,1,15),BSI(1,1,1))  ,(CDARAY(1,1,1,15),CDBSI(1,1,1))
     . ,(CARAY(1,1,1,16),SIT(1,1,1))  ,(CDARAY(1,1,1,16),CDSIT(1,1,1))
      EQUIVALENCE
     .  (CARAY(1,1,1,17),RPOC(1,1,1)) ,(CDARAY(1,1,1,17),CDRPOC(1,1,1))
     . ,(CARAY(1,1,1,18),LPOC(1,1,1)) ,(CDARAY(1,1,1,18),CDLPOC(1,1,1))
     . ,(CARAY(1,1,1,19),RDOC(1,1,1)) ,(CDARAY(1,1,1,19),CDRDOC(1,1,1))
     . ,(CARAY(1,1,1,20),LDOC(1,1,1)) ,(CDARAY(1,1,1,20),CDLDOC(1,1,1))
     . ,(CARAY(1,1,1,21),REDOC(1,1,1)),(CDARAY(1,1,1,21),CDREDOC(1,1,1))
     . ,(CARAY(1,1,1,22),EXDOC(1,1,1)),(CDARAY(1,1,1,22),CDEXDOC(1,1,1))
     . ,(CARAY(1,1,1,23),O2EQ(1,1,1)) ,(CDARAY(1,1,1,23),CDO2EQ(1,1,1))
     . ,(CARAY(1,1,1,24),DO(1,1,1))   ,(CDARAY(1,1,1,24),CDDO(1,1,1))
     . ,(CARAY(1,1,1,25),RCATEMP(1,1,1))
     . ,(CDARAY(1,1,1,25),CDRCATEMP(1,1,1))

      EQUIVALENCE 
     .   (CONST(1),K1C)     , (CONST(2),K1T)     , (CONST(3),TOPT1) ,
     .   (CONST(4),IS1)     , (CONST(5),KMN1)    , (CONST(6),KMP1) ,
     .   (CONST(7),KMS1)    , (CONST(8),K1RC)    , (CONST(9),K1RT) ,
     .   (CONST(10),K1GRZC) , (CONST(11),K1GRZT) , (CONST(12),CCHL1) ,
     .   (CONST(13),CRBP11) , (CONST(14),CRBP12) , (CONST(15),CRBP13) ,
     .   (CONST(16),CRBN11) , (CONST(17),CRBN12) , (CONST(18),CRBN13) ,
     .   (CONST(19),CRBS11) , (CONST(20),CRBS12) , (CONST(21),CRBS13) ,
     .   (CONST(22),K2C)    , (CONST(23),K2T)    , (CONST(24),TOPT2) ,
     .   (CONST(25),IS2)    , (CONST(26),KMN2)   , (CONST(27),KMP2) , 
     .   (CONST(28),KMS2)   , (CONST(29),K2RC)   , (CONST(30),K2RT)
      EQUIVALENCE
     .   (CONST(31),K2GRZC) , (CONST(32),K2GRZT) , (CONST(33),CCHL2) ,
     .   (CONST(34),CRBP21) , (CONST(35),CRBP22) , (CONST(36),CRBP23) ,
     .   (CONST(37),CRBN21) , (CONST(38),CRBN22) , (CONST(39),CRBN23) ,
     .   (CONST(40),CRBS21) , (CONST(41),CRBS22) , (CONST(42),CRBS23) ,
     .   (CONST(43),XKC)    , (CONST(44),KMPHYT) , (CONST(45),FRPOP) ,
     .   (CONST(46),FLPOP)  , (CONST(47),FRDOP)  , (CONST(48),FLDOP) ,
     .   (CONST(49),FPO4)   , (CONST(50),FRPON)  , (CONST(51),FLPON) ,
     .   (CONST(52),FRDON)  , (CONST(53),FLDON)  , (CONST(54),FNH3)  ,
     .   (CONST(55),FRPOC)  , (CONST(56),FLPOC)  , (CONST(57),FRDOC) ,
     .   (CONST(58),FLDOC)  , (CONST(59),K46C)   , (CONST(60),K46T)
      EQUIVALENCE
     .   (CONST(61),K57C)   , (CONST(62),K57T)   , (CONST(63),K68C) , 
     .   (CONST(64),K68T)   , (CONST(65),K78C)   , (CONST(66),K78T) ,
     .   (CONST(67),K911C)  , (CONST(68),K911T)  , (CONST(69),K1012C) ,
     .   (CONST(70),K1012T) , (CONST(71),K1113C) , (CONST(72),K1113T) ,
     .   (CONST(73),K1213C) , (CONST(74),K1213T) , (CONST(75),K1314C) ,
     .   (CONST(76),K1314T) , (CONST(77),KNIT)   , (CONST(78),K140C)  ,
     .   (CONST(79),K140T)  , (CONST(80),KNO3)   , (CONST(81),K1516C) ,
     .   (CONST(82),K1516T) , (CONST(83),K1719C) , (CONST(84),K1719T) ,
     .   (CONST(85),K1820C) , (CONST(86),K1820T) , (CONST(87),K190C)  ,
     .   (CONST(88),K190T)  , (CONST(89),K200C)  , (CONST(90),K200T)
      EQUIVALENCE
     .   (CONST(91),KMLDOC) , (CONST(92),KDOC)    , (CONST(93),K210C)  ,
     .   (CONST(94),K210T)  , (CONST(95),FLOCEX)  , (CONST(96),K220C)  ,
     .   (CONST(97),K220T)  , (CONST(98),K230C)   , (CONST(99),K230T)  ,
     .   (CONST(100),KO2EQ) , (CONST(101),KAT)    , (CONST(102),VSBASE),
     .   (CONST(103),VSNUTR), (CONST(104),VSBAST) , (CONST(105),VSPOM) ,
     .   (CONST(106),VSPIM) , (CONST(107),VSPMT)  , (CONST(108),KADPO4),
     .   (CONST(109),KADSI) , (CONST(110),VSSEDT) , (CONST(111),DTAMMX),
     .   (CONST(112),KDOTAM), (CONST(113),JTAMC)  , (CONST(114),JTAMT) ,
     .   (CONST(115),KTAM)  , (CONST(116),DIFUS)  , (CONST(117),KLMIN) , 
     .   (CONST(118),K2CNL) , (CONST(119),K2CNNL) , (CONST(120),KK2C)  

      EQUIVALENCE 
     .     (CRBP1(1),CRBP11) , (CRBP1(2),CRBP12) , (CRBP1(3),CRBP13)
     .   , (CRBN1(1),CRBN11) , (CRBN1(2),CRBN12) , (CRBN1(3),CRBN13)
     .   , (CRBS1(1),CRBS11) , (CRBS1(2),CRBS12) , (CRBS1(3),CRBS13)
     .   , (CRBP2(1),CRBP21) , (CRBP2(2),CRBP22) , (CRBP2(3),CRBP23)
     .   , (CRBN2(1),CRBN21) , (CRBN2(2),CRBN22) , (CRBN2(3),CRBN23)
     .   , (CRBS2(1),CRBS21) , (CRBS2(2),CRBS22) , (CRBS2(3),CRBS23)

      EQUIVALENCE 
     .   (PARAM2D(1,1,1),KL(1,1))     , (PARAM2D(1,1,2),VSNET1(1,1))
     . , (PARAM2D(1,1,3),VSNET2(1,1)) , (PARAM2D(1,1,4),VSNET3(1,1))
     . , (PARAM2D(1,1,5),KEBASE(1,1))

      REAL    VARRAY(NROW,NCOL,NLAYR,25)
      EQUIVALENCE    (SCRATCH_KIN(1,1,1,1),VARRAY(1,1,1,1))
      EQUIVALENCE
     . (VARRAY(1,1,1,1),XEMPRC(1,1,1)) ,(VARRAY(1,1,1,2),PTAM(1,1,1))
     .,(VARRAY(1,1,1,3),STP20(1,1,1))  ,(VARRAY(1,1,1,4),SR240(1,1,1))
     .,(VARRAY(1,1,1,5),PNH31(1,1,1))  ,(VARRAY(1,1,1,6),PNH32(1,1,1))
     .,(VARRAY(1,1,1,7),GPP1(1,1,1))   ,(VARRAY(1,1,1,8),GPP2(1,1,1)) 
     .,(VARRAY(1,1,1,9),DPP1(1,1,1))  ,(VARRAY(1,1,1,10),DPP2(1,1,1))
     .,(VARRAY(1,1,1,11),RESP(1,1,1))  ,(VARRAY(1,1,1,12),GRAZ(1,1,1)) 
     .,(VARRAY(1,1,1,13),ALG1VS(1,1,1)),(VARRAY(1,1,1,14),ALG2VS(1,1,1)) 
     .,(VARRAY(1,1,1,15),SK1314(1,1,1)),(VARRAY(1,1,1,15),SK2413(1,1,1))
     .,(VARRAY(1,1,1,16),SK140(1,1,1)) ,(VARRAY(1,1,1,16),SK2014(1,1,1))
     .,(VARRAY(1,1,1,17),SK14P(1,1,1)) ,(VARRAY(1,1,1,18),SKE(1,1,1))
      EQUIVALENCE
     . (VARRAY(1,1,1,20),RLGHT1(1,1,1)),(VARRAY(1,1,1,21),RLGHT2(1,1,1))
     .,(VARRAY(1,1,1,22),RNUTR1(1,1,1)),(VARRAY(1,1,1,23),RNUTR2(1,1,1))
     .,(VARRAY(1,1,1,24),ATTENL(1,1,1))
     .,(VARRAY(1,1,1,20),SK190(1,1,1)) ,(VARRAY(1,1,1,21),SK200(1,1,1))
     .,(VARRAY(1,1,1,22),SK210(1,1,1)) ,(VARRAY(1,1,1,23),SK220(1,1,1))
     .,(VARRAY(1,1,1,24),SK230(1,1,1)) 
      EQUIVALENCE
     . (VARRAY(1,1,1,25),TPOP(1,1,1))  ,(VARRAY(1,1,1,25),TDOP(1,1,1))
     .,(VARRAY(1,1,1,25),TPON(1,1,1))  ,(VARRAY(1,1,1,25),TDON(1,1,1))
     .,(VARRAY(1,1,1,25),TPOC(1,1,1))  ,(VARRAY(1,1,1,25),TDOC(1,1,1))
     .,(VARRAY(1,1,1,25),TPSI(1,1,1))  ,(VARRAY(1,1,1,25),TGPDP(1,1,1))
     .,(VARRAY(1,1,1,25),TRESP(1,1,1))

      DATA   FILKE/'kebXX'/
     .   , NUMBER/'01','02','03','04','05','06','07','08','09',
     .            '10','11','12','13','14','15','16','17','18','19',
     .            '20','21','22','23','24','25','26','27','28','29',
     .            '30','31','32'/

      OPEN(60,FILE='HALF',FORM ='FORMATTED')  
      OPEN(61,FILE='ONE',FORM ='FORMATTED')  
      OPEN(62,FILE='ONEHALF',FORM ='FORMATTED')  
      OPEN(63,FILE='TWO',FORM ='FORMATTED')  
      OPEN(64,FILE='TWOHALF',FORM ='FORMATTED')  
      OPEN(65,FILE='THREE',FORM ='FORMATTED')  
      OPEN(66,FILE='THREEHALF',FORM ='FORMATTED')  
      OPEN(67,FILE='FOUR',FORM ='FORMATTED')  
      OPEN(68,FILE='FOURHALF',FORM ='FORMATTED')  
      OPEN(69,FILE='FIVE',FORM ='FORMATTED')  
      OPEN(70,FILE='FIVEHALF',FORM ='FORMATTED')  
      OPEN(71,FILE='SIX',FORM ='FORMATTED')  
      OPEN(72,FILE='GTESIX',FORM ='FORMATTED')  

C        PROVIDE INITIALIZATION, IF FIRST TIME THROUGH -FABLE-

      IF(INITB.EQ.1)   GO TO 50 

C        READ "SWITCH" OPTIONS FOR GLOBAL AND DETAILED DUMP TIME-AVERAGING
C           IGDOPT = GLOBAL DUMP TIME-AVERAGING OPTION
C           IDDOPT = DETAILED DUMP TIME-AVERAGING OPTION
C                  =  0  NO TIME-AVERAGING, I.E. INSTANTANEOUS VALUE
C                  =  1  PERFORM TIME-AVERAGING WILL BE PERFORMED
      READ(IN,1000,ERR=900)  IGDOPT,IDDOPT
 1000 FORMAT(2I5)
      WRITE(OUT,1100)  IGDOPT,IDDOPT
 1100 FORMAT(//20X,'SWITCH OPTIONS FOR DUMP TIME-AVERAGING'/
     .         25X,'IGDOPT =',I3,'    IDDOPT =',I3//)

C        WRITE DDNAMES TO RCAF12

      WRITE(12)  DDNAMES

      DO IL=1,NLAYR
        DO IC=1,NCOL
          DO IR=1,NROW
            HALF(IR,IC,IL) = 0.0
            ONE(IR,IC,IL) = 0.0
            ONEHALF(IR,IC,IL) = 0.0
            TWO(IR,IC,IL) = 0.0
            TWOHALF(IR,IC,IL) = 0.0
            THREE(IR,IC,IL) = 0.0
            THREEHALF(IR,IC,IL) = 0.0
            FOUR(IR,IC,IL) = 0.0
            FOURHALF(IR,IC,IL) = 0.0
            FIVE(IR,IC,IL) = 0.0
            FIVEHALF(IR,IC,IL) = 0.0
            SIX(IR,IC,IL) = 0.0
            GTESIX(IR,IC,IL) = 0.0
            HALFday(IR,IC,IL) = 0.0
            ONEday(IR,IC,IL) = 0.0
            ONEHALFday(IR,IC,IL) = 0.0
            TWOday(IR,IC,IL) = 0.0
            TWOHALFday(IR,IC,IL) = 0.0
            THREEday(IR,IC,IL) = 0.0
            THREEHALFday(IR,IC,IL) = 0.0
            FOURday(IR,IC,IL) = 0.0
            FOURHALFday(IR,IC,IL) = 0.0
            FIVEday(IR,IC,IL) = 0.0
            FIVEHALFday(IR,IC,IL) = 0.0
            SIXday(IR,IC,IL) = 0.0
            GTESIXday(IR,IC,IL) = 0.0
          END DO
        END DO
      END DO

C        RECOMPUTE TERMS FOR VARIABLE NUTRIENT STOICHIOMETRY

      IF(CRBP12.GT.0) THEN
        IF(CRBP12.GT.CRBP11)  THEN
          CRBP12 = CRBP12-CRBP11
        ELSE
          WRITE(OUT,1001)  CRBP11,CRBP12
 1001     FORMAT(///10X,'ERROR...CRBP11 > CRBP12',F6.1,' > ',F6.1)
        ENDIF
      ENDIF
      IF(CRBN12.GT.0) THEN
        IF(CRBN12.GT.CRBN11)  THEN
         CRBN12 = CRBN12-CRBN11
        ELSE
          WRITE(OUT,1002)  CRBN11,CRBN12
 1002     FORMAT(///10X,'ERROR...CRBN11 > CRBN12',F6.2,' > ',F6.2)
        ENDIF
      ENDIF
      IF(CRBS12.GT.0) THEN
        IF(CRBS12.GT.CRBS11)  THEN
         CRBS12 = CRBS12-CRBS11
        ELSE
          WRITE(OUT,1003)  CRBS11,CRBS12
 1003     FORMAT(///10X,'ERROR...CRBS11 > CRBS12',F6.2,' > ',F6.2)
        ENDIF
      ENDIF
      IF(CRBP22.GT.0) THEN
        IF(CRBP22.GT.CRBP21)  THEN
         CRBP22 = CRBP22-CRBP21
        ELSE
          WRITE(OUT,1004)  CRBP21,CRBP22
 1004     FORMAT(///10X,'ERROR...CRBP11 > CRBP12',F6.1,' > ',F6.1)
        ENDIF
      ENDIF
      IF(CRBN22.GT.0) THEN
        IF(CRBN22.GT.CRBN21)  THEN
         CRBN22 = CRBN22-CRBN21
        ELSE
          WRITE(OUT,1005)  CRBN21,CRBN22
 1005     FORMAT(///10X,'ERROR...CRBN11 > CRBN12',F6.2,' > ',F6.2)
        ENDIF
      ENDIF
      IF(CRBS22.GT.0) THEN
        IF(CRBS22.GT.CRBS21)  THEN
         CRBS22 = CRBS22-CRBS21
        ELSE
          WRITE(OUT,1006)  CRBS21,CRBS22
 1006     FORMAT(///10X,'ERROR...CRBS11 > CRBS12',F6.2,' > ',F6.2)
        ENDIF
      ENDIF

C        COMPUTE VALUES OF EXP(-X) FOR  X = 0 TO 10 IN 0.005 INCREMENTS
   
      DO 5 I=1,2000
       EXPVALS(I) = EXP(-(FLOAT(I-1)/200+0.0025))
    5 CONTINUE

C        COMPUTE ALGAL NUTRIENT TO CARBON RATIOS
 
      DO 10 IL=1,NLAYR
       DO 10 IC=1,NCOL
        DO 10 IR=1,NROW
         PCRB1(IR,IC,IL) = 1./CRBP11
         NCRB1(IR,IC,IL) = 1./CRBN11
         SCRB1(IR,IC,IL) = 1./CRBS11
         PCRB2(IR,IC,IL) = 1./CRBP21
         NCRB2(IR,IC,IL) = 1./CRBN21
         SCRB2(IR,IC,IL) = 1./CRBS21
   10 CONTINUE

C        OXYGEN TO CARBON RATIO (2*16/12)
      OCRB = 2.6667

C        LIGHT SATURATION VALUES
      ISAT1 = IS1
      ISAT2 = IS2

C        READ TIME-VARIABLE EXTINCTION COEFFICIENTS

      READ(IN,1200)  NKE,(TKE(I),I=1,NKE)
 1200 FORMAT(I5/(8F10.0))
      IKE=1
      FILKE(4:5)=NUMBER(IKE)
      OPEN(28,FILE=FILKE,FORM='FORMATTED')
      READ(28,1300,ERR=910)  ((KEBASE(IR,IC),IC=2,NCOL-1),IR=2,NROW-1)
 1300 FORMAT(8F10.0)
C     WRITE(OUT,1155) TKE(IKE),((KEBASE(IR,IC),IC=2,NCOL-1),IR=2,NROW-1)
 1155 FORMAT(//30X,'SEGMENT KE (/METER) AT TIME = ',F5.0/
     .    (16F5.1))
      CLOSE(28)
      IKE=2

C        KL'S TO KA'S

      DO 25 IC=1,NCOL
       DO 25 IR=1,NROW
        IF(FSM(IR,IC).EQ.1)  THEN
          KA20(IR,IC) = MAX(KL(IR,IC),KLMIN)/(HBAR(IR,IC)*DZ(1))
        ENDIF
   25 CONTINUE

C        SET REAERATION TERM FOR OXYGEN TO ZERO

      DO 30 IL=1,NLAYR
       DO 30 IC=1,NCOL
        DO 30 IR=1,NROW
         SR240(IR,IC,IL) = 0.0
   30 CONTINUE

C        SETUP TEMPERATURE-CORRECTED REACTION RATES IN TABLE-LOOKUP FORM

      DO 35 ITEMP=1,350
        TEMP = (FLOAT(ITEMP-1)/10.+0.05) - 20.
        K1RCT(ITEMP) = K1RC*K1RT**TEMP
        K1GRZCT(ITEMP) = K1GRZC*K1GRZT**TEMP
        K2RCT(ITEMP) = K2RC*K2RT**TEMP
        K2GRZCT(ITEMP) = K2GRZC*K2GRZT**TEMP
        K46CT(ITEMP) = K46C*K46T**TEMP
        K57CT(ITEMP) = K57C*K57T**TEMP
        K68CT(ITEMP) = K68C*K68T**TEMP
        K78CT(ITEMP) = K78C*K78T**TEMP
        K911CT(ITEMP) = K911C*K911T**TEMP
        K1012CT(ITEMP) = K1012C*K1012T**TEMP
        K1113CT(ITEMP) = K1113C*K1113T**TEMP
        K1213CT(ITEMP) = K1213C*K1213T**TEMP
        K1314CT(ITEMP) = K1314C*K1314T**TEMP
        IF(ITEMP.LE.100.) K1314CT(ITEMP)=0.
        IF(ITEMP.GT.100. .AND.ITEMP.LT.140.)
     .     K1314CT(ITEMP)= (FLOAT(140-ITEMP)/40.)*K1314CT(ITEMP)
        K140CT(ITEMP) = K140C*K140T**TEMP
        K1516CT(ITEMP) = K1516C*K1516T**TEMP
        K1719CT(ITEMP) = K1719C*K1719T**TEMP
        K1820CT(ITEMP) = K1820C*K1820T**TEMP
        K190CT(ITEMP) = K190C*K190T**TEMP
        K200CT(ITEMP) = K200C*K200T**TEMP
        K210CT(ITEMP) = K210C*K210T**TEMP
        K220CT(ITEMP) = K220C*K220T**TEMP
        K230CT(ITEMP) = K230C*K230T**TEMP
        KACT(ITEMP) = KAT**TEMP
   35 CONTINUE

C        SET AVE,MIN,MAX CONCENTRATIONS FOR GLOBAL DUMP ARRAYS

c$doacross local(il,ic,ir) , share(chlaveg,chlmaxg,chlming),
c$& share(doaveg,doming,domaxg)
      DO 40 IL=1,NLAYR
       DO 40 IC=1,NCOL
        DO 40 IR=1,NROW
          CHLAVEG(IR,IC,IL) = 0.
          DOAVEG(IR,IC,IL) = 0.
          CHLMING(IR,IC,IL) = 1000000.
          DOMING(IR,IC,IL) = 1000.
          CHLMAXG(IR,IC,IL) = -1000.
          DOMAXG(IR,IC,IL) = -1000.
   40 CONTINUE
      IAVGGDCNTR = 0

C        INITIALIZE ARRAY FOR GLOBAL DUMP AVERAGING, IF REQUIRED
      IF(IGDOPT.EQ.1)  THEN
c$doacross local(il,ic,ir) , share(sal_gda,ecmsal_gda),
c$& share(phyt1_gda,phyt2_gda),
c$& share(tpop_gda,tdop_gda,dpo4_gda),
c$& share(tpon_gda,tdon_gda,dnh3_gda,no23_gda),
c$& share(tpoc_gda,tdoc_gda,tpsi_gda,dsi_gda),
c$& share(o2eq_gda,rcatmp_gda,ecmtmp_gda,ske_gda),
c$& share(tgpdp_gda,tresp_gda,botbodhs_gda)
        DO 42 IL=1,NLAYR
         DO 42 IC=1,NCOL
          DO 42 IR=1,NROW
            SAL_GDA(IR,IC,IL) = 0.
            ECMSAL_GDA(IR,IC,IL) = 0.
            PHYT1_GDA(IR,IC,IL) = 0.
            PHYT2_GDA(IR,IC,IL) = 0.
            TPOP_GDA(IR,IC,IL) = 0.
            TDOP_GDA(IR,IC,IL) = 0.
            DPO4_GDA(IR,IC,IL) = 0.
            TPON_GDA(IR,IC,IL) = 0.
            TDON_GDA(IR,IC,IL) = 0.
            DNH3_GDA(IR,IC,IL) = 0.
            NO23_GDA(IR,IC,IL) = 0.
            TPSI_GDA(IR,IC,IL) = 0.
            DSI_GDA(IR,IC,IL) = 0.
            TPOC_GDA(IR,IC,IL) = 0.
            TDOC_GDA(IR,IC,IL) = 0.
            O2EQ_GDA(IR,IC,IL) = 0.
            RCATMP_GDA(IR,IC,IL) = 0.
            ECMTMP_GDA(IR,IC,IL) = 0.
            SKE_GDA(IR,IC,IL) = 0.
            TGPDP_GDA(IR,IC,IL) = 0.
            TRESP_GDA(IR,IC,IL) = 0.
            BOTBODHS_GDA(IR,IC,IL) = 0.
   42   CONTINUE
      ENDIF

C        INITIALIZE ARRAY FOR DETAILED DUMP AVERAGING, IF REQUIRED
      IF(IDDOPT.EQ.1)  THEN
c$doacross local(il,ic,ir),share(sal_dda,ecmsal_dda),
c$& share(phyt1_dda,phyt2_dda),
c$& share(rpop_dda,lpop_dda,rdop_dda,ldop_dda,dpo4_dda),
c$& share(rpon_dda,lpon_dda,rdon_dda,ldon_dda,dnh3_dda,no23_dda),
c$& share(bsi_dda,dsi_dda),                                             !RRI
c$& share(rpoc_dda,lpoc_dda,rdoc_dda,ldoc_dda,redoc_dda,exdoc_dda),
c$& share(o2eq_dda,do_dda,tem_dda,ecmtem_dda,do_dmin,do_dmax)
        DO 44 IL=1,NLAYR
         DO 44 IC=1,NCOL
          DO 44 IR=1,NROW
            SAL_DDA(IR,IC,IL) = 0.
            ECMSAL_DDA(IR,IC,IL) = 0.
            PHYT1_DDA(IR,IC,IL) = 0.
            PHYT2_DDA(IR,IC,IL) = 0.
            RPOP_DDA(IR,IC,IL) = 0.
            LPOP_DDA(IR,IC,IL) = 0.
            RDOP_DDA(IR,IC,IL) = 0.
            LDOP_DDA(IR,IC,IL) = 0.
            DPO4_DDA(IR,IC,IL) = 0.
            RPON_DDA(IR,IC,IL) = 0.
            LPON_DDA(IR,IC,IL) = 0.
            RDON_DDA(IR,IC,IL) = 0.
            LDON_DDA(IR,IC,IL) = 0.
            DNH3_DDA(IR,IC,IL) = 0.
            NO23_DDA(IR,IC,IL) = 0.
            BSI_DDA(IR,IC,IL) = 0.
            DSI_DDA(IR,IC,IL) = 0.
            RPOC_DDA(IR,IC,IL) = 0.
            LPOC_DDA(IR,IC,IL) = 0.
            RDOC_DDA(IR,IC,IL) = 0.
            LDOC_DDA(IR,IC,IL) = 0.
            REDOC_DDA(IR,IC,IL) = 0.
            EXDOC_DDA(IR,IC,IL) = 0.
            O2EQ_DDA(IR,IC,IL) = 0.
            DO_DDA(IR,IC,IL) = 0.
            TEM_DDA(IR,IC,IL) = 0.
            ECMTEM_DDA(IR,IC,IL) = 0.
            PO4T_DDA(IR,IC,IL) = 0.
            NH3T_DDA(IR,IC,IL) = 0.
            SIT_DDA(IR,IC,IL) = 0.
            DO_DMIN(IR,IC,IL) = 1000.
            DO_DMAX(IR,IC,IL) = -1000.
   44     CONTINUE
      IAVGDDCNTR = 0
      ENDIF

      DO 48 IL=1,NLAYR
       DO 48 IC=1,NCOL
        DO 48 IR=1,NROW
         CARAY(IR,IC,IL,1) = HYDSAL(IR,IC,IL)
         CARAY(IR,IC,IL,25) = HYDTEMP(IR,IC,IL)
   48 CONTINUE


C        INITIALIZE SEDIMENT
      CALL SED_READ

C 
C                  S Y S T E M   L O O P S
C 

   50 CONTINUE

      IF(NXFUNT.LE.TIME)   THEN
         CALL RCA13(MFUNC,BFUNC,NOFUNC,ITVFPLOPT,ITIMF,NXFUNT,34)
      ENDIF
      ITOTSF = (TIME-NXFUNT)*MFUNC(1) + BFUNC(1)
      F = (TIME-NXFUNT)*MFUNC(2) + BFUNC(2)
      WTRTMP= (TIME-NXFUNT)*MFUNC(3) + BFUNC(3)

      IF(IS1.EQ.0.)  ISAT1=ITOTSF
      IF(IS2.EQ.0.)  ISAT2=ITOTSF
      TSUNRISE = 0.5-F/2.
      TSUNSET = 0.5+F/2.
      TDAY = MOD(TIME,1.0)
      IF(TDAY.LT.TSUNRISE .OR. TDAY.GT.TSUNSET)  THEN
        ISURF = 0.0
      ELSE
        ISURF = ITOTSF/F/0.635 * SIN(3.14159*(TDAY-TSUNRISE)/F)
      ENDIF

C        COMPUTE TLDOC RECYCLE TERM, PARTICULATE TAM AND STP20


cc c$doacross local(il,ic,ir,tldoc,dtam)
      DO 60 IL=1,NLAYR
       DO 60 IC=1,NCOL
        DO 60 IR=1,NROW
         IF(FSM(IR,IC).NE.1)  GO TO 60
         TLDOC = PHYT1(IR,IC,IL) + PHYT2(IR,IC,IL) + REDOC(IR,IC,IL) + 
     .           EXDOC(IR,IC,IL) 
         XEMPRC(IR,IC,IL) = TLDOC/(KMPHYT+TLDOC)
         PTAM(IR,IC,IL) = 0.0
         STP20(IR,IC,IL) = HYDTEMP(IR,IC,IL) - 20.
         ISTP20(IR,IC,IL) = 10.*HYDTEMP(IR,IC,IL) + 1
   60 CONTINUE


C        COMPUTE ALGAL NUTRIENT TO CARBON RATIOS AND AVAILABLE NUTRIENTS

cc c$doacross local(il,ic,ir,algalp,dfpo4,algaln,algals,dfsi)
      DO 75 IL=1,NLAYR
       DO 75 IC=1,NCOL
        DO 75 IR=1,NROW
          IF(FSM(IR,IC).NE.1)  GO TO 75

C        PHOSPHORUS

          IF(CRBP12.NE.0. .OR. CRBP22.NE.0.)  THEN
            CALL STOICH(PO4T(IR,IC,IL),KADPO4,PTAM(IR,IC,IL)
     .                 ,PHYT1(IR,IC,IL),PHYT2(IR,IC,IL),CRBP1,CRBP2
     .                 ,PCRB1(IR,IC,IL),PCRB2(IR,IC,IL))
         ENDIF

C        FRACTION OF PO4T BOUND IN ALGAL BIOMASS
          ALGALP = PCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .           + PCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)

C        FRACTION OF DISSOLVED INORGANIC PHOSPHORUS
          DFPO4 = 1./(1.+KADPO4*PTAM(IR,IC,IL))

C        AVAILABLE INORGANIC PHOSPHORUS
          DPO4(IR,IC,IL) = MAX(0.0,DFPO4*(PO4T(IR,IC,IL)-ALGALP))

C        SORBED PHOSPHORUS
          PO4TAM(IR,IC,IL) = PO4T(IR,IC,IL) - ALGALP - DPO4(IR,IC,IL)

C        NITROGEN

          IF(CRBN12.NE.0. .OR. CRBN22.NE.0.)  THEN
            CALL STOICH(NH3T(IR,IC,IL)+NO23(IR,IC,IL),0.0,PTAM(IR,IC,IL)
     .                 ,PHYT1(IR,IC,IL),PHYT2(IR,IC,IL),CRBN1,CRBN2
     .                 ,NCRB1(IR,IC,IL),NCRB2(IR,IC,IL))
          ENDIF

C        FRACTION OF NH3T BOUND IN ALGAL BIOMASS
          ALGALN = NCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .           + NCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)

C        AVAILABLE AMMONIA
          DNH3(IR,IC,IL) = MAX(0.0,(NH3T(IR,IC,IL) - ALGALN))

C        SILICA

          IF(CRBS12.NE.0. .OR. CRBS22.NE.0.)  THEN
            CALL STOICH(SIT(IR,IC,IL),KADSI,PTAM(IR,IC,IL)
     .                 ,PHYT1(IR,IC,IL),PHYT2(IR,IC,IL),CRBS1,CRBS2
     .                 ,SCRB1(IR,IC,IL),SCRB2(IR,IC,IL))
          ENDIF

C        FRACTION OF SIT BOUND IN ALGAL BIOMASS
          ALGALS = SCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .           + SCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)

C        COMPUTE FRACTION OF DISSOLVED AVAILABLE SILICA
          DFSI = 1./(1.+KADSI*PTAM(IR,IC,IL))

C        AVAILABLE SILICA
          DSI(IR,IC,IL) = MAX(0.0,DFSI*(SIT(IR,IC,IL)-ALGALS))

C        SORBED SILICA
          SITAM(IR,IC,IL) = SIT(IR,IC,IL) - ALGALS - DSI(IR,IC,IL)

   75   CONTINUE

C        LOOP FOR DETAILED DUMP AVERAGING, IF REQUIRED
      IF(IDDOPT.EQ.1)  THEN
c$doacross local(il,ic,ir) , share(sal_dda,ecmsal_dda,phyt1_dda),
c$& share(phyt2_dda,bsi_dda,dsi_dda,o2eq_dda,do_dda,tem_dda,ecmtem_dda)
        DO 80 IL=1,NLAYR
         DO 80 IC=1,NCOL
          DO 80 IR=1,NROW
            SAL_DDA(IR,IC,IL) = SAL_DDA(IR,IC,IL) + SAL(IR,IC,IL)
         ECMSAL_DDA(IR,IC,IL) = ECMSAL_DDA(IR,IC,IL) + HYDSAL(IR,IC,IL)
            PHYT1_DDA(IR,IC,IL) = PHYT1_DDA(IR,IC,IL) + PHYT1(IR,IC,IL)
            PHYT2_DDA(IR,IC,IL) = PHYT2_DDA(IR,IC,IL) + PHYT2(IR,IC,IL)
            BSI_DDA(IR,IC,IL) = BSI_DDA(IR,IC,IL) + BSI(IR,IC,IL)
            DSI_DDA(IR,IC,IL) = DSI_DDA(IR,IC,IL) + DSI(IR,IC,IL)
            SIT_DDA(IR,IC,IL) = SIT_DDA(IR,IC,IL) + SIT(IR,IC,IL)
            O2EQ_DDA(IR,IC,IL) = O2EQ_DDA(IR,IC,IL) + O2EQ(IR,IC,IL)
            DO_DDA(IR,IC,IL) = DO_DDA(IR,IC,IL) + DO(IR,IC,IL)
            TEM_DDA(IR,IC,IL) = TEM_DDA(IR,IC,IL) + RCATEMP(IR,IC,IL)
         ECMTEM_DDA(IR,IC,IL) = ECMTEM_DDA(IR,IC,IL) + HYDTEMP(IR,IC,IL)
   80   CONTINUE
c$doacross local(il,ic,ir) ,
c$& share(rpop_dda,lpop_dda,rdop_dda,ldop_dda,dpo4_dda)                 !RRI
        DO 81 IL=1,NLAYR
         DO 81 IC=1,NCOL
          DO 81 IR=1,NROW
            RPOP_DDA(IR,IC,IL) = RPOP_DDA(IR,IC,IL) + RPOP(IR,IC,IL)
            LPOP_DDA(IR,IC,IL) = LPOP_DDA(IR,IC,IL) + LPOP(IR,IC,IL)
            RDOP_DDA(IR,IC,IL) = RDOP_DDA(IR,IC,IL) + RDOP(IR,IC,IL)
            LDOP_DDA(IR,IC,IL) = LDOP_DDA(IR,IC,IL) + LDOP(IR,IC,IL)
            DPO4_DDA(IR,IC,IL) = DPO4_DDA(IR,IC,IL) + DPO4(IR,IC,IL)
            PO4T_DDA(IR,IC,IL) = PO4T_DDA(IR,IC,IL) + PO4T(IR,IC,IL)
   81   CONTINUE
c$doacross local(il,ic,ir) ,
c$& share(rpon_dda,lpon_dda,rdon_dda,ldon_dda,dnh3_dda,no23_dda)        !RRI
        DO 82 IL=1,NLAYR
         DO 82 IC=1,NCOL
          DO 82 IR=1,NROW
            RPON_DDA(IR,IC,IL) = RPON_DDA(IR,IC,IL) + RPON(IR,IC,IL)
            LPON_DDA(IR,IC,IL) = LPON_DDA(IR,IC,IL) + LPON(IR,IC,IL)
            RDON_DDA(IR,IC,IL) = RDON_DDA(IR,IC,IL) + RDON(IR,IC,IL)
            LDON_DDA(IR,IC,IL) = LDON_DDA(IR,IC,IL) + LDON(IR,IC,IL)
            DNH3_DDA(IR,IC,IL) = DNH3_DDA(IR,IC,IL) + DNH3(IR,IC,IL)
            NO23_DDA(IR,IC,IL) = NO23_DDA(IR,IC,IL) + NO23(IR,IC,IL)
            NH3T_DDA(IR,IC,IL) = NH3T_DDA(IR,IC,IL) + NH3T(IR,IC,IL)
   82   CONTINUE
c$doacross local(il,ic,ir) ,
c$& share(rpoc_dda,lpoc_dda,rdoc_dda,ldoc_dda,redoc_dda,exdoc_dda)        !RRI
        DO 83 IL=1,NLAYR
         DO 83 IC=1,NCOL
          DO 83 IR=1,NROW
            RPOC_DDA(IR,IC,IL) = RPOC_DDA(IR,IC,IL) + RPOC(IR,IC,IL)
            LPOC_DDA(IR,IC,IL) = LPOC_DDA(IR,IC,IL) + LPOC(IR,IC,IL)
            RDOC_DDA(IR,IC,IL) = RDOC_DDA(IR,IC,IL) + RDOC(IR,IC,IL)
            LDOC_DDA(IR,IC,IL) = LDOC_DDA(IR,IC,IL) + LDOC(IR,IC,IL)
            REDOC_DDA(IR,IC,IL) = REDOC_DDA(IR,IC,IL) + REDOC(IR,IC,IL)
            EXDOC_DDA(IR,IC,IL) = EXDOC_DDA(IR,IC,IL) + EXDOC(IR,IC,IL)
   83     CONTINUE
        IAVGDDCNTR = IAVGDDCNTR + 1
      ENDIF

      DO IL=1,NLAYR
        DO IC=1,NCOL
          DO IR=1,NROW
          IF(FSM(IR,IC).LE.0) GO TO 93
          IF(DO(IR,IC,IL).LT.0.5) THEN
            HALF(IR,IC,IL)=HALF(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
            HALFday(IR,IC,IL)=HALFday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.1.0) THEN
            ONE(IR,IC,IL)=ONE(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
            ONEday(IR,IC,IL)=ONEday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.1.5) THEN
            ONEHALF(IR,IC,IL)=ONEHALF(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
            ONEHALFday(IR,IC,IL)=ONEHALFday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.2.0) THEN
            TWO(IR,IC,IL)=TWO(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
            TWOday(IR,IC,IL)=TWOday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.2.5) THEN
            TWOHALF(IR,IC,IL)=TWOHALF(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
            TWOHALFday(IR,IC,IL)=TWOHALFday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.3.0) THEN
            THREE(IR,IC,IL)=THREE(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
            THREEday(IR,IC,IL)=THREEday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.3.5) THEN
           THREEHALF(IR,IC,IL)=THREEHALF(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
           THREEHALFday(IR,IC,IL)=THREEHALFday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.4.0) THEN
           FOUR(IR,IC,IL)=FOUR(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
           FOURday(IR,IC,IL)=FOURday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.4.5) THEN
           FOURHALF(IR,IC,IL)=FOURHALF(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
           FOURHALFday(IR,IC,IL)=FOURHALFday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.5.0) THEN
           FIVE(IR,IC,IL)=FIVE(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
           FIVEday(IR,IC,IL)=FIVEday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.5.5) THEN
           FIVEHALF(IR,IC,IL)=FIVEHALF(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
           FIVEHALFday(IR,IC,IL)=FIVEHALFday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).LT.6.0) THEN
           SIX(IR,IC,IL)=SIX(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
           SIXday(IR,IC,IL)=SIXday(IR,IC,IL)+(DTWQ)
          END IF
          IF(DO(IR,IC,IL).GE.6.0) THEN
           GTESIX(IR,IC,IL)=GTESIX(IR,IC,IL)+(DTWQ*BVOL(IR,IC,IL)/
     .(DZ(IL)*HBAR(IR,IC)))
           GTESIXday(IR,IC,IL)=GTESIXday(IR,IC,IL)+(DTWQ)
          END IF
93        END DO
        END DO
      END DO

      IF(TKE(IKE).LE.TIME)  THEN
        FILKE(4:5)=NUMBER(IKE)
        OPEN(28,FILE=FILKE,FORM='FORMATTED')
        READ(28,1300,ERR=910)  ((KEBASE(IR,IC),IC=2,NCOL-1),IR=2,NROW-1)
        CLOSE(28)
        IKE=IKE+1
      ENDIF

C
C        SYSTEM 1 - SALINITY
C

      DO 90 IL=1,NLAYR
      DO 90 IC=1,NCOL
      DO 90 IR=1,NROW
   90 CDSAL(IR,IC,IL) = 0.0

C        PUT ECOM COMPUTED SALINITY AT BOUNDARY
      DO 95 I=1,NOBCALL
       IR = IBCALL(1,I)
       IC = IBCALL(2,I)
       IL = IBCALL(3,I)
       CARAY(IR,IC,IL,1) = HYDSAL(IR,IC,IL) 
   95 CONTINUE

C
C             SYSTEM 2 - PHYTOPLANKTON - WINTER DIATOMS
C
 
C        GET LIGHT ATTENUATION COEFFICIENTS

c$doacross local(ic,ir,il,chla,itot) , share(chlaveg,chlming,chlmaxg)
      DO 105 IC=1,NCOL
       DO 105 IR=1,NROW
        DO 104 IL=1,NLAYR
C        COMPUTE TOTAL CHLOROPHYLL BASED ON
C        FIXED CARBON TO CHLOROPHYLL RATIO 
          CHLA = PHYT1(IR,IC,IL)/CCHL1 + PHYT2(IR,IC,IL)/CCHL2
          CHLAVEG(IR,IC,IL) = CHLAVEG(IR,IC,IL) + CHLA*1000.
          CHLMING(IR,IC,IL) = AMIN1(CHLMING(IR,IC,IL),CHLA*1000.)
          CHLMAXG(IR,IC,IL) = AMAX1(CHLMAXG(IR,IC,IL),CHLA*1000.)
        IF(FSM(IR,IC).LE.0)  GO TO 104

C        INCLUDE SELF-SHADING EFFECT ON EXTINCTION COEFF

          SKE(IR,IC,IL) = KEBASE(IR,IC) + 1000.*XKC*CHLA
 
C        LIGHT ATTENUATION

          IF(IL.EQ.1)   THEN
             ITOT=ISURF
          ELSE
             ITOT=ATTENL(IR,IC,IL-1)
          ENDIF

          ATTENL(IR,IC,IL) = ITOT*EXP(-SKE(IR,IC,IL)*HBAR(IR,IC)*DZ(IL))

  104   CONTINUE
  105 CONTINUE

C        GROWTH...TEMPERATURE AND LIGHT

c$doacross local(ic,ir,il,gitmax,itot,xemp1,xemp2,xemp3,temp1,temp2)
      DO 115 IL=1,NLAYR
      DO 110 IC=1,NCOL
      DO 110 IR=1,NROW
      IF(FSM(IR,IC).LE.0)   GO TO 110

C        TEMPERATURE CORRECT GROWTH RATE

      IF(HYDTEMP(IR,IC,IL).LE.TOPT1) 
     .      GITMAX = K1C*EXP(-.004*(HYDTEMP(IR,IC,IL)-TOPT1)**2.)
      IF(HYDTEMP(IR,IC,IL).GT.TOPT1) 
     .      GITMAX = K1C*EXP(-.004*(TOPT1-HYDTEMP(IR,IC,IL))**2.) 
 
C        LIGHT ATTENUATION

      IF(IL.EQ.1)   THEN
         ITOT=ISURF
      ELSE
         ITOT=ATTENL(IR,IC,IL-1)
      ENDIF

C        COMPUTE GROWTH RATE REDUCTION DUE TO LIGHT CONDITIONS

      TEMP1(IR,IC) = SKE(IR,IC,IL)*HBAR(IR,IC)*DZ(IL)
      TEMP2(IR,IC) = ITOT/ISAT1

      RLGHT1(IR,IC,IL) = 2.718*(EXP(-TEMP2(IR,IC)*EXP(-TEMP1(IR,IC)))
     .   - EXP(-TEMP2(IR,IC)))/TEMP1(IR,IC)

C        GROWTH...NUTRIENT EFFECTS

C        COMPUTE AMMONIA PREFERENCE 

        PNH31(IR,IC,IL) =
     .        DNH3(IR,IC,IL)*NO23(IR,IC,IL)
     .               /((KMN1+DNH3(IR,IC,IL))*(KMN1+NO23(IR,IC,IL)))
     .      + DNH3(IR,IC,IL)*KMN1
     .        /((DNH3(IR,IC,IL)+NO23(IR,IC,IL))*(KMN1+NO23(IR,IC,IL))) 

C        COMPUTE MICHAELIS TERMS

      XEMP1 = (DNH3(IR,IC,IL)+NO23(IR,IC,IL))
     .       /(KMN1+DNH3(IR,IC,IL)+NO23(IR,IC,IL))

      XEMP2 = DPO4(IR,IC,IL)/(KMP1+DPO4(IR,IC,IL))

      XEMP3 = DSI(IR,IC,IL)/(KMS1+DSI(IR,IC,IL))

C        COMPUTE GROWTH RATE REDUCTION DUE TO NUTRIENT LIMITATION 
C        USE 'MINIMUM' FORM RATHER THAN 'MULTIPLICATIVE' FORM 

      RNUTR1(IR,IC,IL) = AMIN1(XEMP1,XEMP2,XEMP3)

C        EFFECT OF NUTRIENT LIMITATION ON SETTLING

      ALG1VS(IR,IC,IL) = (VSBASE + VSNUTR*(1.-RNUTR1(IR,IC,IL)))
     .   *VSBAST**STP20(IR,IC,IL)

C        GROSS GROWTH RATE ... COMBINE FACTORS

      GPP1(IR,IC,IL) = RNUTR1(IR,IC,IL)*RLGHT1(IR,IC,IL)
     .                             *GITMAX*PHYT1(IR,IC,IL)

 
C        DEATH...RESPIRATION AND GRAZING

C        RESPIRATION

      RESP(IR,IC,IL) = K1RCT(ISTP20(IR,IC,IL))*PHYT1(IR,IC,IL)

C        GRAZING

Crlm9495-18 try to hard-wire grazing based on Battelle zooplankton data
      IF(TIME.LT.31.) GRAZ(IR,IC,IL) = 0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.31.AND.TIME.LT.61.)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.61.AND.TIME.LT.92.)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.92.AND.TIME.LT.123)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.123.AND.TIME.LT.151)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.151.AND.TIME.LT.182)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.182.AND.TIME.LT.212)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.212.AND.TIME.LT.243)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.243.AND.TIME.LT.273)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.273.AND.TIME.LT.304)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.304.AND.TIME.LT.335)GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)
      IF(TIME.GE.335) GRAZ(IR,IC,IL)=0.00*PHYT1(IR,IC,IL)

C        NET LOSS

      DPP1(IR,IC,IL) = (RESP(IR,IC,IL)+GRAZ(IR,IC,IL))

C        FORMULATE DERIVATIVE ... NET GROWTH (GP-DP)

      CDPHYT1(IR,IC,IL) = ((1.-FLOCEX)*GPP1(IR,IC,IL)-DPP1(IR,IC,IL))

      IF(PHYT1(IR,IC,IL).LE.0.01)  PHYT1(IR,IC,IL) = 0.01

  110 CONTINUE

  115 CONTINUE

C
C             SYSTEM 3 - PHYTOPLANKTON - SUMMER FLAGELLATES
C
 
c$doacross local(il,ic,ir,gitmax,itot,xemp1,xemp2,xemp3,resp2,graz2
c$&             ,temp1,temp2,K2CX)
      DO 125 IL=1,NLAYR
      DO 120 IC=1,NCOL
      DO 120 IR=1,NROW
      IF(FSM(IR,IC).LE.0)   GO TO 120

C        GROWTH...TEMPERATURE AND LIGHT

C        TEMPERATURE CORRECT GROWTH RATE

      IF(HYDTEMP(IR,IC,IL).LE.TOPT2)
     .      GITMAX = K2C*EXP(-.004*(HYDTEMP(IR,IC,IL)-TOPT2)**2.)
      IF(HYDTEMP(IR,IC,IL).GT.TOPT2)
     .      GITMAX = K2C*EXP(-.006*(TOPT2-HYDTEMP(IR,IC,IL))**2.) 

C        LIGHT ATTENUATION

      IF(IL.EQ.1)   THEN
         ITOT=ISURF
      ELSE
         ITOT=ATTENL(IR,IC,IL-1)
      ENDIF

C        COMPUTE GROWTH RATE REDUCTION DUE TO LIGHT CONDITIONS

      TEMP1(IR,IC) = SKE(IR,IC,IL)*HBAR(IR,IC)*DZ(IL)
      TEMP2(IR,IC) = ITOT/ISAT2
      RLGHT2(IR,IC,IL) = 2.718*(EXP(-TEMP2(IR,IC)*EXP(-TEMP1(IR,IC)))
     .   - EXP(-TEMP2(IR,IC)))/TEMP1(IR,IC)

C        GROWTH...NUTRIENT EFFECTS

C        COMPUTE AMMONIA PREFERENCE 

        PNH32(IR,IC,IL) =
     .        DNH3(IR,IC,IL)*NO23(IR,IC,IL)
     .               /((KMN2+DNH3(IR,IC,IL))*(KMN2+NO23(IR,IC,IL)))
     .      + DNH3(IR,IC,IL)*KMN2
     .        /((DNH3(IR,IC,IL)+NO23(IR,IC,IL))*(KMN2+NO23(IR,IC,IL))) 

C        COMPUTE MICHAELIS TERMS

      XEMP1 = (DNH3(IR,IC,IL)+NO23(IR,IC,IL))
     .              /(KMN2+DNH3(IR,IC,IL)+NO23(IR,IC,IL))
      XEMP2 = DPO4(IR,IC,IL)/(KMP2+DPO4(IR,IC,IL)) 
      XEMP3 = DSI(IR,IC,IL)/(KMS2+DSI(IR,IC,IL))

C        COMPUTE GROWTH RATE REDUCTION DUE TO NUTRIENT LIMITATION 
C        USE 'MINIMUM' FORM RATHER THAN 'MULTIPLICATIVE' FORM 

      RNUTR2(IR,IC,IL) = AMIN1(XEMP1,XEMP2,XEMP3)

C        EFFECT OF NUTRIENT LIMITATION ON SETTLING

      ALG2VS(IR,IC,IL) = (VSBASE + VSNUTR*(1.-RNUTR2(IR,IC,IL)))
     .   *VSBAST**STP20(IR,IC,IL)

C        GROSS GROWTH RATE ... COMBINE FACTORS

      GPP2(IR,IC,IL) = RNUTR2(IR,IC,IL)*RLGHT2(IR,IC,IL)
     .                             *GITMAX*PHYT2(IR,IC,IL)


C        DEATH...RESPIRATION AND GRAZING

C        RESPIRATION

      RESP2 = K2RCT(ISTP20(IR,IC,IL))*PHYT2(IR,IC,IL)
      RESP(IR,IC,IL) = RESP(IR,IC,IL) + RESP2

C        GRAZING

Crlm9495-18 hard-wire grazing based on Battelle zooplankton data
      IF(TIME.LT.31.) GRAZ2=0.06*PHYT2(IR,IC,IL)
      IF(TIME.GE.31.AND.TIME.LE.61.) GRAZ2=0.03*PHYT2(IR,IC,IL)
      IF(TIME.GE.61.AND.TIME.LT.92.)GRAZ2=0.03*PHYT2(IR,IC,IL)
      IF(TIME.GE.92.AND.TIME.LT.123)GRAZ2=0.03*PHYT2(IR,IC,IL)
      IF(TIME.GE.123.AND.TIME.LT.151)GRAZ2=0.03*PHYT2(IR,IC,IL)
      IF(TIME.GE.151.AND.TIME.LT.182)GRAZ2=0.10*PHYT2(IR,IC,IL)
      IF(TIME.GE.182.AND.TIME.LT.212)GRAZ2=0.10*PHYT2(IR,IC,IL)
      IF(TIME.GE.212.AND.TIME.LT.243)GRAZ2=0.10*PHYT2(IR,IC,IL)
      IF(TIME.GE.243.AND.TIME.LT.273)GRAZ2=0.06*PHYT2(IR,IC,IL)
      IF(TIME.GE.273.AND.TIME.LT.304)GRAZ2=0.06*PHYT2(IR,IC,IL)
      IF(TIME.GE.304.AND.TIME.LT.335)GRAZ2=0.03*PHYT2(IR,IC,IL)
      IF(TIME.GE.335) GRAZ2=0.06*PHYT2(IR,IC,IL)
      GRAZ(IR,IC,IL) = GRAZ(IR,IC,IL) + GRAZ2

C        NET LOSS

      DPP2(IR,IC,IL) = RESP2+GRAZ2

C        FORMULATE DERIVATIVE ... NET GROWTH (GP-DP)

      CDPHYT2(IR,IC,IL) = ((1.-FLOCEX)*GPP2(IR,IC,IL)-DPP2(IR,IC,IL))

  120 CONTINUE

  125 CONTINUE

      IF(IDISK.EQ.2 .OR. IDISK.EQ.3)  THEN
         DO 131 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         APPNET = ((GPP1(IR,IC,IL)-DPP1(IR,IC,IL)) 
     .          + (GPP2(IR,IC,IL)-DPP2(IR,IC,IL)))/(HBAR(IR,IC)*DZ(IL))
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(1,SAL(IR,IC,IL),SKE(IR,IC,IL),
     .        ALG1VS(IR,IC,IL),ALG2VS(IR,IC,IL),APPNET)
         ELSE
           CALL RCAWBUF(1,SAL_DDA(IR,IC,IL)/IAVGDDCNTR,SKE(IR,IC,IL),
     .        ALG1VS(IR,IC,IL),ALG2VS(IR,IC,IL),APPNET)
         ENDIF
  131    CONTINUE
         DO 132 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(2,PHYT1(IR,IC,IL),1000.*PHYT1(IR,IC,IL)/CCHL1,
     .        RLGHT1(IR,IC,IL),RNUTR1(IR,IC,IL),
     .        (GPP1(IR,IC,IL)-DPP1(IR,IC,IL)))
         ELSE
           CALL RCAWBUF(2,PHYT1_DDA(IR,IC,IL)/IAVGDDCNTR,
     .        1000.*(PHYT1_DDA(IR,IC,IL)/IAVGDDCNTR)/CCHL1,
     .        RLGHT1(IR,IC,IL),RNUTR1(IR,IC,IL),
     .        (GPP1(IR,IC,IL)-DPP1(IR,IC,IL)))
         ENDIF
  132    CONTINUE
         DO 133 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(3,PHYT2(IR,IC,IL),1000.*PHYT2(IR,IC,IL)/CCHL2,
     .        RLGHT2(IR,IC,IL),RNUTR2(IR,IC,IL),
     .        (GPP2(IR,IC,IL)-DPP2(IR,IC,IL)))
         ELSE
           CALL RCAWBUF(3,PHYT2_DDA(IR,IC,IL)/IAVGDDCNTR,
     .        1000.*(PHYT2_DDA(IR,IC,IL)/IAVGDDCNTR)/CCHL2,
     .        RLGHT2(IR,IC,IL),RNUTR2(IR,IC,IL),
     .        (GPP2(IR,IC,IL)-DPP2(IR,IC,IL)))
         ENDIF
  133    CONTINUE
      ENDIF

C
C             SYSTEMS 4-8 - PHOSPHOROUS 
C

c$doacross local(il,ic,ir,sard,term,sk46,sk57,sk68,sk78)
      DO 140 IL=1,NLAYR
      DO 140 IC=1,NCOL
      DO 140 IR=1,NROW
      IF(FSM(IR,IC).LE.0)   GO TO 140

C        Algal Respiration and Death
      SARD = (PCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .      + PCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
      TERM = XEMPRC(IR,IC,IL)

C        RPOP - REFRACTORY PARTICULATE ORGANIC PHOSPHORUS
C                 Sources                         Sinks
C           Algal Resp + Death            Mineralization to RDOP
      SK46 = K46CT(ISTP20(IR,IC,IL))*RPOP(IR,IC,IL)*TERM

      CDRPOP(IR,IC,IL) = FRPOP*SARD - SK46
 
C        LPOP - LABILE PARTICULATE ORGANIC PHOSPHORUS
C                 Sources                         Sinks
C           Algal Resp + Death            Mineralization to LDOP
      SK57 = K57CT(ISTP20(IR,IC,IL))*LPOP(IR,IC,IL)*TERM

      CDLPOP(IR,IC,IL) = FLPOP*SARD - SK57
 
C        RDOP - REFRACTORY DISSOLVED ORGANIC PHOSPHORUS
C                 Sources                         Sinks
C           Algal Resp + Death            Mineralization to PO4
C           Mineralization from RDOP
      SK68 = K68CT(ISTP20(IR,IC,IL))*RDOP(IR,IC,IL)*TERM

      CDRDOP(IR,IC,IL) = FRDOP*SARD + SK46 - SK68
 
C        LDOP - LABILE DISSOLVED ORGANIC PHOSPHORUS
C                 Sources                         Sinks
C           Algal Resp + Death            Mineralization to PO4
C           Mineralization from LPOP
      SK78 = K78CT(ISTP20(IR,IC,IL))*LDOP(IR,IC,IL)*TERM

      CDLDOP(IR,IC,IL) = FLDOP*SARD + SK57 - SK78
 
C        PO4T - Total Dissolved Inorganic Phosphorus
C                 Sources                         Sinks
C           Mineralization from RDOP      Algal Resp + Death Going
C           Mineralization from LDOP          to Organic Pools
C     (Note: Algal uptake and release of PO4 represent an exchange of
C            fractions rather than a source or sink of PO4, therefore
C            no direct algal-related source/sink terms are included in
C            the TPO4 equation.)
C     SK8P = PCRB1(IR,IC,IL)*(1.-FLOCEX)*GPP1(IR,IC,IL)
C    .     + PCRB2(IR,IC,IL)*(1.-FLOCEX)*GPP2(IR,IC,IL)

      CDPO4T(IR,IC,IL) = SK68 + SK78 - (1.-FPO4)*SARD

  140 CONTINUE

      IF(IDISK.EQ.2 .OR. IDISK.EQ.3)  THEN
        DO 144 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         SARD = (PCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + PCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = XEMPRC(IR,IC,IL)
         SK46 = K46CT(ISTP20(IR,IC,IL))*RPOP(IR,IC,IL)*TERM
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(4,RPOP(IR,IC,IL),FRPOP*SARD,SK46,
     .          PCRB1(IR,IC,IL),PCRB2(IR,IC,IL))
         ELSE
           CALL RCAWBUF(4,RPOP_DDA(IR,IC,IL)/IAVGDDCNTR,FRPOP*SARD,
     .          SK46,PCRB1(IR,IC,IL),PCRB2(IR,IC,IL))
         ENDIF
  144   CONTINUE
        DO 145 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TPP = PCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .       + PCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)
     .       + RPOP(IR,IC,IL) + LPOP(IR,IC,IL)
         SARD = (PCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + PCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = XEMPRC(IR,IC,IL)
         SK57 = K57CT(ISTP20(IR,IC,IL))*LPOP(IR,IC,IL)*TERM
         IF(IDDOPT.EQ.0)  THEN
         CALL RCAWBUF(5,LPOP(IR,IC,IL),FLPOP*SARD,SK57,TPP,
     .        XEMPRC(IR,IC,IL))
         ELSE
         CALL RCAWBUF(5,LPOP_DDA(IR,IC,IL)/IAVGDDCNTR,FLPOP*SARD,SK57,
     .        TPP,XEMPRC(IR,IC,IL))
         ENDIF
  145   CONTINUE
        DO 146 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         SARD = (PCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + PCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = XEMPRC(IR,IC,IL)
         SK68 = K68CT(ISTP20(IR,IC,IL))*RDOP(IR,IC,IL)*TERM
         EZPRNT = RZ(IR,IC,IL+1)*0.5*(DZ(IL+1)+DZ(IL))*H(IR,IC)
     .                                                     /XAZ(IR,IC)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(6,RDOP(IR,IC,IL),FRDOP*SARD,SK68,TIME,EZPRNT)
         ELSE
           CALL RCAWBUF(6,RDOP_DDA(IR,IC,IL)/IAVGDDCNTR,FRDOP*SARD,
     .          SK68,TIME,EZPRNT)
         ENDIF
  146   CONTINUE
        DO 147 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TPP = PCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .       + PCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)
     .       + RPOP(IR,IC,IL) + LPOP(IR,IC,IL)
         TDP = RDOP(IR,IC,IL) + LDOP(IR,IC,IL) + DPO4(IR,IC,IL)
         DOP = RDOP(IR,IC,IL) + LDOP(IR,IC,IL)
         TP = TPP + TDP
         SARD = (PCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + PCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = XEMPRC(IR,IC,IL)
         SK78 = K78CT(ISTP20(IR,IC,IL))*LDOP(IR,IC,IL)*TERM
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(7,LDOP(IR,IC,IL),FLDOP*SARD,SK78,TP,DOP)
         ELSE
           CALL RCAWBUF(7,LDOP_DDA(IR,IC,IL)/IAVGDDCNTR,FLDOP*SARD,
     .          SK78,TP,DOP)
         ENDIF
  147   CONTINUE
        DO 148 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TPP = PCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .       + PCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)
     .       + RPOP(IR,IC,IL) + LPOP(IR,IC,IL)
     .       + PO4T(IR,IC,IL) - DPO4(IR,IC,IL)
         TDP = RDOP(IR,IC,IL) + LDOP(IR,IC,IL) + DPO4(IR,IC,IL)
         TP = TPP + TDP
         SARD = (PCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + PCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = XEMPRC(IR,IC,IL)
         SK8P = PCRB1(IR,IC,IL)*(1.-FLOCEX)*GPP1(IR,IC,IL)
     .        + PCRB2(IR,IC,IL)*(1.-FLOCEX)*GPP2(IR,IC,IL)
         IF(FSM(IR,IC).LE.0.) DPO4(IR,IC,IL) = PO4T(IR,IC,IL)
     .        - (PCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)+PCRB2(IR,IC,IL)*
     .        PHYT2(IR,IC,IL))
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(8,PO4T(IR,IC,IL),FPO4*SARD,SK8P,DPO4(IR,IC,IL),
     .          TDP)
         ELSE
           CALL RCAWBUF(8,PO4T_DDA(IR,IC,IL)/IAVGDDCNTR,FPO4*SARD,SK8P,
     .          DPO4_DDA(IR,IC,IL)/IAVGDDCNTR,TDP)
         ENDIF
  148   CONTINUE
      ENDIF

C
C             SYSTEMS 9-14 - NITROGEN
C

c$doacross local(il,ic,ir,sard,term,sk911,sk1012,sk1113,sk1213)
      DO 150 IL=1,NLAYR
      DO 150 IC=1,NCOL
      DO 150 IR=1,NROW
      IF(FSM(IR,IC).LE.0)   GO TO 150

C        Algal Resp and Death
      SARD = (NCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .      + NCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
      TERM = XEMPRC(IR,IC,IL)

C        RPON - REFRACTORY PARTICULATE ORGANIC NITROGEN
C                 Sources                         Sinks
C           Algal Resp + Death            Mineralization to RDON
      SK911 = K911CT(ISTP20(IR,IC,IL))*RPON(IR,IC,IL)*TERM

      CDRPON(IR,IC,IL) = FRPON*SARD - SK911
 
C        LPON - LABILE PARTICULATE ORGANIC NITROGEN
C                 Sources                         Sinks
C           Algal Resp + Death            Mineralization to LDON
      SK1012 = K1012CT(ISTP20(IR,IC,IL))*LPON(IR,IC,IL)*TERM

      CDLPON(IR,IC,IL) = FLPON*SARD - SK1012
 
C        RDON - REFRACTORY DISSOLVED ORGANIC NITROGEN
C                 Sources                         Sinks
C           Algal Resp + Death            Mineralization to NH3
      SK1113 = K1113CT(ISTP20(IR,IC,IL))*RDON(IR,IC,IL)*TERM

      CDRDON(IR,IC,IL) = FRDON*SARD + SK911 - SK1113
 
C        LDON - LABILE DISSOLVED ORGANIC NITROGEN
C                 Sources                         Sinks
C           Algal Resp + Death            Mineralization to NH3
C           Mineralization from LPON
      SK1213 = K1213CT(ISTP20(IR,IC,IL))*LDON(IR,IC,IL)*TERM

      CDLDON(IR,IC,IL) = FLDON*SARD + SK1012 - SK1213
 
C        AMMONIA
C                 Sources                         Sinks
C           Mineralization from LRON      Algal Resp + Death Going
C           Mineralization from LDON          to Organic Pools
C           Algal Uptake of NO23          Nitrification
C     (Note: Algal uptake and release of NH3 represent an exchange of
C            fractions rather than a source or sink of NH3, therefore
C            no direct source/sink terms are included in TNH3 equation.)
      TERM = (1.-FLOCEX)
C     SR13P = FNH3*SARD
C     SK13P = PNH31(IR,IC,IL)*NCRB1(IR,IC,IL)*TERM*GPP1(IR,IC,IL)
C    .      + PNH32(IR,IC,IL)*NCRB2(IR,IC,IL)*TERM*GPP2(IR,IC,IL)

      SK1314(IR,IC,IL) = K1314CT(ISTP20(IR,IC,IL))
     .                  *DNH3(IR,IC,IL)*DO(IR,IC,IL)/(KNIT+DO(IR,IC,IL))
      IF(HYDTEMP(IR,IC,IL).LE.7.0) SK1314(IR,IC,IL)=0.0

      SK14P(IR,IC,IL) =
     .        (1.-PNH31(IR,IC,IL))*NCRB1(IR,IC,IL)*TERM*GPP1(IR,IC,IL)
     .      + (1.-PNH32(IR,IC,IL))*NCRB2(IR,IC,IL)*TERM*GPP2(IR,IC,IL)

      CDNH3T(IR,IC,IL) = SK1113 + SK1213 + SK14P(IR,IC,IL)
     .            - (1.-FNH3)*SARD - SK1314(IR,IC,IL)

C        NITRITE+NITRATE
C                 Sources                         Sinks
C           Nitrification                 Algal Uptake
C                                         Denitrification
      SK140(IR,IC,IL) = K140CT(ISTP20(IR,IC,IL))
     .                  *NO23(IR,IC,IL)*KNO3/(KNO3+DO(IR,IC,IL))
     .                  *LDOC(IR,IC,IL)/(KMLDOC+LDOC(IR,IC,IL))

      CDNO23(IR,IC,IL) = SK1314(IR,IC,IL) - SK14P(IR,IC,IL)
     .                 - SK140(IR,IC,IL)

  150 CONTINUE

      IF(IDISK.EQ.2 .OR. IDISK.EQ.3)  THEN
        DO 152 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         SARD = (NCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + NCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = XEMPRC(IR,IC,IL)
         SK911 = K911CT(ISTP20(IR,IC,IL))*RPON(IR,IC,IL)*TERM
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(9,RPON(IR,IC,IL),FRPON*SARD,SK911,
     .          NCRB1(IR,IC,IL),NCRB2(IR,IC,IL))
         ELSE
           CALL RCAWBUF(9,RPON_DDA(IR,IC,IL)/IAVGDDCNTR,FRPON*SARD,
     .          SK911,NCRB1(IR,IC,IL),NCRB2(IR,IC,IL))
         ENDIF
  152   CONTINUE
        DO 153 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         SARD = (NCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + NCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = XEMPRC(IR,IC,IL)
         SK1012 = K1012CT(ISTP20(IR,IC,IL))*LPON(IR,IC,IL)*TERM
         TTDON = RDON(IR,IC,IL) + LDON(IR,IC,IL)
         TTPON = RPON(IR,IC,IL) + LPON(IR,IC,IL)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(10,LPON(IR,IC,IL),FLPON*SARD,SK1012,TTDON,TTPON)
         ELSE
           CALL RCAWBUF(10,LPON_DDA(IR,IC,IL)/IAVGDDCNTR,FLPON*SARD,
     .          SK1012,TTDON,TTPON)
         ENDIF
  153   CONTINUE
        DO 154 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TPN = NCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .       + NCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)
     .       + RPON(IR,IC,IL) + LPON(IR,IC,IL) 
         SARD = (NCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + NCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = XEMPRC(IR,IC,IL)
         SK1113 = K1113CT(ISTP20(IR,IC,IL))*RDON(IR,IC,IL)*TERM
         TON = RDON(IR,IC,IL) + LDON(IR,IC,IL) 
     .       + RPON(IR,IC,IL) + LPON(IR,IC,IL)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(11,RDON(IR,IC,IL),FRDON*SARD,SK1113,TPN,TON)
         ELSE
           CALL RCAWBUF(11,RDON_DDA(IR,IC,IL)/IAVGDDCNTR,FRDON*SARD,
     .          SK1113,TPN,TON)
         ENDIF
  154   CONTINUE
        DO 155 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         SARD = (NCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + NCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = XEMPRC(IR,IC,IL)
         SK1213 = K1213CT(ISTP20(IR,IC,IL))*LDON(IR,IC,IL)*TERM
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(12,LDON(IR,IC,IL),FLDON*SARD,SK1213,
     .          PNH31(IR,IC,IL),PNH32(IR,IC,IL))
         ELSE
           CALL RCAWBUF(12,LDON_DDA(IR,IC,IL)/IAVGDDCNTR,FLDON*SARD,
     .          SK1213,PNH31(IR,IC,IL),PNH32(IR,IC,IL))
         ENDIF
  155   CONTINUE
        DO 156 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TDN = RDON(IR,IC,IL) + LDON(IR,IC,IL) + DNH3(IR,IC,IL)
     .       + NO23(IR,IC,IL)
         DIN = DNH3(IR,IC,IL) + NO23(IR,IC,IL)
         SARD = (NCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + NCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         TERM = (1.-FLOCEX)
         SR13P = FNH3*SARD
         SK13P = PNH31(IR,IC,IL)*NCRB1(IR,IC,IL)*TERM*GPP1(IR,IC,IL)
     .         + PNH32(IR,IC,IL)*NCRB2(IR,IC,IL)*TERM*GPP2(IR,IC,IL)
         IF(FSM(IR,IC).LE.0.) DNH3(IR,IC,IL) = NH3T(IR,IC,IL)
     .        - (NCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)+NCRB2(IR,IC,IL)*
     .        PHYT2(IR,IC,IL))
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(13,NH3T(IR,IC,IL),SR13P,SK1314(IR,IC,IL),
     .        SK13P,DNH3(IR,IC,IL))
         ELSE
           CALL RCAWBUF(13,NH3T_DDA(IR,IC,IL)/IAVGDDCNTR,SR13P,
     .          SK1314(IR,IC,IL),SK13P,DNH3_DDA(IR,IC,IL)/IAVGDDCNTR)
         ENDIF
  156   CONTINUE
        DO 157 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TPN = NCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .       + NCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)
     .       + RPON(IR,IC,IL) + LPON(IR,IC,IL) 
         TDN = RDON(IR,IC,IL) + LDON(IR,IC,IL) + DNH3(IR,IC,IL)
     .       + NO23(IR,IC,IL)
         DIN = DNH3(IR,IC,IL) + NO23(IR,IC,IL)
         TN = TPN + TDN
         TERM = (1.-FLOCEX)
         SK14P(IR,IC,IL) =
     .     (1.-PNH31(IR,IC,IL))*NCRB1(IR,IC,IL)*TERM*GPP1(IR,IC,IL)
     .   + (1.-PNH32(IR,IC,IL))*NCRB2(IR,IC,IL)*TERM*GPP2(IR,IC,IL)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(14,NO23(IR,IC,IL),SK14P(IR,IC,IL),
     .          SK140(IR,IC,IL),DIN,TN)
         ELSE
           CALL RCAWBUF(14,NO23_DDA(IR,IC,IL)/IAVGDDCNTR,
     .          SK14P(IR,IC,IL),SK140(IR,IC,IL),DIN,TN)
         ENDIF
  157   CONTINUE
      ENDIF

C
C             SYSTEMS 15-16 - SILICA
C

c$doacross local(il,ic,ir,sard,sk1516,sk16p)
      DO 160 IL=1,NLAYR
      DO 160 IC=1,NCOL
      DO 160 IR=1,NROW
      IF(FSM(IR,IC).LE.0)   GO TO 160

C        Algal Resp and Death
      SARD = (SCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .      + SCRB2(IR,IC,IL)*DPP2(IR,IC,IL))

C        BSI - BIOGENIC SILICA
C                 Sources                         Sinks
C           Algal Resp + Death            Mineralization to Avail Si
      SK1516 = K1516CT(ISTP20(IR,IC,IL))
     .                            *BSI(IR,IC,IL)*XEMPRC(IR,IC,IL)

      CDBSI(IR,IC,IL) = SARD - SK1516
 
C        SIT - TOTAL SILICA
C                 Sources                         Sinks
C           Mineralization of BSi         Algal Resp + Death Going
C                                             to Organic Pools
C     (Note: Algal uptake and release of Si represent an exchange of
C            fractions rather than a source or sink of Si, therefore
C            no direct source/sink terms are included in TSi equation.)
      SK16P = SCRB1(IR,IC,IL)*(1.-FLOCEX)*GPP1(IR,IC,IL)
     .      + SCRB2(IR,IC,IL)*(1.-FLOCEX)*GPP2(IR,IC,IL)

      CDSIT(IR,IC,IL) = SK1516 - SARD

  160 CONTINUE

      IF(IDISK.EQ.2 .OR. IDISK.EQ.3)  THEN
        DO 165 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         SARD = (SCRB1(IR,IC,IL)*DPP1(IR,IC,IL)
     .         + SCRB2(IR,IC,IL)*DPP2(IR,IC,IL))
         SK1516 = K1516CT(ISTP20(IR,IC,IL))
     .                            *BSI(IR,IC,IL)*XEMPRC(IR,IC,IL)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(15,BSI(IR,IC,IL),SARD,SK1516,SCRB1(IR,IC,IL),
     .          SCRB2(IR,IC,IL))
         ELSE
           CALL RCAWBUF(15,BSI_DDA(IR,IC,IL)/IAVGDDCNTR,SARD,SK1516,
     .          SCRB1(IR,IC,IL),SCRB2(IR,IC,IL))
         ENDIF
  165   CONTINUE
        DO 166 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TSI = SCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .       + SCRB2(IR,IC,IL)*PHYT2(IR,IC,IL) 
         SK16P = SCRB1(IR,IC,IL)*(1.-FLOCEX)*GPP1(IR,IC,IL)
     .         + SCRB2(IR,IC,IL)*(1.-FLOCEX)*GPP2(IR,IC,IL)
         IF(FSM(IR,IC).LE.0.) DSI(IR,IC,IL) = SIT(IR,IC,IL)
     .        - (SCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)+SCRB2(IR,IC,IL)*
     .        PHYT2(IR,IC,IL))
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(16,SIT(IR,IC,IL),SK16P,DSI(IR,IC,IL),TSI,
     .          HBAR(IR,IC))
         ELSE
           CALL RCAWBUF(16,SIT_DDA(IR,IC,IL)/IAVGDDCNTR,SK16P,
     .          DSI_DDA(IR,IC,IL)/IAVGDDCNTR,TSI,HBAR(IR,IC))
         ENDIF
  166   CONTINUE
      ENDIF

C
C             SYSTEMS 17-22 - ORGANIC CARBON
C

c$doacross local(il,ic,ir,term,sk1719,sk1820,sr22p)
c$share(botbod,botbodhs)
      DO 170 IL=1,NLAYR
      DO 170 IC=1,NCOL
      DO 170 IR=1,NROW
      IF(FSM(IR,IC).LE.0)   GO TO 170

      TERM = XEMPRC(IR,IC,IL)

C        RPOC - REFRACTORY PARTICULATE ORGANIC CARBON
C                 Sources                         Sinks
C              Grazed Algae               Hydrolysis to RDOC
      SK1719 = K1719CT(ISTP20(IR,IC,IL))*RPOC(IR,IC,IL)*TERM

      CDRPOC(IR,IC,IL) = FRPOC*GRAZ(IR,IC,IL) - SK1719
 
C        LPOC - LABILE PARTICULATE ORGANIC CARBON
C                 Sources                         Sinks
C              Grazed Algae               Hydrolysis to LDOC
      SK1820 = K1820CT(ISTP20(IR,IC,IL))*LPOC(IR,IC,IL)*TERM

      CDLPOC(IR,IC,IL) = FLPOC*GRAZ(IR,IC,IL) - SK1820
 
C        RDOC - REFRACTORY DISSOLVED ORGANIC CARBON
C                 Sources                         Sinks
C              Grazed Algae                     Oxidation
C           From RPOC
      SK190(IR,IC,IL) = K190CT(ISTP20(IR,IC,IL))*RDOC(IR,IC,IL)
     .            *TERM*DO(IR,IC,IL)/(KDOC+DO(IR,IC,IL))

      CDRDOC(IR,IC,IL) = FRDOC*GRAZ(IR,IC,IL) + SK1719 - SK190(IR,IC,IL)
 
C        LDOC - LABILE DISSOLVED ORGANIC CARBON
C                 Sources                         Sinks
C              Grazed Algae                     Oxidation
C              From LPOC                     Denitrification
      SK200(IR,IC,IL) = K200CT(ISTP20(IR,IC,IL))*LDOC(IR,IC,IL)
     .            *LDOC(IR,IC,IL)/(KMLDOC+LDOC(IR,IC,IL))
     .            *TERM*DO(IR,IC,IL)/(KDOC+DO(IR,IC,IL))

C        Denitrification
C            ( 5 CH2O + 5 H2O + 4 NO3- + 4 H+ = 5 CO2 + 2 N2 + 12 H2O ) 
      SK2014(IR,IC,IL) = 5./4.*12./14.*SK140(IR,IC,IL)
  
      CDLDOC(IR,IC,IL) = FLDOC*GRAZ(IR,IC,IL) + SK1820
     .                 - SK2014(IR,IC,IL) - SK200(IR,IC,IL)
 
C        REDOC - REACTIVE ORGANIC CARBON (STPs)
C                 Sources                         Sinks
C                                               Oxidation
      SK210(IR,IC,IL) = K210CT(ISTP20(IR,IC,IL))*REDOC(IR,IC,IL)
     .            *REDOC(IR,IC,IL)/(KMLDOC+REDOC(IR,IC,IL))
     .            *TERM*DO(IR,IC,IL)/(KDOC+DO(IR,IC,IL))

      CDREDOC(IR,IC,IL) = - SK210(IR,IC,IL)

C        EXDOC - ALGAL EXUDATE ORGANIC CARBON
C                 Sources                         Sinks
C              Algal Exudate                    Oxidation
      SR22P = FLOCEX*(GPP1(IR,IC,IL)+GPP2(IR,IC,IL))

      SK220(IR,IC,IL) = K220CT(ISTP20(IR,IC,IL))*EXDOC(IR,IC,IL)
     .            *EXDOC(IR,IC,IL)/(KMLDOC+EXDOC(IR,IC,IL))
     .            *TERM*DO(IR,IC,IL)/(KDOC+DO(IR,IC,IL))

      CDEXDOC(IR,IC,IL) = SR22P - SK220(IR,IC,IL)
C           BOTTLE BOD5 - INCLUDES EFFECTS OF ALGAL RESPIRATION
      BOTBOD(IR,IC,IL)=OCRB*( RDOC(IR,IC,IL)*(1.-EXP(-5.*K190CT(201)))       
     .                 + LDOC(IR,IC,IL)*(1.-EXP(-5.*K200CT(201)))
     .                 + REDOC(IR,IC,IL)*(1.-EXP(-5.*K210CT(201)))
     .                 + EXDOC(IR,IC,IL)*(1.-EXP(-5.*K220CT(201)))
     .                 + PHYT1(IR,IC,IL)*(1.-EXP(-5.*K1RCT(201)))
     .                 + PHYT2(IR,IC,IL)*(1.-EXP(-5.*K2RCT(201))) )
     .          + 4.57*DNH3(IR,IC,IL)*(1.-EXP(-5.*K1314CT(201)))
      BOTBODHS(IR,IC,IL) = BOTBOD(IR,IC,IL)+
     .                 O2EQ(IR,IC,IL)*(1.-EXP(-5.*K230CT(201)))

  170 CONTINUE

      IF(IDISK.EQ.2 .OR. IDISK.EQ.3)  THEN
        DO 172 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TERM = XEMPRC(IR,IC,IL)
         SK1719 = K1719CT(ISTP20(IR,IC,IL))*RPOC(IR,IC,IL)*TERM
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(17,RPOC(IR,IC,IL),FRPOC*GRAZ(IR,IC,IL),SK1719,
     .          QX(IR,IC,IL),QY(IR,IC,IL))
         ELSE
           CALL RCAWBUF(17,RPOC_DDA(IR,IC,IL)/IAVGDDCNTR,
     .          FRPOC*GRAZ(IR,IC,IL),SK1719,QX(IR,IC,IL),QY(IR,IC,IL))
         ENDIF
  172   CONTINUE
        DO 173 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TTPOC = PHYT1(IR,IC,IL) + PHYT2(IR,IC,IL)
     .        + RPOC(IR,IC,IL) + LPOC(IR,IC,IL) 
         TERM = XEMPRC(IR,IC,IL)
         SK1820 = K1820CT(ISTP20(IR,IC,IL))*LPOC(IR,IC,IL)*TERM
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(18,LPOC(IR,IC,IL),FLPOC*GRAZ(IR,IC,IL),SK1820,
     .          TTPOC,DUMMY)
         ELSE
           CALL RCAWBUF(18,LPOC_DDA(IR,IC,IL)/IAVGDDCNTR,
     .          FLPOC*GRAZ(IR,IC,IL),SK1820,TTPOC,DUMMY)
         ENDIF
  173   CONTINUE
        DO 174 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TERM = XEMPRC(IR,IC,IL)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(19,RDOC(IR,IC,IL),FRDOC*GRAZ(IR,IC,IL),
     .          SK190(IR,IC,IL),RX(IR,IC,IL),RY(IR,IC,IL) )
         ELSE
           CALL RCAWBUF(19,RDOC_DDA(IR,IC,IL)/IAVGDDCNTR,
     .          FRDOC*GRAZ(IR,IC,IL),SK190(IR,IC,IL),RX(IR,IC,IL),
     .          RY(IR,IC,IL) )
         ENDIF
  174   CONTINUE
        DO 175 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(20,LDOC(IR,IC,IL),FLDOC*GRAZ(IR,IC,IL),
     .          SK200(IR,IC,IL),SK2014(IR,IC,IL),DUMMY)
         ELSE
           CALL RCAWBUF(20,LDOC_DDA(IR,IC,IL)/IAVGDDCNTR,
     .          FLDOC*GRAZ(IR,IC,IL),SK200(IR,IC,IL),SK2014(IR,IC,IL),
     .          DUMMY)
         ENDIF
  175   CONTINUE
        DO 176 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
C        ALGAL PRODUCTION USING CO2 AND NH3
      SR24PA = 32./12.*(PNH31(IR,IC,IL)*GPP1(IR,IC,IL)
     .                + PNH32(IR,IC,IL)*GPP2(IR,IC,IL))

C        ALGAL PRODUCTION USING CO2 AND NO3  (HNO3 + H2O => NH3 + 2O2)
      SR24PB = (1.-PNH31(IR,IC,IL))*GPP1(IR,IC,IL)
     .                   * 32.*(1./12.+2.*NCRB1(IR,IC,IL)/14.)
     .       + (1.-PNH32(IR,IC,IL))*GPP2(IR,IC,IL)
     .                   * 32.*(1./12.+2.*NCRB2(IR,IC,IL)/14.)

C        ALGAL RESPIRATION
      SK24P = OCRB*RESP(IR,IC,IL)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(21,REDOC(IR,IC,IL),SK210(IR,IC,IL),
     .          BOTBOD(IR,IC,IL),BOTBODHS(IR,IC,IL),DUMMY )
         ELSE
           CALL RCAWBUF(21,REDOC_DDA(IR,IC,IL)/IAVGDDCNTR,
     .     SK210(IR,IC,IL),SR24PA+SR24PB,SK24P,BOTBODHS(IR,IC,IL) )
         ENDIF
  176   CONTINUE
        DO 177 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         TTPOC = PHYT1(IR,IC,IL) + PHYT2(IR,IC,IL)
     .        + RPOC(IR,IC,IL) + LPOC(IR,IC,IL) 
         TTDOC = RDOC(IR,IC,IL) + LDOC(IR,IC,IL) + REDOC(IR,IC,IL) 
     .        + EXDOC(IR,IC,IL) 
         TOC = TTPOC + TTDOC
         SR22P = FLOCEX*(GPP1(IR,IC,IL)+GPP2(IR,IC,IL))
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(22,EXDOC(IR,IC,IL),SK220(IR,IC,IL),SR22P,
     .          TTDOC,TOC)
         ELSE
           CALL RCAWBUF(22,EXDOC_DDA(IR,IC,IL)/IAVGDDCNTR,
     .          SK220(IR,IC,IL),SR22P,TTDOC,TOC)
         ENDIF
  177   CONTINUE
      ENDIF

C
C             SYSTEM 23 - O2*
C

c$doacross local(il,ic,ir)
      DO 180 IL=1,NLAYR
      DO 180 IC=1,NCOL
      DO 180 IR=1,NROW
      IF(FSM(IR,IC).LE.0)   GO TO 180

C        O2 Demanding Equivalents (H2S + CH4)
C                 Sources                         Sinks
C             (Sediment Flux)                   Oxidation
      SK230(IR,IC,IL) = K230CT(ISTP20(IR,IC,IL))*O2EQ(IR,IC,IL)
     .            *XEMPRC(IR,IC,IL)*DO(IR,IC,IL)/(KO2EQ+DO(IR,IC,IL))
      CDO2EQ(IR,IC,IL) = - SK230(IR,IC,IL)

  180 CONTINUE

      IF(IDISK.EQ.2 .OR. IDISK.EQ.3)  THEN
        DO 185 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         SK24OC(IR,IC,IL) = OCRB*(SK190(IR,IC,IL) + SK200(IR,IC,IL)
     .             + SK210(IR,IC,IL) + SK220(IR,IC,IL))
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(23,O2EQ(IR,IC,IL),SK230(IR,IC,IL),
     .          SK2413(IR,IC,IL),SK24OC(IR,IC,IL),HYDSAL(IR,IC,IL))
         ELSE
           CALL RCAWBUF(23,O2EQ_DDA(IR,IC,IL)/IAVGDDCNTR,
     .          SK230(IR,IC,IL),SK2413(IR,IC,IL)*64./14.,
     .          SK24OC(IR,IC,IL),ECMSAL_DDA(IR,IC,IL)/IAVGDDCNTR)
         ENDIF
  185   CONTINUE
      ENDIF

C
C             SYSTEM 24 - DISSOLVED OXYGEN
C

c$doacross local(il,ic,ir,sr24pa,sr24pb,sk24p) ,
c$& share (cs,doaveg,doming,domaxg) ,
c$& local (qxave,qyave,qvect1,qvect2,klvel,kavel)
      DO 190 IL=1,NLAYR
      DO 190 IC=1,NCOL
      DO 190 IR=1,NROW
      DOAVEG(IR,IC,IL) = DOAVEG(IR,IC,IL) + DO(IR,IC,IL)
      DOMING(IR,IC,IL) = AMIN1(DOMING(IR,IC,IL),DO(IR,IC,IL))
      DOMAXG(IR,IC,IL) = AMAX1(DOMAXG(IR,IC,IL),DO(IR,IC,IL))
      DO_DMIN(IR,IC,IL) = AMIN1(DO_DMIN(IR,IC,IL),DO(IR,IC,IL))
      DO_DMAX(IR,IC,IL) = AMAX1(DO_DMAX(IR,IC,IL),DO(IR,IC,IL))
      IF(FSM(IR,IC).LE.0)   GO TO 190


C                 Sources                         Sinks
C               Reaeration                    Nitrification
C              Photosynthesis                  Respiration
C                                         Oxidation of Org Carbon
C                                         Oxidation of O2* (H2S/CH4)
C        REAERATION - SURFACE SEGMENT ONLY
      IF(IL.EQ.1)  THEN
C           COMPUTE OXYGEN SATURATION
        CS(IR,IC) = 14.6244 - 0.36713*HYDTEMP(IR,IC,1)
     .    + 0.0044972*HYDTEMP(IR,IC,1)*HYDTEMP(IR,IC,1)
     .    - 0.0966*SAL(IR,IC,1) + 0.00205*SAL(IR,IC,1)*HYDTEMP(IR,IC,1)
     .    + 0.0002739*SAL(IR,IC,1)*SAL(IR,IC,1)
        QXAVE = (QX(IR,IC,1)/XAX(IR,IC,1)+QX(IR+1,IC,1)/XAX(IR+1,IC,1))
     .           /(FSM(IR,IC)+ABS(FSM(IR+1,IC)))
        QYAVE = (QY(IR,IC,1)/XAY(IR,IC,1)+QY(IR,IC+1,1)/XAY(IR,IC+1,1))
     .          /(FSM(IR,IC)+ABS(FSM(IR,IC+1)))
        QVECT1 = SQRT(QXAVE*QXAVE + QYAVE*QYAVE)
        QXAVE = (QX(IR,IC,2)/XAX(IR,IC,2)+QX(IR+1,IC,2)/XAX(IR+1,IC,2))
     .           /(FSM(IR,IC)+ABS(FSM(IR+1,IC)))
        QYAVE = (QY(IR,IC,2)/XAY(IR,IC,2)+QY(IR,IC+1,2)/XAY(IR,IC+1,2))
     .          /(FSM(IR,IC)+ABS(FSM(IR,IC+1)))
        QVECT2 = SQRT(QXAVE*QXAVE + QYAVE*QYAVE)
        KLVEL = SQRT(DIFUS*ABS(QVECT1-QVECT2)/
     .                       ((ZZ(1)+ZZ(2))/2. * HBAR(IR,IC)))
        KAVEL = MAX(KLVEL,KLMIN)/(HBAR(IR,IC)*DZ(1))
        SR240(IR,IC,1) = KA20(IR,IC)*KACT(ISTP20(IR,IC,1))
     .     * (CS(IR,IC)-DO(IR,IC,1))
      ENDIF

C        ALGAL PRODUCTION USING CO2 AND NH3
      SR24PA = 32./12.*(PNH31(IR,IC,IL)*GPP1(IR,IC,IL)
     .                + PNH32(IR,IC,IL)*GPP2(IR,IC,IL))

C        ALGAL PRODUCTION USING CO2 AND NO3  (HNO3 + H2O => NH3 + 2O2)
      SR24PB = (1.-PNH31(IR,IC,IL))*GPP1(IR,IC,IL)
     .                   * 32.*(1./12.+2.*NCRB1(IR,IC,IL)/14.)
     .       + (1.-PNH32(IR,IC,IL))*GPP2(IR,IC,IL)
     .                   * 32.*(1./12.+2.*NCRB2(IR,IC,IL)/14.)

C        ALGAL RESPIRATION
      SK24P = OCRB*RESP(IR,IC,IL)

C        NITRIFICATION  (NH3 + 2O2 => NO3 + H2O + H)
      SK2413(IR,IC,IL) = 64./14.*SK1314(IR,IC,IL)

C        OXIDATION OF ORGANIC CARBON
      SK24OC(IR,IC,IL) = OCRB*(SK190(IR,IC,IL) + SK200(IR,IC,IL)
     .                  + SK210(IR,IC,IL) + SK220(IR,IC,IL))

      CDDO(IR,IC,IL) = SR240(IR,IC,IL) + SR24PA + SR24PB - SK24P 
     .    - SK2413(IR,IC,IL) - SK24OC(IR,IC,IL) - SK230(IR,IC,IL)

  190 CONTINUE

      IF(IDISK.EQ.2 .OR. IDISK.EQ.3)  THEN
        DO 195 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         SR24PA = 32./12.*(PNH31(IR,IC,IL)*GPP1(IR,IC,IL)
     .                + PNH32(IR,IC,IL)*GPP2(IR,IC,IL))
         SR24PB = (1.-PNH31(IR,IC,IL))*GPP1(IR,IC,IL)
     .                   * 32.*(1./12.+2.*NCRB1(IR,IC,IL)/14.)
     .          + (1.-PNH32(IR,IC,IL))*GPP2(IR,IC,IL)
     .                   * 32.*(1./12.+2.*NCRB2(IR,IC,IL)/14.)
         SK24P = OCRB*RESP(IR,IC,IL)
         IF(IDDOPT.EQ.0)  THEN
           CALL RCAWBUF(24,DO(IR,IC,IL),SR240(IR,IC,IL),SR24PA,SR24PB,
     .          SK24P)
         ELSE
           CALL RCAWBUF(24,DO_DDA(IR,IC,IL)/IAVGDDCNTR,SR240(IR,IC,IL),
     .        DO_DMIN(IR,IC,IL),DO_DMAX(IR,IC,IL),(SR24PA+SR24PB-SK24P))
         ENDIF
  195   CONTINUE
      ENDIF

C
C        SYSTEM 25 - TOTAL ACTIVE METAL
C
      DO 200 IL=1,NLAYR
      DO 200 IC=1,NCOL
      DO 200 IR=1,NROW
        CDRCATEMP(IR,IC,IL) = 0.0
  200 CONTINUE

      IF(IDISK.EQ.2 .OR. IDISK.EQ.3)  THEN
        DO 205 IDMP=1,NDMPS
         IR = IFDMPS(IDMP,1)
         IC = IFDMPS(IDMP,2)
         IL = IFDMPS(IDMP,3)
         IF(IL.EQ.1)  THEN
           DOSAT = CS(IR,IC)
         ELSE
           DOSAT = 0.0
         ENDIF

         IF(IDDOPT.EQ.0)  THEN
         CALL RCAWBUF(25,RCATEMP(IR,IC,IL),PTAM(IR,IC,IL),DOSAT,
     .        BVOL(IR,IC,IL),HYDTEMP(IR,IC,IL))
         ELSE
           CALL RCAWBUF(25,TEM_DDA(IR,IC,IL)/IAVGDDCNTR,PTAM(IR,IC,IL),
     .        DOSAT,BVOL(IR,IC,IL),ECMTEM_DDA(IR,IC,IL)/IAVGDDCNTR)
         ENDIF
  205   CONTINUE
      ENDIF

C        CLOSE BUFFER

      IF(IDISK.EQ.2 .OR. IDISK.EQ.3)  THEN
         CALL RCAWRIT

C        RE-INITIALIZE ARRAY FOR DETAILED DUMP AVERAGING, IF REQUIRED
        IF(IDDOPT.EQ.1)  THEN
c$doacross local(il,ic,ir) , share(sal_dda,ecmsal_dda),
c$& share(phyt1_dda,phyt2_dda),
c$& share(rpop_dda,lpop_dda,rdop_dda,ldop_dda,dpo4_dda),
c$& share(rpon_dda,lpon_dda,rdon_dda,ldon_dda,dnh3_dda,no23_dda),
c$& share(bsi_dda,dsi_dda),                                             !RRI
c$& share(rpoc_dda,lpoc_dda,rdoc_dda,ldoc_dda,redoc_dda,exdoc_dda),
c$& share(o2eq_dda,do_dda,tem_dda,ecmtem_dda)
          DO 210 IL=1,NLAYR
           DO 210 IC=1,NCOL
            DO 210 IR=1,NROW
              SAL_DDA(IR,IC,IL) = 0.
              ECMSAL_DDA(IR,IC,IL) = 0.
              PHYT1_DDA(IR,IC,IL) = 0.
              PHYT2_DDA(IR,IC,IL) = 0.
              RPOP_DDA(IR,IC,IL) = 0.
              LPOP_DDA(IR,IC,IL) = 0.
              RDOP_DDA(IR,IC,IL) = 0.
              LDOP_DDA(IR,IC,IL) = 0.
              DPO4_DDA(IR,IC,IL) = 0.
              RPON_DDA(IR,IC,IL) = 0.
              LPON_DDA(IR,IC,IL) = 0.
              RDON_DDA(IR,IC,IL) = 0.
              LDON_DDA(IR,IC,IL) = 0.
              DNH3_DDA(IR,IC,IL) = 0.
              NO23_DDA(IR,IC,IL) = 0.
              BSI_DDA(IR,IC,IL) = 0.
              DSI_DDA(IR,IC,IL) = 0.
              RPOC_DDA(IR,IC,IL) = 0.
              LPOC_DDA(IR,IC,IL) = 0.
              RDOC_DDA(IR,IC,IL) = 0.
              LDOC_DDA(IR,IC,IL) = 0.
              REDOC_DDA(IR,IC,IL) = 0.
              EXDOC_DDA(IR,IC,IL) = 0.
              O2EQ_DDA(IR,IC,IL) = 0.
              DO_DDA(IR,IC,IL) = 0.
              TEM_DDA(IR,IC,IL) = 0.
              ECMTEM_DDA(IR,IC,IL) = 0.
              PO4T_DDA(IR,IC,IL) = 0.
              NH3T_DDA(IR,IC,IL) = 0.
              SIT_DDA(IR,IC,IL) = 0.
              DO_DMIN(IR,IC,IL) = 1000.
              DO_DMAX(IR,IC,IL) = -1000.
  210       CONTINUE
          IAVGDDCNTR = 0
        ENDIF
      ENDIF

C        PERFORM GLOBAL DUMP AVERAGING, IF REQUIRED
      IF(IGDOPT.EQ.1)  THEN
c$doacross local(il,ic,ir) , share(sal_gda,ecmsal_gda,phyt1_gda),
c$& share(phyt2_gda,tpop_gda,tdop_gda,dpo4_gda),
c$& share(tpon_gda,tdon_gda,dnh3_gda,no23_gda)
        DO 220 IL=1,NLAYR
         DO 220 IC=1,NCOL
          DO 220 IR=1,NROW
            SAL_GDA(IR,IC,IL) = SAL_GDA(IR,IC,IL) + SAL(IR,IC,IL)
            ECMSAL_GDA(IR,IC,IL) = ECMSAL_GDA(IR,IC,IL) +
     .             HYDSAL(IR,IC,IL)
            PHYT1_GDA(IR,IC,IL) = PHYT1_GDA(IR,IC,IL) + PHYT1(IR,IC,IL)
            PHYT2_GDA(IR,IC,IL) = PHYT2_GDA(IR,IC,IL) + PHYT2(IR,IC,IL)
            TPOP_GDA(IR,IC,IL) = TPOP_GDA(IR,IC,IL) +
     .             PCRB1(IR,IC,IL)*PHYT1(IR,IC,IL) +
     .             PCRB2(IR,IC,IL)*PHYT2(IR,IC,IL) +
     .             RPOP(IR,IC,IL) + LPOP(IR,IC,IL)
            TDOP_GDA(IR,IC,IL) = TDOP_GDA(IR,IC,IL) +
     .             RDOP(IR,IC,IL) + LDOP(IR,IC,IL)
            DPO4_GDA(IR,IC,IL) = DPO4_GDA(IR,IC,IL) + DPO4(IR,IC,IL)
            TPON_GDA(IR,IC,IL) = TPON_GDA(IR,IC,IL) +
     .             NCRB1(IR,IC,IL)*PHYT1(IR,IC,IL) +
     .             NCRB2(IR,IC,IL)*PHYT2(IR,IC,IL) +
     .             RPON(IR,IC,IL) + LPON(IR,IC,IL)
            TDON_GDA(IR,IC,IL) = TDON_GDA(IR,IC,IL) +
     .             RDON(IR,IC,IL) + LDON(IR,IC,IL)
            DNH3_GDA(IR,IC,IL) = DNH3_GDA(IR,IC,IL) + DNH3(IR,IC,IL)
            NO23_GDA(IR,IC,IL) = NO23_GDA(IR,IC,IL) + NO23(IR,IC,IL)
  220   CONTINUE
c$doacross local(il,ic,ir,depth) ,
c$& share(tpoc_gda,tdoc_gda,tpsi_gda,dsi_gda),
c$& share(o2eq_gda,tgpdp_gda,tresp_gda,ske_gda),
c$& share(rcatmp_gda,ecmtmp_gda,botbodhs_gda)
        DO 225 IL=1,NLAYR
         DO 225 IC=1,NCOL
          DO 225 IR=1,NROW
            TPSI_GDA(IR,IC,IL) = TPSI_GDA(IR,IC,IL) + BSI(IR,IC,IL) +
     .             SCRB1(IR,IC,IL)*PHYT1(IR,IC,IL) +
     .             SCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)
            DSI_GDA(IR,IC,IL) = DSI_GDA(IR,IC,IL) + DSI(IR,IC,IL)
            TPOC_GDA(IR,IC,IL) = TPOC_GDA(IR,IC,IL) +
     .             PHYT1(IR,IC,IL) + PHYT2(IR,IC,IL) +
     .             RPOC(IR,IC,IL) + LPOC(IR,IC,IL)
            TDOC_GDA(IR,IC,IL) = TDOC_GDA(IR,IC,IL) +
     .             RDOC(IR,IC,IL) + LDOC(IR,IC,IL) +
     .             REDOC(IR,IC,IL) + EXDOC(IR,IC,IL)
            O2EQ_GDA(IR,IC,IL) = O2EQ_GDA(IR,IC,IL) + O2EQ(IR,IC,IL)
            TGPDP_GDA(IR,IC,IL) = TGPDP_GDA(IR,IC,IL) + 0.0
C        NET PRIMARY PRODUCTIVITY TO APPROXIMATELY THE
C        1 PERCENT LIGHT LEVEL  -  GM C/M**2-DAY
            DEPTH =(ZZ(IL)+ZZ(IL+1))/2. * HBAR(IR,IC) 
            IF(EXP(-SKE(IR,IC,IL)*DEPTH).GT.0.01) THEN
              TGPDP_GDA(IR,IC,IL) = TGPDP_GDA(IR,IC,IL) +
     .            DZ(IL)*HBAR(IR,IC) * (GPP1(IR,IC,IL) + GPP2(IR,IC,IL)
     .                                - DPP1(IR,IC,IL) - DPP2(IR,IC,IL))
            ELSE
             TGPDP_GDA(IR,IC,IL) = 0.0
            ENDIF
            TRESP_GDA(IR,IC,IL) = TRESP_GDA(IR,IC,IL) +
     .             OCRB*RESP(IR,IC,IL) + SK24OC(IR,IC,IL) +
     .             SK230(IR,IC,IL) + SK2413(IR,IC,IL)
            SKE_GDA(IR,IC,IL) = SKE_GDA(IR,IC,IL) + SKE(IR,IC,IL)
            RCATMP_GDA(IR,IC,IL) = RCATMP_GDA(IR,IC,IL) +
     .             RCATEMP(IR,IC,IL)
            ECMTMP_GDA(IR,IC,IL) = ECMTMP_GDA(IR,IC,IL) +
     .             HYDTEMP(IR,IC,IL)
            BOTBODHS_GDA(IR,IC,IL) = BOTBODHS_GDA(IR,IC,IL) +
     .             BOTBODHS(IR,IC,IL)
  225   CONTINUE
      ENDIF

      IAVGGDCNTR = IAVGGDCNTR + 1

C        CONVERT TO MASS UNITS

c$doacross local(il,ic,ir,isys)
      DO 230 ISYS=1,NOSYS
       DO 230 IL=1,NLAYR
        DO 230 IC=1,NCOL
         DO 230 IR=1,NROW
          CDARAY(IR,IC,IL,ISYS) = BVOL(IR,IC,IL)*CDARAY(IR,IC,IL,ISYS)
  230 CONTINUE

C        SETTLING AND DEPOSITION OF POM

C        SETTLING
c$doacross local(ic,ir,il,alg1flx,alg2flx,algpflx,algnflx,algsflx),
c$& local(flux,pflux,sflux,term)
      DO 240 IC=1,NCOL
       DO 240 IL=1,NLAYR-1
        DO 240 IR=1,NROW
         IF(FSM(IR,IC).LE.0)   GO TO 240

C        WINTER DIATOMS
      ALG1FLX =  ALG1VS(IR,IC,IL)*XAZ(IR,IC)*PHYT1(IR,IC,IL)
      CDPHYT1(IR,IC,IL) = CDPHYT1(IR,IC,IL) - ALG1FLX
      CDPHYT1(IR,IC,IL+1) = CDPHYT1(IR,IC,IL+1) + ALG1FLX

C        SUMMER PHYTOPLANKTON
      ALG2FLX = ALG2VS(IR,IC,IL)*XAZ(IR,IC)*PHYT2(IR,IC,IL)
      CDPHYT2(IR,IC,IL) = CDPHYT2(IR,IC,IL) - ALG2FLX
      CDPHYT2(IR,IC,IL+1) = CDPHYT2(IR,IC,IL+1) + ALG2FLX

C        ALGAL BOUND PHOSPHORUS, NITROGEN AND SILICA
      ALGPFLX = PCRB1(IR,IC,IL)*ALG1FLX + PCRB2(IR,IC,IL)*ALG2FLX
      CDPO4T(IR,IC,IL) = CDPO4T(IR,IC,IL) - ALGPFLX
      CDPO4T(IR,IC,IL+1) = CDPO4T(IR,IC,IL+1) + ALGPFLX
      ALGNFLX = NCRB1(IR,IC,IL)*ALG1FLX + NCRB2(IR,IC,IL)*ALG2FLX
      CDNH3T(IR,IC,IL) = CDNH3T(IR,IC,IL) - ALGNFLX
      CDNH3T(IR,IC,IL+1) = CDNH3T(IR,IC,IL+1) + ALGNFLX
      ALGSFLX = SCRB1(IR,IC,IL)*ALG1FLX + SCRB2(IR,IC,IL)*ALG2FLX
      CDSIT(IR,IC,IL) = CDSIT(IR,IC,IL) - ALGSFLX
      CDSIT(IR,IC,IL+1) = CDSIT(IR,IC,IL+1) + ALGSFLX

C        INORGANIC SETTLING OF PHOSPHORUS AND SILICA ASSOCIATED WITH TAM
      FLUX = VSPIM*VSPMT**STP20(IR,IC,IL)*XAZ(IR,IC)
      PFLUX = FLUX*PO4TAM(IR,IC,IL)
      CDPO4T(IR,IC,IL) = CDPO4T(IR,IC,IL) - PFLUX
      CDPO4T(IR,IC,IL+1) = CDPO4T(IR,IC,IL+1) + PFLUX
      SFLUX = FLUX*SITAM(IR,IC,IL)
      CDSIT(IR,IC,IL) = CDSIT(IR,IC,IL) - SFLUX
      CDSIT(IR,IC,IL+1) = CDSIT(IR,IC,IL+1) + SFLUX

C        PARTICULATE ORGANIC MATTER
      FLUX = VSPOM*VSPMT**STP20(IR,IC,IL)*XAZ(IR,IC)
      TERM = FLUX*RPOP(IR,IC,IL)
      CDRPOP(IR,IC,IL) = CDRPOP(IR,IC,IL) - TERM
      CDRPOP(IR,IC,IL+1) = CDRPOP(IR,IC,IL+1) + TERM
      TERM = FLUX*LPOP(IR,IC,IL)
      CDLPOP(IR,IC,IL) = CDLPOP(IR,IC,IL) - TERM
      CDLPOP(IR,IC,IL+1) = CDLPOP(IR,IC,IL+1) + TERM
      TERM = FLUX*RPON(IR,IC,IL)
      CDRPON(IR,IC,IL) = CDRPON(IR,IC,IL) - TERM
      CDRPON(IR,IC,IL+1) = CDRPON(IR,IC,IL+1) + TERM
      TERM = FLUX*LPON(IR,IC,IL)
      CDLPON(IR,IC,IL) = CDLPON(IR,IC,IL) - TERM
      CDLPON(IR,IC,IL+1) = CDLPON(IR,IC,IL+1) + TERM
      TERM = FLUX*BSI(IR,IC,IL)
      CDBSI(IR,IC,IL) = CDBSI(IR,IC,IL) - TERM
      CDBSI(IR,IC,IL+1) = CDBSI(IR,IC,IL+1) + TERM
      TERM = FLUX*RPOC(IR,IC,IL)
      CDRPOC(IR,IC,IL) = CDRPOC(IR,IC,IL) - TERM
      CDRPOC(IR,IC,IL+1) = CDRPOC(IR,IC,IL+1) + TERM
      TERM = FLUX*LPOC(IR,IC,IL)
      CDLPOC(IR,IC,IL) = CDLPOC(IR,IC,IL) - TERM
      CDLPOC(IR,IC,IL+1) = CDLPOC(IR,IC,IL+1) + TERM
  240 CONTINUE

C        DEPOSITION
Cc$doacross local(ic,ir,tcorr,vssed)
c$doacross local(ic,ir,tcorr,vssed,alg1flx,alg2flx),
c$& local(algpflx,algnflx,algsflx)
      DO 250 IC=1,NCOL
      DO 250 IR=1,NROW
      IF(FSM(IR,IC).LE.0)   GO TO 250

      TCORR = VSSEDT**STP20(IR,IC,NLAYR)
      DEPFLUX(IR,IC,1) = ALG1VS(IR,IC,NLAYR)*VSNET1(IR,IC)*TCORR
     .               *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR)
Crlm9495-35 revise depflux to include region specific temperature dependent
Crlm        benthic filtration
      IF(IR.GE.25.AND.IR.LE.27.AND.IC.GE.25.AND.IC.LE.26) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.02*XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Cjrw RARITAN, HACKENSACK, AND PASAIC RIVERS 7-11-01
      IF(IR.EQ.2.AND.IC.GE.24.AND.IC.LE.61) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.GE.3.AND.IR.LE.4.AND.IC.EQ.24) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.6.AND.IC.GE.46.AND.IC.LE.64) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.7.AND.IC.EQ.46) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.8.AND.IC.GE.47.AND.IC.LE.67) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm NEWARK BAY + KILLS
      IF(IR.EQ.4.AND.IC.GE.26.AND.IC.LE.39) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.14     
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.GE.5.AND.IR.LE.7.AND.IC.EQ.39) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.14
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.GE.7.AND.IR.LE.15.AND.IC.EQ.40) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.14
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.8.AND.IC.GE.41.AND.IC.LE.46) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.14
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.7.AND.IC.GE.41.AND.IC.LE.43) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.14
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm RARITAN BAY
      IF(IR.GE.5.AND.IR.LE.17.AND.IC.GE.20.AND.IC.LE.33) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.4        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm NY BIGHT
      IF(IR.GE.18.AND.IC.LE.24) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.1        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm NY BIGHT
      IF(IR.GE.18.AND.IR.LE.24.AND.IC.GE.20.AND.IC.LE.24) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.3        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm LOWER NY BAY
      IF(IR.GE.18.AND.IR.LE.24.AND.IC.LE.35.AND.IC.GE.25) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*1.5        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm JAMAICA BAY
      IF(IR.GE.25.AND.IR.LE.36.AND.IC.GE.28.AND.IC.LE.34) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.06       
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.GE.37.AND.IR.LE.39.AND.IC.GE.28.AND.IC.LE.29) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.06       
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm UPPER NY BAY
      IF(IR.GE.16.AND.IR.LE.23.AND.IC.GE.36.AND.IC.LE.47) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.4        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm NORTH RIVER
      IF(IR.GE.18.AND.IR.LE.20.AND.IC.GE.48.AND.IC.LE.73) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.4        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm EAST RIVER+ LITTLE NECK BAY area
      IF(IR.GE.22.AND.IR.LE.29.AND.IC.GE.48.AND.IC.LE.59) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*5.0
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.GE.30.AND.IR.LE.36.AND.IC.GE.48.AND.IC.LE.59) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.2        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.GE.37.AND.IR.LE.39.AND.IC.GE.53.AND.IC.LE.64) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.2        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
Crlm LONG ISLAND SOUND ( to mp 60 )does not include Little Neck Bay area
      IF(IR.GE.36.AND.IR.LE.46.AND.IC.GE.45.AND.IC.LE.52) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.55        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF
      IF(IR.GE.40.AND.IR.LE.46.AND.IC.GE.53.AND.IC.LE.58) THEN
      DEPFLUX(IR,IC,1) = DEPFLUX(IR,IC,1)+(1.08**STP20(IR,IC,NLAYR)*0.55        
     .     *XAZ(IR,IC)*PHYT1(IR,IC,NLAYR))
      END IF

      ALG1FLX=DEPFLUX(IR,IC,1)
      CDPHYT1(IR,IC,NLAYR) = CDPHYT1(IR,IC,NLAYR) - DEPFLUX(IR,IC,1)
      DEPFLUX(IR,IC,2) = ALG2VS(IR,IC,NLAYR)*VSNET2(IR,IC)*TCORR
     .               *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR)
Crlm9495-35 revise depflux to include region specific temperature dependent
Crlm        benthic filtration
      IF(IR.GE.25.AND.IR.LE.27.AND.IC.GE.25.AND.IC.LE.26) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.02*XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Cjrw RARITAN, HACKENSACK, AND PASAIC RIVERS 7-11-01
      IF(IR.EQ.2.AND.IC.GE.24.AND.IC.LE.61) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.GE.3.AND.IR.LE.4.AND.IC.EQ.24) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.6.AND.IC.GE.46.AND.IC.LE.64) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.7.AND.IC.EQ.46) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.8.AND.IC.GE.47.AND.IC.LE.67) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*
     .     0.046*XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm NEWARK BAY + KILLS
      IF(IR.EQ.4.AND.IC.GE.26.AND.IC.LE.39) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.14        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.GE.5.AND.IR.LE.7.AND.IC.EQ.39) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.14
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.GE.7.AND.IR.LE.15.AND.IC.EQ.40) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.14
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.8.AND.IC.GE.41.AND.IC.LE.46) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.14
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.EQ.7.AND.IC.GE.41.AND.IC.LE.43) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.14
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm RARITAN BAY
      IF(IR.GE.5.AND.IR.LE.17.AND.IC.GE.20.AND.IC.LE.33) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.4        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm NY BIGHT
      IF(IR.GE.18.AND.IC.LE.24) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.1        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm NY BIGHT
      IF(IR.GE.18.AND.IR.LE.24.AND.IC.GE.20.AND.IC.LE.24) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.3        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm LOWER NY BAY
      IF(IR.GE.18.AND.IR.LE.24.AND.IC.LE.35.AND.IC.GE.25) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*1.5        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm JAMAICA BAY
      IF(IR.GE.25.AND.IR.LE.36.AND.IC.GE.28.AND.IC.LE.34) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.06       
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.GE.37.AND.IR.LE.39.AND.IC.GE.28.AND.IC.LE.29) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.06       
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm UPPER NY BAY
      IF(IR.GE.16.AND.IR.LE.23.AND.IC.GE.36.AND.IC.LE.47) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.4        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm NORTH RIVER
      IF(IR.GE.18.AND.IR.LE.20.AND.IC.GE.48.AND.IC.LE.73) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.4        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm EAST RIVER+LITTLE NECK BAY AREA
      IF(IR.GE.22.AND.IR.LE.29.AND.IC.GE.48.AND.IC.LE.59) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*5.0        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.GE.30.AND.IR.LE.36.AND.IC.GE.48.AND.IC.LE.59) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.2        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.GE.37.AND.IR.LE.39.AND.IC.GE.53.AND.IC.LE.64) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.2        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
Crlm LONG ISLAND SOUND ( to mp 60 )excludes Little Neck Bay
      IF(IR.GE.36.AND.IR.LE.46.AND.IC.GE.45.AND.IC.LE.52) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.55        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF
      IF(IR.GE.40.AND.IR.LE.46.AND.IC.GE.53.AND.IC.LE.58) THEN
      DEPFLUX(IR,IC,2) = DEPFLUX(IR,IC,2)+(1.08**STP20(IR,IC,NLAYR)*0.55        
     .     *XAZ(IR,IC)*PHYT2(IR,IC,NLAYR))
      END IF

      ALG2FLX=DEPFLUX(IR,IC,2)
      CDPHYT2(IR,IC,NLAYR) = CDPHYT2(IR,IC,NLAYR) - DEPFLUX(IR,IC,2)

C     ALGAL BOUND PHOSPHORUS, NITROGEN, AND SILICA
      ALGPFLX = PCRB1(IR,IC,NLAYR)*ALG1FLX + PCRB2(IR,IC,NLAYR)*ALG2FLX
      CDPO4T(IR,IC,NLAYR) = CDPO4T(IR,IC,NLAYR) - ALGPFLX
      ALGNFLX = NCRB1(IR,IC,NLAYR)*ALG1FLX + NCRB2(IR,IC,NLAYR)*ALG2FLX
      CDNH3T(IR,IC,NLAYR) = CDNH3T(IR,IC,NLAYR) - ALGNFLX
      ALGSFLX = SCRB1(IR,IC,NLAYR)*ALG1FLX + SCRB2(IR,IC,NLAYR)*ALG2FLX
      CDSIT(IR,IC,NLAYR) = CDSIT(IR,IC,NLAYR) - ALGSFLX

      VSSED = VSPOM*VSPMT**STP20(IR,IC,NLAYR)
     .           *VSNET3(IR,IC)*TCORR*XAZ(IR,IC)
      IF(IR.GE.25.AND.IR.LE.27.AND.IC.GE.25.AND.IC.LE.26) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.02*XAZ(IR,IC))
      END IF
Cjrw RARITAN, HACKENSACK, AND PASAIC RIVERS 7-11-01
      IF(IR.EQ.2.AND.IC.GE.24.AND.IC.LE.61) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.046*XAZ(IR,IC))
      END IF
      IF(IR.GE.3.AND.IR.LE.4.AND.IC.EQ.24) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.046*XAZ(IR,IC))
      END IF
      IF(IR.EQ.6.AND.IC.GE.46.AND.IC.LE.64) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.046*XAZ(IR,IC))
      END IF
      IF(IR.EQ.7.AND.IC.EQ.46) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.046*XAZ(IR,IC))
      END IF
      IF(IR.EQ.8.AND.IC.GE.47.AND.IC.LE.67) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.046*XAZ(IR,IC))
      END IF
Crlm NEWARK BAY + KILLS
      IF(IR.EQ.4.AND.IC.GE.26.AND.IC.LE.39) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.14*XAZ(IR,IC))
      END IF
      IF(IR.GE.5.AND.IR.LE.7.AND.IC.EQ.39) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.14*XAZ(IR,IC))
      END IF
      IF(IR.GE.7.AND.IR.LE.15.AND.IC.EQ.40) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.14*XAZ(IR,IC))
      END IF
      IF(IR.EQ.8.AND.IC.GE.41.AND.IC.LE.46) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.14*XAZ(IR,IC))
      END IF
      IF(IR.EQ.7.AND.IC.GE.41.AND.IC.LE.43) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.14*XAZ(IR,IC))
      END IF
Crlm RARITAN BAY
      IF(IR.GE.5.AND.IR.LE.17.AND.IC.GE.20.AND.IC.LE.33) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.4*XAZ(IR,IC))
      END IF
Crlm NY BIGHT
      IF(IR.GE.18.AND.IC.LE.24) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.1*XAZ(IR,IC))
      END IF
Crlm NY BIGHT
      IF(IR.GE.18.AND.IR.LE.24.AND.IC.GE.20.AND.IC.LE.24) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.3*XAZ(IR,IC))
      END IF
Crlm LOWER NY BAY
      IF(IR.GE.18.AND.IR.LE.24.AND.IC.LE.35.AND.IC.GE.25) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*1.5*XAZ(IR,IC))
      END IF
Crlm JAMAICA BAY
      IF(IR.GE.25.AND.IR.LE.36.AND.IC.GE.28.AND.IC.LE.34) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.06*XAZ(IR,IC))
      END IF
      IF(IR.GE.37.AND.IR.LE.39.AND.IC.GE.28.AND.IC.LE.29) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.06*XAZ(IR,IC))
      END IF
Crlm UPPER NY BAY
      IF(IR.GE.16.AND.IR.LE.23.AND.IC.GE.36.AND.IC.LE.47) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.4*XAZ(IR,IC))
      END IF
Crlm NORTH RIVER
      IF(IR.GE.18.AND.IR.LE.20.AND.IC.GE.48.AND.IC.LE.73) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.4*XAZ(IR,IC))
      END IF
Crlm EAST RIVER+LITTLE NECK BAY AREA
Crlm do what it takes thru Rikers Island
      IF(IR.GE.22.AND.IR.LE.29.AND.IC.GE.48.AND.IC.LE.59) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*5.0*XAZ(IR,IC))
      END IF
      IF(IR.GE.30.AND.IR.LE.36.AND.IC.GE.48.AND.IC.LE.59) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.2*XAZ(IR,IC))
      END IF
      IF(IR.GE.37.AND.IR.LE.39.AND.IC.GE.53.AND.IC.LE.64) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.2*XAZ(IR,IC))
      END IF
Crlm LONG ISLAND SOUND ( to mp 60 )excludes Little Neck bay area
      IF(IR.GE.36.AND.IR.LE.46.AND.IC.GE.45.AND.IC.LE.52) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.55*XAZ(IR,IC))
      END IF
      IF(IR.GE.40.AND.IR.LE.46.AND.IC.GE.53.AND.IC.LE.58) THEN
      VSSED = VSSED+((1.08**STP20(IR,IC,NLAYR))*0.55*XAZ(IR,IC))
      END IF
Crlm end benthic filtration change

      DEPFLUX(IR,IC,3) = VSSED*RPOP(IR,IC,NLAYR)
      CDRPOP(IR,IC,NLAYR) = CDRPOP(IR,IC,NLAYR) - DEPFLUX(IR,IC,3)
      DEPFLUX(IR,IC,4) = VSSED*LPOP(IR,IC,NLAYR)
      CDLPOP(IR,IC,NLAYR) = CDLPOP(IR,IC,NLAYR) - DEPFLUX(IR,IC,4)
      DEPFLUX(IR,IC,5) = VSSED*RPON(IR,IC,NLAYR)
      CDRPON(IR,IC,NLAYR) = CDRPON(IR,IC,NLAYR) - DEPFLUX(IR,IC,5)
      DEPFLUX(IR,IC,6) = VSSED*LPON(IR,IC,NLAYR)
      CDLPON(IR,IC,NLAYR) = CDLPON(IR,IC,NLAYR) - DEPFLUX(IR,IC,6)
      DEPFLUX(IR,IC,7) = VSSED*BSI(IR,IC,NLAYR)
      CDBSI(IR,IC,NLAYR) = CDBSI(IR,IC,NLAYR) - DEPFLUX(IR,IC,7)
      DEPFLUX(IR,IC,8) = VSSED*RPOC(IR,IC,NLAYR)
      CDRPOC(IR,IC,NLAYR) = CDRPOC(IR,IC,NLAYR) - DEPFLUX(IR,IC,8)
      DEPFLUX(IR,IC,9) = VSSED*LPOC(IR,IC,NLAYR)
      CDLPOC(IR,IC,NLAYR) = CDLPOC(IR,IC,NLAYR) - DEPFLUX(IR,IC,9)
      DEPFLUX(IR,IC,10) = VSSED*PO4TAM(IR,IC,NLAYR)
      CDPO4T(IR,IC,NLAYR) = CDPO4T(IR,IC,NLAYR) - DEPFLUX(IR,IC,10)
      DEPFLUX(IR,IC,11) = VSSED*SITAM(IR,IC,NLAYR)
      CDSIT(IR,IC,NLAYR) = CDSIT(IR,IC,NLAYR) - DEPFLUX(IR,IC,11)
  250 CONTINUE

C        CALL SEDMODL TO COMPUTE NUTRIENT FLUXES/SOD

      CALL SED_FLUX

C        NUTRIENT FLUXES AND SOD

c$doacross local(ic,ir,sa)
      DO 260 IC=1,NCOL
      DO 260 IR=1,NROW
      IF(FSM(IR,IC).LE.0)   GO TO 260

      SA = XAZ(IR,IC)
C        PO4
      CDPO4T(IR,IC,NLAYR) = CDPO4T(IR,IC,NLAYR) + JPO4(IR,IC)*SA
C        NH3-N
      CDNH3T(IR,IC,NLAYR) = CDNH3T(IR,IC,NLAYR) + JNH3(IR,IC)*SA
C        NO3-N
      CDNO23(IR,IC,NLAYR) = CDNO23(IR,IC,NLAYR) + JNO3(IR,IC)*SA
C        SILICA
      CDSIT(IR,IC,NLAYR) = CDSIT(IR,IC,NLAYR) + JSI(IR,IC)*SA
C        LEACHING O2*  (JHS OR JCH4AQ+JCH4G)
      CDO2EQ(IR,IC,NLAYR) = CDO2EQ(IR,IC,NLAYR) + (JHS(IR,IC)
     .     +JCH4AQ(IR,IC)+JCH4G(IR,IC))*SA
C        SOD
      CDDO(IR,IC,NLAYR) = CDDO(IR,IC,NLAYR) - SODS(IR,IC)*SA

  260 CONTINUE

C         CHECK IF TIME TO DUMP TO DISK
      IF(TIME.GE.303.9948.AND.TIME.LT.304.0052) THEN
       OPEN(UNIT=152,file='RCAFICAUG',form='UNFORMATTED')
       WRITE(152)   CARAY
       CLOSE(152)
      ENDIF
      IF(IDISK.EQ.0)   RETURN

      PTIME = TIME+0.0005*TIME

C        GLOBAL DUMPS

      IF(IDISK.EQ.1 .OR. IDISK.EQ.3)  THEN
       IF(IGDOPT.EQ.0)  THEN
         WRITE(10)  PTIME
         WRITE(11)   SAL
         WRITE(11)   PHYT1
         WRITE(11)   PHYT2
c$doacross local(il,ic,ir)
         DO 300 IL=1,NLAYR
          DO 300 IC=1,NCOL
           DO 300 IR=1,NROW
             TPOP(IR,IC,IL) = PCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .                      + PCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)
     .                      + RPOP(IR,IC,IL) + LPOP(IR,IC,IL)
  300    CONTINUE
         WRITE(11)   TPOP
c$doacross local(il,ic,ir)
         DO 305 IL=1,NLAYR
          DO 305 IC=1,NCOL
           DO 305 IR=1,NROW
             TDOP(IR,IC,IL) = RDOP(IR,IC,IL) + LDOP(IR,IC,IL)
  305    CONTINUE
         WRITE(11)   TDOP
         WRITE(11)   DPO4
c$doacross local(il,ic,ir)
         DO 310 IL=1,NLAYR
          DO 310 IC=1,NCOL
           DO 310 IR=1,NROW
             TPON(IR,IC,IL) = NCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .                      + NCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)
     .                      + RPON(IR,IC,IL) + LPON(IR,IC,IL)
  310    CONTINUE
         WRITE(11)   TPON
c$doacross local(il,ic,ir)
         DO 315 IL=1,NLAYR
          DO 315 IC=1,NCOL
           DO 315 IR=1,NROW
             TDON(IR,IC,IL) = RDON(IR,IC,IL) + LDON(IR,IC,IL)
  315    CONTINUE
         WRITE(11)   TDON
         WRITE(11)   DNH3
         WRITE(11)   NO23
c$doacross local(il,ic,ir)
         DO 320 IL=1,NLAYR
          DO 320 IC=1,NCOL
           DO 320 IR=1,NROW
             TPSI(IR,IC,IL) = SCRB1(IR,IC,IL)*PHYT1(IR,IC,IL)
     .                      + SCRB2(IR,IC,IL)*PHYT2(IR,IC,IL)
     .                      + BSI(IR,IC,IL)
  320    CONTINUE
         WRITE(11)   TPSI
         WRITE(11)   DSI
c$doacross local(il,ic,ir)
         DO 325 IL=1,NLAYR
          DO 325 IC=1,NCOL
           DO 325 IR=1,NROW
             TPOC(IR,IC,IL) = PHYT1(IR,IC,IL) + PHYT2(IR,IC,IL)
     .                      + RPOC(IR,IC,IL) + LPOC(IR,IC,IL)
  325    CONTINUE
         WRITE(11)   TPOC
c$doacross local(il,ic,ir)
         DO 330 IL=1,NLAYR
          DO 330 IC=1,NCOL
           DO 330 IR=1,NROW
             TDOC(IR,IC,IL) = RDOC(IR,IC,IL) + LDOC(IR,IC,IL)
     .                      + REDOC(IR,IC,IL) + EXDOC(IR,IC,IL)
  330    CONTINUE
         WRITE(11)   TDOC
         WRITE(11)   O2EQ
c$doacross local(il,ic,ir) , share(doaveg)
         DO 335 IL=1,NLAYR
          DO 335 IC=1,NCOL
           DO 335 IR=1,NROW
             DOAVEG(IR,IC,IL) = DOAVEG(IR,IC,IL)/FLOAT(IAVGGDCNTR)
  335    CONTINUE
         WRITE(11)   DOAVEG
         WRITE(11)   DOMING
         WRITE(11)   DOMAXG
c        WRITE(11)   TAM
C        WRITE(11)   RCATEMP
         WRITE(11)   BOTBODHS
c$doacross local(il,ic,ir) , share(chlaveg)
         DO 340 IL=1,NLAYR
          DO 340 IC=1,NCOL
           DO 340 IR=1,NROW
             CHLAVEG(IR,IC,IL) = CHLAVEG(IR,IC,IL)/FLOAT(IAVGGDCNTR)
  340    CONTINUE
         WRITE(11)   CHLAVEG
         WRITE(11)   CHLMING
         WRITE(11)   CHLMAXG
C        NET PRIMARY PRODUCTIVITY TO APPROXIMATELY THE
C        1 PERCENT LIGHT LEVEL  -  MG C/M**2-DAY
         DO 345 IL=1,NLAYR
          DO 345 IC=1,NCOL
           DO 345 IR=1,NROW
             DEPTH = (ZZ(IL)+ZZ(IL+1))/2. * HBAR(IR,IC)
             IF(EXP(-SKE(IR,IC,IL)*DEPTH).GT.0.01) THEN
               TGPDP(IR,IC,IL) = DZ(IL)*1000.*DEPTH*
     .                (GPP1(IR,IC,IL) + GPP2(IR,IC,IL)
     .               - DPP1(IR,IC,IL) - DPP2(IR,IC,IL))
             ELSE
               TGPDP(IR,IC,IL) = 0.0
             ENDIF
  345    CONTINUE
         WRITE(11)   HYDSAL
C        WRITE(11)   TGPDP
         DO 350 IL=1,NLAYR
          DO 350 IC=1,NCOL
           DO 350 IR=1,NROW
             TRESP(IR,IC,IL) = OCRB*RESP(IR,IC,IL) + SK24OC(IR,IC,IL)
     .                       + SK230(IR,IC,IL) + SK2413(IR,IC,IL)
  350    CONTINUE
         WRITE(11)   HYDTEMP
C        WRITE(11)   TRESP
         WRITE(11)   SKE
       ELSE
c$doacross local(il,ic,ir) , share(sal_gda,ecmsal_gda),
c$& share(phyt1_gda,phyt2_gda),
c$& share(tpop_gda,tdop_gda,dpo4_gda),
c$& share(tpon_gda,tdon_gda,dnh3_gda,no23_gda),
c$& share(tpsi_gda,dsi_gda,tpoc_gda,tdoc_gda),
c$& share(o2eq_gda,doaveg,rcatmp_gda,ecmtmp_gda,chlaveg),
c$& share(tgpdp_gda,tresp_gda,ske_gda,botbodhs_gda)
         DO 360 IL=1,NLAYR
          DO 360 IC=1,NCOL
           DO 360 IR=1,NROW
            SAL_GDA(IR,IC,IL) = SAL_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            ECMSAL_GDA(IR,IC,IL)=ECMSAL_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            PHYT1_GDA(IR,IC,IL) = PHYT1_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            PHYT2_GDA(IR,IC,IL) = PHYT2_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            TPOP_GDA(IR,IC,IL) = TPOP_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            TDOP_GDA(IR,IC,IL) = TDOP_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            DPO4_GDA(IR,IC,IL) = DPO4_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            TPON_GDA(IR,IC,IL) = TPON_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            TDON_GDA(IR,IC,IL) = TDON_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            DNH3_GDA(IR,IC,IL) = DNH3_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            NO23_GDA(IR,IC,IL) = NO23_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            TPSI_GDA(IR,IC,IL) = TPSI_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            DSI_GDA(IR,IC,IL) = DSI_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            TPOC_GDA(IR,IC,IL) = TPOC_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            TDOC_GDA(IR,IC,IL) = TDOC_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            O2EQ_GDA(IR,IC,IL) = O2EQ_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            DOAVEG(IR,IC,IL) = DOAVEG(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            RCATMP_GDA(IR,IC,IL)=RCATMP_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            ECMTMP_GDA(IR,IC,IL)=ECMTMP_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            CHLAVEG(IR,IC,IL) = CHLAVEG(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            TGPDP_GDA(IR,IC,IL) = TGPDP_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
            TRESP_GDA(IR,IC,IL) = TRESP_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
      BOTBODHS_GDA(IR,IC,IL) = BOTBODHS_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)    
            SKE_GDA(IR,IC,IL) = SKE_GDA(IR,IC,IL)/FLOAT(IAVGGDCNTR)
  360    CONTINUE
         WRITE(10)  PTIME - PRNTG/2.
         WRITE(11)   SAL_GDA
         WRITE(11)   PHYT1_GDA
         WRITE(11)   PHYT2_GDA
         WRITE(11)   TPOP_GDA
         WRITE(11)   TDOP_GDA
         WRITE(11)   DPO4_GDA
         WRITE(11)   TPON_GDA
         WRITE(11)   TDON_GDA
         WRITE(11)   DNH3_GDA
         WRITE(11)   NO23_GDA
         WRITE(11)   TPSI_GDA
         WRITE(11)   DSI_GDA
         WRITE(11)   TPOC_GDA
         WRITE(11)   TDOC_GDA
C15      WRITE(11)   O2EQ_GDA
         WRITE(11)   TGPDP_GDA
         WRITE(11)   DOAVEG
         WRITE(11)   DOMING
         WRITE(11)   DOMAXG
C19      WRITE(11)   RCATMP_GDA
C19      WRITE(11)   TRESP_GDA
         WRITE(11)   BOTBODHS_GDA
         WRITE(11)   CHLAVEG
         WRITE(11)   CHLMING
         WRITE(11)   CHLMAXG
         WRITE(11)   ECMSAL_GDA
         WRITE(11)   ECMTMP_GDA
         WRITE(11)   SKE_GDA
       ENDIF
         

c$doacross local(il,ic,ir) , share(chlaveg,chlmaxg,chlming),
c$& share(doaveg,doming,domaxg)
       DO 375 IL=1,NLAYR
        DO 375 IC=1,NCOL
         DO 375 IR=1,NROW
          CHLAVEG(IR,IC,IL) = 0.
          DOAVEG(IR,IC,IL) = 0.
          CHLMING(IR,IC,IL) = 1000.
          DOMING(IR,IC,IL) = 1000.
          CHLMAXG(IR,IC,IL) = -1000.
          DOMAXG(IR,IC,IL) = -1000.
  375  CONTINUE
       IF(IGDOPT.EQ.1)  THEN
c$doacross local(il,ic,ir) , share(sal_gda,ecmsal_gda),
c$& share(phyt1_gda,phyt2_gda),
c$& share(tpop_gda,tdop_gda,dpo4_gda),
c$& share(tpon_gda,tdon_gda,dnh3_gda,no23_gda),
c$& share(tpoc_gda,tdoc_gda,tpsi_gda,dsi_gda),
c$& share(o2eq_gda,rcatmp_gda,ecmtmp_gda,ske_gda),
c$& share(tgpdp_gda,tresp_gda,botbodhs_gda)
         DO 380 IL=1,NLAYR
          DO 380 IC=1,NCOL
           DO 380 IR=1,NROW
            SAL_GDA(IR,IC,IL) = 0.
            ECMSAL_GDA(IR,IC,IL) = 0.
            PHYT1_GDA(IR,IC,IL) = 0.
            PHYT2_GDA(IR,IC,IL) = 0.
            TPOP_GDA(IR,IC,IL) = 0.
            TDOP_GDA(IR,IC,IL) = 0.
            DPO4_GDA(IR,IC,IL) = 0.
            TPON_GDA(IR,IC,IL) = 0.
            TDON_GDA(IR,IC,IL) = 0.
            DNH3_GDA(IR,IC,IL) = 0.
            NO23_GDA(IR,IC,IL) = 0.
            TPSI_GDA(IR,IC,IL) = 0.
            DSI_GDA(IR,IC,IL) = 0.
            TPOC_GDA(IR,IC,IL) = 0.
            TDOC_GDA(IR,IC,IL) = 0.
            O2EQ_GDA(IR,IC,IL) = 0.
            RCATMP_GDA(IR,IC,IL) = 0.
            ECMTMP_GDA(IR,IC,IL) = 0.
            TGPDP_GDA(IR,IC,IL) = 0.
            TRESP_GDA(IR,IC,IL) = 0.
            BOTBODHS_GDA(IR,IC,IL) = 0.
            SKE_GDA(IR,IC,IL) = 0.
  380    CONTINUE
       ENDIF

       IAVGGDCNTR = 0

       ENDIF

C        INITIAL CONDITION FILE
      REWIND 15
      WRITE(15)  CARAY

9989  FORMAT(2I5,20E10.4)
      REWIND 60
      REWIND 61
      REWIND 62
      REWIND 63
      REWIND 64
      REWIND 65
      REWIND 66
      REWIND 67
      REWIND 68
      REWIND 69
      REWIND 70
      REWIND 71
      REWIND 72
      DO IC=1,NCOL
        DO IR = 1,NROW
        IF(FSM(IR,IC).LE.0) GO TO 875
          WRITE(60,9989) IR,IC,(HALF(IR,IC,IL),IL=1,10)
     .,(HALFday(IR,IC,IL),IL=1,10)
          WRITE(61,9989) IR,IC,(ONE(IR,IC,IL),IL=1,10)
     .,(ONEday(IR,IC,IL),IL=1,10)
          WRITE(62,9989) IR,IC,(ONEHALF(IR,IC,IL),IL=1,10)
     .,(ONEHALFday(IR,IC,IL),IL=1,10)
          WRITE(63,9989) IR,IC,(TWO(IR,IC,IL),IL=1,10)
     .,(TWOday(IR,IC,IL),IL=1,10)
          WRITE(64,9989) IR,IC,(TWOHALF(IR,IC,IL),IL=1,10)
     .,(TWOHALFday(IR,IC,IL),IL=1,10)
          WRITE(65,9989) IR,IC,(THREE(IR,IC,IL),IL=1,10)
     .,(THREEday(IR,IC,IL),IL=1,10)
          WRITE(66,9989) IR,IC,(THREEHALF(IR,IC,IL),IL=1,10)
     .,(THREEHALFday(IR,IC,IL),IL=1,10)
          WRITE(67,9989) IR,IC,(FOUR(IR,IC,IL),IL=1,10)
     .,(FOURday(IR,IC,IL),IL=1,10)
          WRITE(68,9989) IR,IC,(FOURHALF(IR,IC,IL),IL=1,10)
     .,(FOURHALFday(IR,IC,IL),IL=1,10)
          WRITE(69,9989) IR,IC,(FIVE(IR,IC,IL),IL=1,10)
     .,(FIVEday(IR,IC,IL),IL=1,10)
          WRITE(70,9989) IR,IC,(FIVEHALF(IR,IC,IL),IL=1,10)
     .,(FIVEHALFday(IR,IC,IL),IL=1,10)
          WRITE(71,9989) IR,IC,(SIX(IR,IC,IL),IL=1,10)
     .,(SIXday(IR,IC,IL),IL=1,10)
          WRITE(72,9989) IR,IC,(GTESIX(IR,IC,IL),IL=1,10)
     .,(GTESIXday(IR,IC,IL),IL=1,10)
875     END DO
      END DO

      RETURN

  900 WRITE(OUT,9990)
 9990 FORMAT(///5X,'INPUT ERROR WHILE READING IGDOPT,IDDOPT'//)
      CALL EXIT
  999 STOp 'PROGRAM TERMINATED ABNORMALLY'

C       ERROR ON TEMPERATURE READ
  910 WRITE(OUT,9100)  IKE
 9100 FORMAT(' Error encountered in ke file no.',I3)
      CALL EXIT
      END
      SUBROUTINE BCREADER(BUFHEM,IBCRECL,TNEXT_T)
      REAL BUFHEM(IBCRECL)
      READ(52) BUFHEM
      READ(51) TNEXT_T
      RETURN
      END
      
